@@ -1,22 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2004 Janne Jalkanen (Janne.Jalkanen@iki.fi),
-                            Erik Bunn (ebu@memecry.net)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.    
  */
 package com.ecyrd.jspwiki;
 
@@ -121,15 +121,15 @@ public class ReferenceManager
      *  must contain Strings. The Collection may contain names of non-existing
      *  pages.
      */
-    private Map            m_refersTo;
-    private Map            m_unmutableRefersTo;
+    private Map<String,Collection<String>> m_refersTo;
+    private Map<String,Collection<String>> m_unmutableRefersTo;
 
     /** Maps page wikiname to a Set of referring pages. The Set must
      *  contain Strings. Non-existing pages (a reference exists, but not a file
      *  for the page contents) may have an empty Set in m_referredBy.
      */
-    private Map            m_referredBy;
-    private Map            m_unmutableReferredBy;
+    private Map<String,Set<String>> m_referredBy;
+    private Map<String,Set<String>> m_unmutableReferredBy;
 
     /** The WikiEngine that owns this object. */
     private WikiEngine     m_engine;
@@ -142,7 +142,7 @@ public class ReferenceManager
     private static final String SERIALIZATION_DIR  = "refmgr-attr";
 
     /** We use this also a generic serialization id */
-    private static final long serialVersionUID = 2L;
+    private static final long serialVersionUID = 4L;
 
     /**
      *  Builds a new ReferenceManager.
@@ -151,8 +151,8 @@ public class ReferenceManager
      */
     public ReferenceManager( WikiEngine engine )
     {
-        m_refersTo   = new HashMap();
-        m_referredBy = new HashMap();
+        m_refersTo   = new HashMap<String,Collection<String>>();
+        m_referredBy = new HashMap<String,Set<String>>();
         m_engine = engine;
 
         m_matchEnglishPlurals = TextUtil.getBooleanProperty( engine.getWikiProperties(),
@@ -169,14 +169,15 @@ public class ReferenceManager
     /**
      *  Does a full reference update.  Does not sync; assumes that you do it afterwards.
      */
+    @SuppressWarnings("unchecked")
     private void updatePageReferences( WikiPage page )
         throws ProviderException
     {
         String content = m_engine.getPageManager().getPageText( page.getName(),
                                                                 WikiPageProvider.LATEST_VERSION );
 
-        TreeSet res = new TreeSet();
-        Collection links = m_engine.scanWikiLinks( page, content );
+        TreeSet<String> res = new TreeSet<String>();
+        Collection<String> links = m_engine.scanWikiLinks( page, content );
 
         res.addAll( links );
         Collection attachments = m_engine.getAttachmentManager().listAttachments( page );
@@ -196,6 +197,7 @@ public class ReferenceManager
      *  @param pages A collection of all pages you want to be included in the reference
      *               count.
      *  @since 2.2
+     *  @throws ProviderException If reading of pages fail.
      */
     public void initialize( Collection pages )
         throws ProviderException
@@ -302,6 +304,7 @@ public class ReferenceManager
      *  Reads the serialized data from the disk back to memory.
      *  Returns the date when the data was last written on disk
      */
+    @SuppressWarnings("unchecked")
     private synchronized long unserializeFromDisk()
         throws IOException,
                ClassNotFoundException
@@ -339,11 +342,7 @@ public class ReferenceManager
         }
         finally
         {
-            try
-            {
-                if( in != null ) in.close();
-            }
-            catch( IOException ex ) {}
+            if( in != null ) in.close();
         }
 
         return saved;
@@ -472,6 +471,7 @@ public class ReferenceManager
 
             sw.stop();
             log.debug("Read serialized data for "+name+" successfully in "+sw);
+            p.setHasMetadata();
         }
         catch( NoSuchAlgorithmException e )
         {
@@ -479,11 +479,7 @@ public class ReferenceManager
         }
         finally
         {
-            try
-            {
-                if( in != null ) in.close();
-            }
-            catch( IOException ex ) {}
+            if( in != null ) in.close();
         }
 
         return saved;
@@ -495,17 +491,11 @@ public class ReferenceManager
     private synchronized void serializeAttrsToDisk( WikiPage p )
     {
         ObjectOutputStream out = null;
+        StopWatch sw = new StopWatch();
+        sw.start();
 
         try
         {
-            // FIXME: There is a concurrency issue here...
-            Set entries = p.getAttributes().entrySet();
-
-            if( entries.size() == 0 ) return;
-
-            StopWatch sw = new StopWatch();
-            sw.start();
-
             File f = new File( m_engine.getWorkDir(), SERIALIZATION_DIR );
 
             if( !f.exists() ) f.mkdirs();
@@ -515,6 +505,18 @@ public class ReferenceManager
             //
             f = new File( f, getHashFileName(p.getName()) );
 
+            // FIXME: There is a concurrency issue here...
+            Set entries = p.getAttributes().entrySet();
+
+            if( entries.size() == 0 ) 
+            {
+                //  Nothing to serialize, therefore we will just simply remove the
+                //  serialization file so that the next time we boot, we don't
+                //  deserialize old data.
+                f.delete();
+                return;
+            }
+
             out = new ObjectOutputStream( new BufferedOutputStream(new FileOutputStream(f)) );
 
             out.writeLong( serialVersionUID );
@@ -536,9 +538,6 @@ public class ReferenceManager
 
             out.close();
 
-            sw.stop();
-
-            log.debug("serialization for "+p.getName()+" done - took "+sw);
         }
         catch( IOException e )
         {
@@ -554,10 +553,19 @@ public class ReferenceManager
         {
             log.fatal("No MD5 algorithm!?!");
         }
+        finally
+        {
+            sw.stop();
+
+            log.debug("serialization for "+p.getName()+" done - took "+sw);
+        }
     }
 
     /**
      *  After the page has been saved, updates the reference lists.
+     *  
+     *  @param context {@inheritDoc}
+     *  @param content {@inheritDoc}
      */
     public void postSave( WikiContext context, String content )
     {
@@ -590,15 +598,15 @@ public class ReferenceManager
 
     private void pageRemoved(String pageName)
     {
-        Collection refTo = (Collection)m_refersTo.get( pageName );
+        Collection<String> refTo = m_refersTo.get( pageName );
 
         if( refTo != null )
         {
-            Iterator it_refTo = refTo.iterator();
-            while( it_refTo.hasNext() )
+            Iterator itRefTo = refTo.iterator();
+            while( itRefTo.hasNext() )
             {
-                String referredPageName = (String)it_refTo.next();
-                Set refBy = (Set)m_referredBy.get( referredPageName );
+                String referredPageName = (String)itRefTo.next();
+                Set<String> refBy = m_referredBy.get( referredPageName );
 
                 if( refBy == null )
                     throw new InternalWikiException("Refmgr out of sync: page "+pageName+" refers to "+referredPageName+", which has null referrers.");
@@ -619,7 +627,7 @@ public class ReferenceManager
             m_refersTo.remove( pageName );
         }
 
-        Set refBy = (Set) m_referredBy.get( pageName );
+        Set<String> refBy = m_referredBy.get( pageName );
         if( refBy == null || refBy.isEmpty() )
         {
             m_referredBy.remove( pageName );
@@ -681,10 +689,10 @@ public class ReferenceManager
         //
         // Create a new entry in m_refersTo.
         //
-        Collection oldRefTo = (Collection)m_refersTo.get( page );
+        Collection oldRefTo = m_refersTo.get( page );
         m_refersTo.remove( page );
 
-        TreeSet cleanedRefs = new TreeSet();
+        TreeSet<String> cleanedRefs = new TreeSet<String>();
         for( Iterator i = references.iterator(); i.hasNext(); )
         {
             String ref = (String)i.next();
@@ -703,7 +711,7 @@ public class ReferenceManager
         //
         if( !m_referredBy.containsKey( page ) )
         {
-            m_referredBy.put( page, new TreeSet() );
+            m_referredBy.put( page, new TreeSet<String>() );
         }
 
         //
@@ -716,16 +724,18 @@ public class ReferenceManager
         //
         //  Notify all referred pages of their referinesshoodicity.
         //
-        Iterator it = cleanedRefs.iterator();
+        Iterator<String> it = cleanedRefs.iterator();
         while( it.hasNext() )
         {
-            String referredPageName = (String)it.next();
+            String referredPageName = it.next();
             updateReferredBy( getFinalPageName(referredPageName), page );
         }
     }
 
     /**
      * Returns the refers-to list. For debugging.
+     * 
+     * @return The refers-to list.
      */
     protected Map getRefersTo()
     {
@@ -734,6 +744,8 @@ public class ReferenceManager
 
     /**
      * Returns the referred-by list. For debugging.
+     * 
+     * @return Referred-by lists.
      */
     protected Map getReferredBy()
     {
@@ -746,7 +758,7 @@ public class ReferenceManager
      */
     private void cleanReferredBy( String referrer,
                                   Collection oldReferred,
-                                  Collection newReferred )
+                                  Collection<String> newReferred )
     {
         // Two ways to go about this. One is to look up all pages previously
         // referred by referrer and remove referrer from their lists, and let
@@ -763,7 +775,7 @@ public class ReferenceManager
         while( it.hasNext() )
         {
             String referredPage = (String)it.next();
-            Set oldRefBy = (Set)m_referredBy.get( referredPage );
+            Set oldRefBy = m_referredBy.get( referredPage );
             if( oldRefBy != null )
             {
                 oldRefBy.remove( referrer );
@@ -808,7 +820,7 @@ public class ReferenceManager
             {
                 WikiPage page = (WikiPage)it.next();
                 // We add a non-null entry to referredBy to indicate the referred page exists
-                m_referredBy.put( page.getName(), new TreeSet() );
+                m_referredBy.put( page.getName(), new TreeSet<String>() );
                 // Just add a key to refersTo; the keys need to be in sync with referredBy.
                 m_refersTo.put( page.getName(), null );
             }
@@ -831,11 +843,12 @@ public class ReferenceManager
     private void updateReferredBy( String page, String referrer )
     {
         // We're not really interested in first level self-references.
+        /*
         if( page.equals( referrer ) )
         {
             return;
         }
-
+        */
         // Neither are we interested if plural forms refer to each other.
         if( m_matchEnglishPlurals )
         {
@@ -847,14 +860,14 @@ public class ReferenceManager
             }
         }
 
-        Set referrers = (Set)m_referredBy.get( page );
+        Set<String> referrers = m_referredBy.get( page );
 
         // Even if 'page' has not been created yet, it can still be referenced.
         // This requires we don't use m_referredBy keys when looking up missing
         // pages, of course.
         if(referrers == null)
         {
-            referrers = new TreeSet();
+            referrers = new TreeSet<String>();
             m_referredBy.put( page, referrers );
         }
         referrers.add( referrer );
@@ -874,13 +887,13 @@ public class ReferenceManager
         //  Remove this item from the referredBy list of any page
         //  which this item refers to.
         //
-        Collection c = (Collection)m_refersTo.get( pagename );
+        Collection<String> c = m_refersTo.get( pagename );
 
         if( c != null )
         {
-            for( Iterator i = c.iterator(); i.hasNext(); )
+            for( String key : c )
             {
-                Collection dref = (Collection) m_referredBy.get( i.next() );
+                Collection<?> dref = m_referredBy.get( key );
 
                 dref.remove( pagename );
             }
@@ -897,19 +910,17 @@ public class ReferenceManager
     /**
      *  Finds all unreferenced pages. This requires a linear scan through
      *  m_referredBy to locate keys with null or empty values.
+     *  
+     *  @return The Collection of Strings
      */
     public synchronized Collection findUnreferenced()
     {
-        ArrayList unref = new ArrayList();
+        ArrayList<String> unref = new ArrayList<String>();
 
-        Set keys = m_referredBy.keySet();
-        Iterator it = keys.iterator();
-
-        while( it.hasNext() )
+        for( String key : m_referredBy.keySet() )
         {
-            String key = (String) it.next();
-            //Set refs = (Set) m_referredBy.get( key );
-            Set refs = getReferenceList( m_referredBy, key );
+            Set<?> refs = getReferenceList( m_referredBy, key );
+            
             if( refs == null || refs.isEmpty() )
             {
                 unref.add( key );
@@ -929,29 +940,24 @@ public class ReferenceManager
      * Returns a Collection containing Strings of unreferenced page names.
      * Each non-existant page name is shown only once - we don't return information
      * on who referred to it.
+     * 
+     * @return A Collection of Strings
      */
     public synchronized Collection findUncreated()
     {
-        TreeSet uncreated = new TreeSet();
+        TreeSet<String> uncreated = new TreeSet<String>();
 
         // Go through m_refersTo values and check that m_refersTo has the corresponding keys.
         // We want to reread the code to make sure our HashMaps are in sync...
 
-        Collection allReferences = m_refersTo.values();
-        Iterator it = allReferences.iterator();
+        Collection<Collection<String>> allReferences = m_refersTo.values();
 
-        while( it.hasNext() )
+        for( Collection<String> refs : allReferences )
         {
-            Collection refs = (Collection)it.next();
-
             if( refs != null )
             {
-                Iterator rit = refs.iterator();
-
-                while( rit.hasNext() )
+                for( String aReference : refs )
                 {
-                    String aReference = (String)rit.next();
-
                     if( m_engine.pageExists( aReference ) == false )
                     {
                         uncreated.add( aReference );
@@ -964,26 +970,32 @@ public class ReferenceManager
     }
 
     /**
-     *  Searches for the given page in the given Map.
+     *  Searches for the given page in the given Map, and returns
+     *  the set of references.  This method also takes care of English plural
+     *  matching.
+     *  
+     *  @param coll The Map to search in
+     *  @param pagename The name to find.
+     *  @return The references list.
      */
-    private Set getReferenceList( Map coll, String pagename )
+    private <T> Set<T> getReferenceList( Map<String,Set<T>> coll, String pagename )
     {
-        Set refs = (Set)coll.get( pagename );
+        Set<T> refs = coll.get( pagename );
 
         if( m_matchEnglishPlurals )
         {
             //
             //  We'll add also matches from the "other" page.
             //
-            Set refs2;
+            Set<T> refs2;
 
             if( pagename.endsWith("s") )
             {
-                refs2 = (Set)coll.get( pagename.substring(0,pagename.length()-1) );
+                refs2 = coll.get( pagename.substring(0,pagename.length()-1) );
             }
             else
             {
-                refs2 = (Set)coll.get( pagename+"s" );
+                refs2 = coll.get( pagename+"s" );
             }
 
             if( refs2 != null )
@@ -1010,7 +1022,7 @@ public class ReferenceManager
     // FIXME: Return a Set instead of a Collection.
     public synchronized Collection findReferrers( String pagename )
     {
-        Set refs = getReferenceList( m_referredBy, pagename );
+        Set<String> refs = getReferenceList( m_referredBy, pagename );
 
         if( refs == null || refs.isEmpty() )
         {
@@ -1039,7 +1051,7 @@ public class ReferenceManager
      */
     public Set findReferredBy( String pageName )
     {
-        return (Set)m_unmutableReferredBy.get( getFinalPageName(pageName) );
+        return m_unmutableReferredBy.get( getFinalPageName(pageName) );
     }
 
     /**
@@ -1062,7 +1074,7 @@ public class ReferenceManager
      */
     public Collection findRefersTo( String pageName )
     {
-        return (Collection)m_unmutableRefersTo.get( getFinalPageName(pageName) );
+        return m_unmutableRefersTo.get( getFinalPageName(pageName) );
     }
 
     /**
@@ -1076,12 +1088,12 @@ public class ReferenceManager
      *         ReferenceManager
      * @since 2.3.24
      */
-    /*
-       This method traps and retries if a concurrent
-       modifcaition occurs.
-       TODO: It is unnecessary to calculate the hashcode; it should be calculated only
-             when the hashmaps are changed.  This is slow.
-    */
+    //
+    //   This method traps and retries if a concurrent
+    //   modifcaition occurs.
+    //   TODO: It is unnecessary to calculate the hashcode; it should be calculated only
+    //         when the hashmaps are changed.  This is slow.
+    //
     public int deepHashCode()
     {
         boolean failed = true;
@@ -1117,7 +1129,7 @@ public class ReferenceManager
      */
     public Set findCreated()
     {
-        return new HashSet( m_refersTo.keySet() );
+        return new HashSet<String>( m_refersTo.keySet() );
     }
 
     private String getFinalPageName( String orig )
@@ -1138,6 +1150,9 @@ public class ReferenceManager
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void actionPerformed(WikiEvent event)
     {
         if( (event instanceof WikiPageEvent) && (event.getType() == WikiPageEvent.PAGE_DELETED) )
