@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.    
  */
 package com.ecyrd.jspwiki;
 
@@ -38,6 +39,7 @@ import com.ecyrd.jspwiki.auth.user.UserDatabase;
 import com.ecyrd.jspwiki.i18n.InternationalizationManager;
 import com.ecyrd.jspwiki.tags.WikiTagBase;
 import com.ecyrd.jspwiki.ui.*;
+import com.ecyrd.jspwiki.preferences.Preferences;
 
 /**
  *  <p>Provides state information throughout the processing of a page.  A
@@ -61,7 +63,6 @@ import com.ecyrd.jspwiki.ui.*;
  *
  *  @see com.ecyrd.jspwiki.plugin.Counter
  *
- *  @author Janne Jalkanen
  *  @author Andrew R. Jaquith
  */
 public class WikiContext
@@ -74,7 +75,7 @@ public class WikiContext
     private    WikiEngine m_engine;
     private    String     m_template = "default";
 
-    private    Map        m_variableMap = new HashMap();
+    private    HashMap<String,Object> m_variableMap = new HashMap<String,Object>();
 
     /**
      *  Stores the HttpServletRequest.  May be null, if the request did not
@@ -194,15 +195,6 @@ public class WikiContext
      * request, and determine if a WikiSession object is present. If not, a new
      * one is created.
      * </p>
-     * <p>
-     * After the WikiSession object is obtained, the current authentication
-     * status is checked. If not authenticated, or if the login status reported
-     * by the container has changed, the constructor attempts to log in the user
-     * with
-     * {@link com.ecyrd.jspwiki.auth.AuthenticationManager#login(HttpServletRequest)}.
-     * If an login process throws an exception, this method logs the error but
-     * does not re-throw it.
-     * </p>
      * @param engine The WikiEngine that is handling the request
      * @param request The HttpServletRequest that should be associated with this
      *            context. This parameter may be <code>null</code>.
@@ -250,35 +242,12 @@ public class WikiContext
             m_command = command.targetedCommand( m_page );
         }
 
-        // Log in the user if new session or the container status changed
-        boolean doLogin = (request != null) && m_session.isNew();
-
         // Debugging...
         if( log.isDebugEnabled() )
         {
             HttpSession session = ( request == null ) ? null : request.getSession( false );
             String sid = ( session == null ) ? "(null)" : session.getId();
             log.debug( "Creating WikiContext for session ID=" + sid + "; target=" + getName() );
-            log.debug( "Do we need to log the user in? " + doLogin );
-        }
-
-        if( doLogin || m_session.isContainerStatusChanged( request ) )
-        {
-            try
-            {
-                engine.getAuthenticationManager().login( request );
-            }
-            catch ( WikiSecurityException e )
-            {
-                // Login failed because config was screwy
-                log.error( "Could not log in user: " + e.getMessage() );
-            }
-        }
-
-        // Mark the session as "not new"
-        if( m_session.isNew() )
-        {
-            m_session.setNew( false );
         }
 
         // Figure out what template to use
@@ -684,6 +653,38 @@ public class WikiContext
         return null;
     }
 
+    /**
+     *  Creates a deep clone of the WikiContext.  This is useful when you want
+     *  to be sure that you don't accidentally mess with page attributes, etc.
+     *  
+     *  @since  2.8.0
+     *  @return A deep clone of the WikiContext.
+     */
+    public WikiContext deepClone()
+    {
+        try
+        {
+            // super.clone() must always be called to make sure that inherited objects
+            // get the right type
+            WikiContext copy = (WikiContext)super.clone();
+
+            //  No need to deep clone these
+            copy.m_engine  = m_engine;
+            copy.m_command = m_command; // Static structure
+
+            copy.m_template       = m_template;
+            copy.m_variableMap    = (HashMap<String,Object>)m_variableMap.clone();
+            copy.m_request        = m_request;
+            copy.m_session        = m_session;
+            copy.m_page           = (WikiPage)m_page.clone();
+            copy.m_realPage       = (WikiPage)m_realPage.clone();
+            return copy;
+        }
+        catch( CloneNotSupportedException e ){} // Never happens
+
+        return null;
+    }
+    
     /**
      *  Returns the WikiSession associated with the context.
      *  This method is guaranteed to always return a valid WikiSession.
@@ -966,11 +967,8 @@ public class WikiContext
     // FIXME: This method should really cache the ResourceBundles or something...
     public ResourceBundle getBundle( String bundle ) throws MissingResourceException
     {
-        Locale loc = null;
-
-        if( m_request != null )
-            loc = m_request.getLocale();
-
+        Locale loc = Preferences.getLocale( this );
+       
         ResourceBundle b = m_engine.getInternationalizationManager().getBundle(bundle, loc);
 
         return b;
@@ -985,9 +983,12 @@ public class WikiContext
      */
     public static Locale getLocale( WikiContext context )
     {
+        return Preferences.getLocale( context );
+/*
         HttpServletRequest request = context.getHttpRequest();
         return ( request != null )
                 ? request.getLocale() : Locale.getDefault();
+*/
     }
 
 }
