@@ -1,21 +1,22 @@
-/*
+/* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki;
 
@@ -44,6 +45,7 @@ import com.ecyrd.jspwiki.auth.UserManager;
 import com.ecyrd.jspwiki.auth.acl.AclManager;
 import com.ecyrd.jspwiki.auth.acl.DefaultAclManager;
 import com.ecyrd.jspwiki.auth.authorize.GroupManager;
+import com.ecyrd.jspwiki.content.PageRenamer;
 import com.ecyrd.jspwiki.diff.DifferenceManager;
 import com.ecyrd.jspwiki.event.WikiEngineEvent;
 import com.ecyrd.jspwiki.event.WikiEventListener;
@@ -85,8 +87,6 @@ import com.ecyrd.jspwiki.workflow.*;
  *  <p>
  *  There's basically only a single WikiEngine for each web application, and
  *  you should always get it using the WikiEngine.getInstance() method.
- *
- *  @author Janne Jalkanen
  */
 public class WikiEngine
 {
@@ -154,6 +154,9 @@ public class WikiEngine
     /** The name of the property containing the ACLManager implementing class.
      *  The value is {@value}. */
     public static final String PROP_ACL_MANAGER_IMPL = "jspwiki.aclManager";
+    
+    /** If this property is set to false, we don't allow the creation of empty pages */
+    public static final String PROP_ALLOW_CREATION_OF_EMPTY_PAGES = "jspwiki.allowCreationOfEmptyPages";
 
     /** Should the user info be saved with the page data as well? */
     private boolean          m_saveUserInfo = true;
@@ -259,6 +262,9 @@ public class WikiEngine
 
     private AdminBeanManager m_adminBeanManager;
 
+    /** Stores wikiengine attributes. */
+    private Map<String,Object> m_attributes = Collections.synchronizedMap(new HashMap<String,Object>());
+
     /**
      *  Gets a WikiEngine related to this servlet.  Since this method
      *  is only called from JSP pages (and JspInit()) to be specific,
@@ -374,22 +380,32 @@ public class WikiEngine
     protected WikiEngine( ServletContext context, String appid, Properties props )
         throws WikiException
     {
+        super();
         m_servletContext = context;
         m_appid          = appid;
 
+        // Stash the WikiEngine in the servlet context
+        if ( context != null )
+        {
+            context.setAttribute( ATTR_WIKIENGINE,  this );
+            m_rootPath = context.getRealPath("/");
+        }
+        
         try
         {
             //
             //  Note: May be null, if JSPWiki has been deployed in a WAR file.
             //
-            m_rootPath = context.getRealPath("/");
             initialize( props );
             log.info("Root path for this Wiki is: '"+m_rootPath+"'");
         }
         catch( Exception e )
         {
             String msg = Release.APPNAME+": Unable to load and setup properties from jspwiki.properties. "+e.getMessage();
-            context.log( msg );
+            if ( context != null )
+            {
+                context.log( msg );
+            }
             throw new WikiException( msg );
         }
     }
@@ -596,7 +612,14 @@ public class WikiEngine
             log.fatal( "JSPWiki could not start, URLConstructor cannot be accessed: ",e );
             throw new WikiException(e.getMessage());
         }
-
+        catch( Exception e )
+        {
+            // Final catch-all for everything
+            
+            log.fatal( "JSPWiki could not start, due to an unknown exception when starting.",e );
+            throw new WikiException("Failed to start; please check log files for better information.");
+        }
+        
         //
         //  Initialize the good-to-have-but-not-fatal modules.
         //
@@ -651,11 +674,12 @@ public class WikiEngine
      *
      *  @throws WikiException If the reference manager initialization fails.
      */
+    @SuppressWarnings("unchecked")
     public void initReferenceManager() throws WikiException
     {
         try
         {
-            ArrayList pages = new ArrayList();
+            ArrayList<WikiPage> pages = new ArrayList<WikiPage>();
             pages.addAll( m_pageManager.getAllPages() );
             pages.addAll( m_attachmentManager.getAllAttachments() );
 
@@ -730,6 +754,7 @@ public class WikiEngine
      *  Don't use.
      *  @since 1.8.0
      *  @deprecated
+     *  @return Something magical.
      */
     public String getPluginSearchPath()
     {
@@ -811,6 +836,9 @@ public class WikiEngine
      *  @see #getURL(String, String, String, boolean)
      *  @see WikiContext#getURL(String, String)
      *  @deprecated
+     *  
+     *  @param pageName The name of the page.
+     *  @return An URI.
      *
      *  @since 2.0.3
      */
@@ -828,6 +856,7 @@ public class WikiEngine
      *  @since 2.0.42.
      *  @param attName Attachment name
      *  @deprecated
+     *  @return An URI.
      */
     public String getAttachmentURL( String attName )
     {
@@ -845,6 +874,7 @@ public class WikiEngine
      */
     public String getURL( String context, String pageName, String params, boolean absolute )
     {
+        if( pageName == null ) pageName = getFrontPage();
         return m_urlConstructor.makeURL( context, pageName, absolute, params );
     }
 
@@ -886,6 +916,9 @@ public class WikiEngine
      *  Incidentally, this is almost the same as encodeName(), below.
      *  I am not yet entirely sure if it's safe to merge the code.
      *
+     *  @param request The servlet request
+     *  @param name    The parameter name to get.
+     *  @return The parameter value or null
      *  @since 1.5.3
      *  @deprecated JSPWiki now requires servlet API 2.3, which has a better
      *              way of dealing with this stuff.  This will be removed in
@@ -982,7 +1015,7 @@ public class WikiEngine
      */
     public Collection getAllInterWikiLinks()
     {
-        Vector v = new Vector();
+        Vector<String> v = new Vector<String>();
 
         for( Enumeration i = m_properties.propertyNames(); i.hasMoreElements(); )
         {
@@ -1039,7 +1072,7 @@ public class WikiEngine
     {
         String appName = TextUtil.getStringProperty(m_properties,PROP_APPNAME,Release.APPNAME);
 
-        return appName;
+        return MarkupParser.cleanLink( appName );
     }
 
     /**
@@ -1502,7 +1535,7 @@ public class WikiEngine
      *  @param pagedata The page contents
      *  @return a Collection of Strings
      */
-    protected Collection scanWikiLinks( WikiPage page, String pagedata )
+    public Collection scanWikiLinks( WikiPage page, String pagedata )
     {
         LinkCollector localCollector = new LinkCollector();
 
@@ -1678,6 +1711,15 @@ public class WikiEngine
             return;
         }
 
+        // Check if creation of empty pages is allowed; bail if not
+        boolean allowEmpty = TextUtil.getBooleanProperty( m_properties, 
+                                                          PROP_ALLOW_CREATION_OF_EMPTY_PAGES, 
+                                                          false );
+        if ( !allowEmpty && !pageExists( page ) && text.trim().equals( "" ) )  
+        {
+            return;
+        }
+        
         // Create approval workflow for page save; add the diffed, proposed
         // and old text versions as Facts for the approver (if approval is required)
         // If submitter is authenticated, any reject messages will appear in his/her workflow inbox.
@@ -1754,14 +1796,15 @@ public class WikiEngine
 
     // FIXME: Should really get a Date object and do proper comparisons.
     //        This is terribly wasteful.
+    @SuppressWarnings("unchecked")
     public Collection getRecentChanges()
     {
         try
         {
-            Collection pages = m_pageManager.getAllPages();
-            Collection  atts = m_attachmentManager.getAllAttachments();
+            Collection<WikiPage>   pages = m_pageManager.getAllPages();
+            Collection<Attachment>  atts = m_attachmentManager.getAllAttachments();
 
-            TreeSet sortedPages = new TreeSet( new PageTimeComparator() );
+            TreeSet<WikiPage> sortedPages = new TreeSet<WikiPage>( new PageTimeComparator() );
 
             sortedPages.addAll( pages );
             sortedPages.addAll( atts );
@@ -2186,6 +2229,9 @@ public class WikiEngine
     }
 
     /**
+     * Returns the RSSGenerator. If the property <code>jspwiki.rss.generate</code>
+     * has not been set to <code>true</code>, this method will return <code>null</code>,
+     * <em>and callers should check for this value.</em>
      * @since 2.1.165
      * @return the RSS generator
      */
@@ -2219,6 +2265,16 @@ public class WikiEngine
         return m_pageRenamer.renamePage(context, renameFrom, renameTo, changeReferrers);
     }
 
+    /**
+     *  Returns the PageRenamer employed by this WikiEngine.
+     *  @since 2.5.141
+     *  @return The current PageRenamer instance.
+     */
+    public PageRenamer getPageRenamer()
+    {
+        return m_pageRenamer;
+    }
+
     /**
      *  Returns the UserManager employed by this WikiEngine.
      *  @since 2.3
@@ -2340,8 +2396,6 @@ public class WikiEngine
         }
     }
 
-    private Map m_attributes = Collections.synchronizedMap(new HashMap());
-
     /**
      * Adds an attribute to the engine for the duration of this engine.  The
      * value is not persisted.
