@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2005 The JSPWiki Development Group
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki;
 
@@ -25,9 +26,6 @@ import java.security.PrivilegedAction;
 import java.util.*;
 
 import javax.security.auth.Subject;
-import javax.security.auth.callback.CallbackHandler;
-import javax.security.auth.login.LoginContext;
-import javax.security.auth.login.LoginException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
 
@@ -37,8 +35,6 @@ import com.ecyrd.jspwiki.auth.*;
 import com.ecyrd.jspwiki.auth.authorize.Group;
 import com.ecyrd.jspwiki.auth.authorize.GroupManager;
 import com.ecyrd.jspwiki.auth.authorize.Role;
-import com.ecyrd.jspwiki.auth.login.CookieAssertionLoginModule;
-import com.ecyrd.jspwiki.auth.login.PrincipalWrapper;
 import com.ecyrd.jspwiki.auth.user.UserDatabase;
 import com.ecyrd.jspwiki.auth.user.UserProfile;
 import com.ecyrd.jspwiki.event.WikiEvent;
@@ -110,23 +106,15 @@ public final class WikiSession implements WikiEventListener
 
     private static final String ALL                   = "*";
 
-    private static ThreadLocal  c_guestSession        = new ThreadLocal();
+    private static ThreadLocal<WikiSession> c_guestSession = new ThreadLocal<WikiSession>();
 
     private final Subject       m_subject             = new Subject();
 
-    private final Map           m_messages            = new HashMap();
-
-    private String              m_cachedCookieIdentity= null;
-
-    private String              m_cachedRemoteUser    = null;
-
-    private Principal           m_cachedUserPrincipal = null;
+    private final Map<String,Set<String>> m_messages  = new HashMap<String,Set<String>>();
 
     /** The WikiEngine that created this session. */
     private WikiEngine          m_engine              = null;
 
-    private boolean             m_isNew               = true;
-
     private String              m_status              = ANONYMOUS;
 
     private Principal           m_userPrincipal       = WikiPrincipal.GUEST;
@@ -144,10 +132,9 @@ public final class WikiSession implements WikiEventListener
      */
     protected final boolean isInGroup( Group group )
     {
-        Principal[] principals = getPrincipals();
-        for ( int i = 0; i < principals.length; i++ )
+        for ( Principal principal : getPrincipals() )
         {
-          if ( isAuthenticated() && group.isMember( principals[i] ) )
+          if ( isAuthenticated() && group.isMember( principal ) )
           {
               return true;
           }
@@ -155,25 +142,6 @@ public final class WikiSession implements WikiEventListener
         return false;
     }
 
-    /**
-     * Returns <code>true</code> if the wiki session is newly initialized.
-     *
-     * @return True, if this is a new session.
-     */
-    protected final boolean isNew()
-    {
-        return m_isNew;
-    }
-
-    /**
-     * Sets the status of this wiki session.
-     * @param isNew whether this session should be considered "new".
-     */
-    protected final void setNew( boolean isNew )
-    {
-        m_isNew = isNew;
-    }
-
     /**
      * Private constructor to prevent WikiSession from being instantiated
      * directly.
@@ -239,26 +207,12 @@ public final class WikiSession implements WikiEventListener
      */
     public final boolean isAnonymous()
     {
-        Set principals = m_subject.getPrincipals();
+        Set<Principal> principals = m_subject.getPrincipals();
         return principals.contains( Role.ANONYMOUS ) ||
                  principals.contains( WikiPrincipal.GUEST ) ||
                  isIPV4Address( getUserPrincipal().getName() );
     }
 
-    /**
-     * Creates and returns a new login context for this wiki session.
-     * This method is called by
-     * {@link com.ecyrd.jspwiki.auth.AuthenticationManager}.
-     * @param application the name of the application
-     * @param handler the callback handler
-     * @return A new login context
-     * @throws LoginException If the login context cannot be created
-     */
-    public final LoginContext getLoginContext( String application, CallbackHandler handler ) throws LoginException
-    {
-        return new LoginContext( application, m_subject, handler );
-    }
-
     /**
      * <p> Returns the Principal used to log in to an authenticated session. The
      * login principal is determined by examining the Subject's Principal set
@@ -273,10 +227,6 @@ public final class WikiSession implements WikiEventListener
      */
     public final Principal getLoginPrincipal()
     {
-        if ( m_loginPrincipal instanceof PrincipalWrapper )
-        {
-            return ((PrincipalWrapper)m_loginPrincipal).getPrincipal();
-        }
         return m_loginPrincipal;
     }
 
@@ -295,10 +245,6 @@ public final class WikiSession implements WikiEventListener
      */
     public final Principal getUserPrincipal()
     {
-        if ( m_loginPrincipal instanceof PrincipalWrapper )
-        {
-            return ((PrincipalWrapper)m_userPrincipal).getPrincipal();
-        }
         return m_userPrincipal;
     }
 
@@ -347,10 +293,10 @@ public final class WikiSession implements WikiEventListener
         {
             message = "";
         }
-        Set messages = (Set)m_messages.get( topic );
+        Set<String> messages = m_messages.get( topic );
         if (messages == null )
         {
-            messages = new LinkedHashSet();
+            messages = new LinkedHashSet<String>();
             m_messages.put( topic, messages );
         }
         messages.add( message );
@@ -370,7 +316,7 @@ public final class WikiSession implements WikiEventListener
      */
     public final void clearMessages( String topic )
     {
-        Set messages = (Set)m_messages.get( topic );
+        Set<String> messages = m_messages.get( topic );
         if ( messages != null )
         {
             m_messages.clear();
@@ -381,7 +327,7 @@ public final class WikiSession implements WikiEventListener
      * Returns all generic messages associated with this session.
      * The messages stored with the session persist throughout the
      * session unless they have been reset with {@link #clearMessages()}.
-     * @return the current messsages.
+     * @return the current messages.
      */
     public final String[] getMessages()
     {
@@ -392,17 +338,17 @@ public final class WikiSession implements WikiEventListener
      * Returns all messages associated with a session topic.
      * The messages stored with the session persist throughout the
      * session unless they have been reset with {@link #clearMessages(String)}.
-     * @return the current messsages.
+     * @return the current messages.
      * @param topic The topic
      */
     public final String[] getMessages( String topic )
     {
-        Set messages = (Set)m_messages.get( topic );
+        Set<String> messages = m_messages.get( topic );
         if ( messages == null || messages.size() == 0 )
         {
             return new String[0];
         }
-        return (String[])messages.toArray( new String[messages.size()] );
+        return messages.toArray( new String[messages.size()] );
     }
 
     /**
@@ -414,19 +360,18 @@ public final class WikiSession implements WikiEventListener
      */
     public final Principal[] getPrincipals()
     {
-        ArrayList principals = new ArrayList();
+        ArrayList<Principal> principals = new ArrayList<Principal>();
 
         // Take the first non Role as the main Principal
-        for( Iterator it = m_subject.getPrincipals().iterator(); it.hasNext(); )
+        for( Principal principal : m_subject.getPrincipals() )
         {
-            Principal principal = (Principal) it.next();
             if ( AuthenticationManager.isUserPrincipal( principal ) )
             {
                 principals.add( principal );
             }
         }
 
-        return (Principal[]) principals.toArray( new Principal[principals.size()] );
+        return principals.toArray( new Principal[principals.size()] );
     }
 
     /**
@@ -444,7 +389,7 @@ public final class WikiSession implements WikiEventListener
      */
     public final Principal[] getRoles()
     {
-        Set roles = new HashSet();
+        Set<Principal> roles = new HashSet<Principal>();
 
         // Add all of the Roles possessed by the Subject directly
         roles.addAll( m_subject.getPrincipals( Role.class ) );
@@ -453,7 +398,7 @@ public final class WikiSession implements WikiEventListener
         roles.addAll( m_subject.getPrincipals( GroupPrincipal.class ) );
 
         // Return a defensive copy
-        Principal[] roleArray = ( Principal[] )roles.toArray( new Principal[roles.size()] );
+        Principal[] roleArray = roles.toArray( new Principal[roles.size()] );
         Arrays.sort( roleArray, WikiPrincipal.COMPARATOR );
         return roleArray;
     }
@@ -528,11 +473,36 @@ public final class WikiSession implements WikiEventListener
                         break;
                     }
                     case WikiSecurityEvent.LOGIN_INITIATED:
+                    {
+                        // Do nothing
+                    }
+                    case WikiSecurityEvent.PRINCIPAL_ADD:
+                    {
+                        WikiSession target = (WikiSession)e.getTarget();
+                        if ( this.equals( target ) && m_status == AUTHENTICATED )
+                        {
+                            Set<Principal> principals = m_subject.getPrincipals();
+                            principals.add( (Principal)e.getPrincipal());
+                        }
+                        break;
+                    }
+                    case WikiSecurityEvent.LOGIN_ANONYMOUS:
                     {
                         WikiSession target = (WikiSession)e.getTarget();
                         if ( this.equals( target ) )
                         {
-                            updatePrincipals();
+                            m_status = ANONYMOUS;
+                            
+                            // Set the login/user principals and login status
+                            Set<Principal> principals = m_subject.getPrincipals();
+                            m_loginPrincipal = (Principal)e.getPrincipal();
+                            m_userPrincipal = m_loginPrincipal;
+                            
+                            // Add the login principal to the Subject, and set the built-in roles
+                            principals.clear();
+                            principals.add( m_loginPrincipal );
+                            principals.add( Role.ALL );
+                            principals.add( Role.ANONYMOUS );
                         }
                         break;
                     }
@@ -542,6 +512,17 @@ public final class WikiSession implements WikiEventListener
                         if ( this.equals( target ) )
                         {
                             m_status = ASSERTED;
+                            
+                            // Set the login/user principals and login status
+                            Set<Principal> principals = m_subject.getPrincipals();
+                            m_loginPrincipal = (Principal)e.getPrincipal();
+                            m_userPrincipal = m_loginPrincipal;
+                            
+                            // Add the login principal to the Subject, and set the built-in roles
+                            principals.clear();
+                            principals.add( m_loginPrincipal );
+                            principals.add( Role.ALL );
+                            principals.add( Role.ASSERTED );
                         }
                         break;
                     }
@@ -551,9 +532,21 @@ public final class WikiSession implements WikiEventListener
                         if ( this.equals( target ) )
                         {
                             m_status = AUTHENTICATED;
+                            
+                            // Set the login/user principals and login status
+                            Set<Principal> principals = m_subject.getPrincipals();
+                            m_loginPrincipal = (Principal)e.getPrincipal();
+                            m_userPrincipal = m_loginPrincipal;
+                            
+                            // Add the login principal to the Subject, and set the built-in roles
+                            principals.clear();
+                            principals.add( m_loginPrincipal );
+                            principals.add( Role.ALL );
+                            principals.add( Role.AUTHENTICATED );
+                            
+                            // Add the user and group principals
                             injectUserProfilePrincipals();  // Add principals for the user profile
-                            injectRolePrincipals();  // Inject role and group principals
-                            updatePrincipals();
+                            injectGroupPrincipals();  // Inject group principals
                         }
                         break;
                     }
@@ -563,7 +556,7 @@ public final class WikiSession implements WikiEventListener
                         if ( this.equals( source ) )
                         {
                             injectUserProfilePrincipals();  // Add principals for the user profile
-                            updatePrincipals();
+                            injectGroupPrincipals();  // Inject group principals
                         }
                         break;
                     }
@@ -571,7 +564,7 @@ public final class WikiSession implements WikiEventListener
                     {
                         // Refresh user principals based on new user profile
                         WikiSession source = (WikiSession)e.getSource();
-                        if ( this.equals( source ) )
+                        if ( this.equals( source ) && m_status == AUTHENTICATED )
                         {
                             // To prepare for refresh, set the new full name as the primary principal
                             UserProfile[] profiles = (UserProfile[])e.getTarget();
@@ -580,11 +573,19 @@ public final class WikiSession implements WikiEventListener
                             {
                                 throw new IllegalStateException( "User profile FullName cannot be null." );
                             }
-                            m_userPrincipal = new WikiPrincipal( newProfile.getFullname() );
-
-                            // Refresh principals for the user profile
-                            injectUserProfilePrincipals();
-                            updatePrincipals();
+                            
+                            Set<Principal> principals = m_subject.getPrincipals();
+                            m_loginPrincipal = new WikiPrincipal( newProfile.getLoginName() );
+                            
+                            // Add the login principal to the Subject, and set the built-in roles
+                            principals.clear();
+                            principals.add( m_loginPrincipal );
+                            principals.add( Role.ALL );
+                            principals.add( Role.AUTHENTICATED );
+                            
+                            // Add the user and group principals
+                            injectUserProfilePrincipals();  // Add principals for the user profile
+                            injectGroupPrincipals();  // Inject group principals
                         }
                         break;
                     }
@@ -609,116 +610,37 @@ public final class WikiSession implements WikiEventListener
         m_subject.getPrincipals().add( WikiPrincipal.GUEST );
         m_subject.getPrincipals().add( Role.ANONYMOUS );
         m_subject.getPrincipals().add( Role.ALL );
-        m_cachedCookieIdentity = null;
-        m_cachedRemoteUser = null;
-        m_cachedUserPrincipal = null;
+        m_userPrincipal = WikiPrincipal.GUEST;
+        m_loginPrincipal = WikiPrincipal.GUEST;
     }
 
     /**
-     * Returns whether the HTTP servlet container's authentication status has
-     * changed. Used to detect whether the container has logged in a user since
-     * the last call to this function. This method is stateful. After calling
-     * this function, the cached values are set to those in the current request.
-     * If the servlet request is <code>null</code>, this method always returns
-     * <code>false</code>. Note that once a user authenticates, the container
-     * status for the session will <em>never</em> change again, unless the
-     * session is invalidated by timeout or logout.
-     * @param request the servlet request
-     * @return <code>true</code> if the status has changed, <code>false</code>
-     * otherwise
+     * Injects GroupPrincipal objects into the user's Principal set based on the
+     * groups the user belongs to. For Groups, the algorithm first calls the
+     * {@link GroupManager#getRoles()} to obtain the array of GroupPrincipals
+     * the authorizer knows about. Then, the method
+     * {@link GroupManager#isUserInRole(WikiSession, Principal)} is called for
+     * each Principal. If the user is a member of the group, an equivalent
+     * GroupPrincipal is injected into the user's principal set. Existing
+     * GroupPrincipals are flushed and replaced. This method should generally be
+     * called after a user's {@link com.ecyrd.jspwiki.auth.user.UserProfile} is
+     * saved. If the wiki session is null, or there is no matching user profile,
+     * the method returns silently.
      */
-    protected final boolean isContainerStatusChanged( HttpServletRequest request )
-    {
-        if ( request == null || m_status.equals( AUTHENTICATED ) )
-        {
-            return false;
-        }
-
-        String remoteUser = request.getRemoteUser();
-        Principal userPrincipal = request.getUserPrincipal();
-        String cookieIdentity = CookieAssertionLoginModule.getUserCookie( request );
-        boolean changed = false;
-
-        // If request contains non-null remote user, update cached value if changed
-        if ( remoteUser != null && !remoteUser.equals( m_cachedRemoteUser) )
-        {
-            m_cachedRemoteUser = remoteUser;
-            log.info( "Remote user changed to " + remoteUser );
-            changed = true;
-        }
-
-        // If request contains non-null user principal, updated cached value if changed
-        if ( userPrincipal != null && !userPrincipal.equals( m_cachedUserPrincipal ) )
-        {
-            m_cachedUserPrincipal = userPrincipal;
-            log.info( "User principal changed to " + userPrincipal.getName() );
-            changed = true;
-        }
-
-        // If cookie identity changed (to a different value or back to null), update cache
-        if ( ( cookieIdentity != null && !cookieIdentity.equals( m_cachedCookieIdentity ) )
-               || ( cookieIdentity == null && m_cachedCookieIdentity != null ) )
-        {
-            m_cachedCookieIdentity = cookieIdentity;
-            log.info( "Cookie changed to " + cookieIdentity );
-            changed = true;
-        }
-        return changed;
-    }
-
-    /**
-     * Injects GroupPrincipal and Role objects into the user's Principal set
-     * based on the groups and roles the user belongs to.
-     * For Roles, the algorithm first calls the
-     * {@link Authorizer#getRoles()} to obtain the array of
-     * Principals the authorizer knows about. Then, the method
-     * {@link Authorizer#isUserInRole(WikiSession, Principal)} is
-     * called for each Principal. If the user possesses the role,
-     * an equivalent role Principal is injected into the user's
-     * principal set.
-     * Reloads user Principals into the suppplied WikiSession's Subject.
-     * Existing Role principals are preserved; all other Principal
-     * types are flushed and replaced by those returned by
-     * {@link com.ecyrd.jspwiki.auth.user.UserDatabase#getPrincipals(String)}.
-     * This method should generally be called after a user's {@link com.ecyrd.jspwiki.auth.user.UserProfile}
-     * is saved. If the wiki session is null, or there is no matching user profile, the
-     * method returns silently.
-     */
-    protected final void injectRolePrincipals()
+    protected final void injectGroupPrincipals()
     {
+        // Flush the existing GroupPrincipals
+        m_subject.getPrincipals().removeAll( m_subject.getPrincipals(GroupPrincipal.class) );
+        
         // Get the GroupManager and test for each Group
         GroupManager manager = m_engine.getGroupManager();
-        Principal[] groups = manager.getRoles();
-        for ( int i = 0; i < groups.length; i++ )
+        for ( Principal group : manager.getRoles() )
         {
-            if ( manager.isUserInRole( this, groups[i] ) )
+            if ( manager.isUserInRole( this, group ) )
             {
-                m_subject.getPrincipals().add( groups[i] );
+                m_subject.getPrincipals().add( group );
             }
         }
-
-        // Get the authorizer's known roles, then test for each
-        try
-        {
-            Authorizer authorizer = m_engine.getAuthorizationManager().getAuthorizer();
-            Principal[] roles = authorizer.getRoles();
-            for ( int i = 0; i < roles.length; i++ )
-            {
-                Principal role = roles[i];
-                if ( authorizer.isUserInRole( this, role ) )
-                {
-                    String roleName = role.getName();
-                    if ( !Role.isReservedName( roleName ) )
-                    {
-                        m_subject.getPrincipals().add( new Role( roleName ) );
-                    }
-                }
-            }
-        }
-        catch ( WikiException e )
-        {
-            log.error( "Could not refresh role principals: " + e.getMessage() );
-        }
     }
 
     /**
@@ -730,18 +652,8 @@ public final class WikiSession implements WikiEventListener
      */
     protected final void injectUserProfilePrincipals()
     {
-        // Copy all Role and GroupPrincipal principals into a temporary cache
-        Set oldPrincipals = m_subject.getPrincipals();
-        Set newPrincipals = new HashSet();
-        for (Iterator it = oldPrincipals.iterator(); it.hasNext();)
-        {
-            Principal principal = (Principal)it.next();
-            if ( AuthenticationManager.isRolePrincipal( principal ) )
-            {
-                newPrincipals.add( principal );
-            }
-        }
-        String searchId = getUserPrincipal().getName();
+        // Search for the user profile
+        String searchId = m_loginPrincipal.getName();
         if ( searchId == null )
         {
             // Oh dear, this wasn't an authenticated user after all
@@ -759,15 +671,22 @@ public final class WikiSession implements WikiEventListener
         {
             UserProfile profile = database.find( searchId );
             Principal[] principals = database.getPrincipals( profile.getLoginName() );
-            for (int i = 0; i < principals.length; i++)
+            for ( Principal principal : principals )
             {
-                Principal principal = principals[i];
-                newPrincipals.add( principal );
+                // Add the Principal to the Subject
+                m_subject.getPrincipals().add( principal );
+                
+                // Set the user principal if needed; we prefer FullName, but the WikiName will also work
+                boolean isFullNamePrincipal = ( principal instanceof WikiPrincipal && ((WikiPrincipal)principal).getType() == WikiPrincipal.FULL_NAME );
+                if ( isFullNamePrincipal )
+                {
+                   m_userPrincipal = principal; 
+                }
+                else if ( !( m_userPrincipal instanceof WikiPrincipal ) )
+                {
+                    m_userPrincipal = principal; 
+                }
             }
-
-            // Replace the Subject's old Principals with the new ones
-            oldPrincipals.clear();
-            oldPrincipals.addAll( newPrincipals );
         }
         catch ( NoSuchPrincipalException e )
         {
@@ -777,92 +696,6 @@ public final class WikiSession implements WikiEventListener
         }
     }
 
-    /**
-     * Updates the internally cached principals returned by {@link #getUserPrincipal()} and
-     * {@link #getLoginPrincipal()}. This method is called when the WikiSession receives
-     * the {@link com.ecyrd.jspwiki.event.WikiSecurityEvent#LOGIN_INITIATED} message.
-     */
-    protected final void updatePrincipals()
-    {
-        Set principals = m_subject.getPrincipals();
-        m_loginPrincipal = null;
-        m_userPrincipal = null;
-        Principal wikinamePrincipal = null;
-        Principal fullnamePrincipal = null;
-        Principal otherPrincipal = null;
-
-        for( Iterator it = principals.iterator(); it.hasNext(); )
-        {
-            Principal currentPrincipal = (Principal) it.next();
-            if ( !( currentPrincipal instanceof Role || currentPrincipal instanceof GroupPrincipal ) )
-            {
-                // For login principal, take the first PrincipalWrapper or WikiPrincipal of type LOGIN_NAME
-                // For user principal take the  first WikiPrincipal of type WIKI_NAME
-                if ( currentPrincipal instanceof PrincipalWrapper )
-                {
-                    m_loginPrincipal = currentPrincipal;
-                }
-                else if ( currentPrincipal instanceof WikiPrincipal )
-                {
-                    WikiPrincipal wp = (WikiPrincipal) currentPrincipal;
-                    if ( wp.getType().equals( WikiPrincipal.LOGIN_NAME ) )
-                    {
-                        m_loginPrincipal = wp;
-                    }
-                    else if ( wp.getType().equals( WikiPrincipal.WIKI_NAME ) )
-                    {
-                        wikinamePrincipal = currentPrincipal;
-                        m_userPrincipal = wp;
-                    }
-                    else if ( wp.getType().equals( WikiPrincipal.FULL_NAME ) )
-                    {
-                        fullnamePrincipal = currentPrincipal;
-                    }
-                    else
-                    {
-                        otherPrincipal = currentPrincipal;
-                    }
-                }
-                else if ( otherPrincipal == null )
-                {
-                    otherPrincipal = currentPrincipal;
-                }
-            }
-        }
-        if ( otherPrincipal == null )
-        {
-            otherPrincipal = WikiPrincipal.GUEST;
-        }
-
-        // If no login principal, use wiki name, full name or other principal (in that order)
-        if ( m_loginPrincipal == null )
-        {
-            m_loginPrincipal = wikinamePrincipal;
-            if ( m_loginPrincipal == null )
-            {
-                m_loginPrincipal = fullnamePrincipal;
-                if ( m_loginPrincipal == null )
-                {
-                    m_loginPrincipal = otherPrincipal;
-                }
-            }
-        }
-
-        // If no user principal, use wiki name, full name or login principal (in that order)
-        if ( m_userPrincipal == null )
-        {
-            m_userPrincipal = wikinamePrincipal;
-            if ( m_userPrincipal == null )
-            {
-                m_userPrincipal = fullnamePrincipal;
-                if ( m_userPrincipal == null )
-                {
-                    m_userPrincipal = m_loginPrincipal;
-                }
-            }
-        }
-    }
-
     /**
      * <p>Returns the status of the wiki session as a text string. Valid values are:</p>
      * <ul>
@@ -957,7 +790,7 @@ public final class WikiSession implements WikiEventListener
 
     private static WikiSession staticGuestSession( WikiEngine engine )
     {
-        WikiSession session = (WikiSession) c_guestSession.get();
+        WikiSession session = c_guestSession.get();
 
         if( session == null )
         {
@@ -1009,7 +842,7 @@ public final class WikiSession implements WikiEventListener
      * @throws java.security.AccessControlException if the action is not permitted
      * by the security policy
      */
-    public static final Object doPrivileged( WikiSession session, PrivilegedAction action ) throws AccessControlException
+    public static final Object doPrivileged( WikiSession session, PrivilegedAction<?> action ) throws AccessControlException
     {
         return Subject.doAsPrivileged( session.m_subject, action, null );
     }
