@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.    
  */
 package com.ecyrd.jspwiki.attachment;
 
@@ -25,6 +26,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.*;
 
+import org.apache.commons.lang.StringUtils;
 import org.apache.log4j.Logger;
 
 import com.ecyrd.jspwiki.*;
@@ -43,7 +45,6 @@ import com.opensymphony.oscache.base.NeedsRefreshException;
  *  that is in use.  It is created by the WikiEngine as a singleton object, and
  *  can be requested through the WikiEngine.
  *
- *  @author Janne Jalkanen
  *  @since 1.9.28
  */
 public class AttachmentManager
@@ -121,8 +122,8 @@ public class AttachmentManager
         //
         try
         {
-            Class providerclass = ClassUtil.findClass( "com.ecyrd.jspwiki.providers",
-                                                       classname );
+            Class<?> providerclass = ClassUtil.findClass( "com.ecyrd.jspwiki.providers",
+                                                          classname );
 
             m_provider = (WikiAttachmentProvider)providerclass.newInstance();
 
@@ -307,6 +308,7 @@ public class AttachmentManager
      */
 
     // FIXME: This API should be changed to return a List.
+    @SuppressWarnings("unchecked")
     public Collection listAttachments( WikiPage wikipage )
         throws ProviderException
     {
@@ -479,6 +481,15 @@ public class AttachmentManager
             return;
         }
 
+        //
+        //  Checks if the actual, real page exists without any modifications
+        //  or aliases.  We cannot store an attachment to a non-existant page.
+        //
+        if( !m_engine.getPageManager().pageExists( att.getParentName() ) )
+        {
+            throw new ProviderException("Parent page does not exist");
+        }
+        
         m_provider.putAttachmentData( att, in );
 
         m_engine.getReferenceManager().updateReferences( att.getName(),
@@ -533,7 +544,7 @@ public class AttachmentManager
             return m_provider.listAllChanged( new Date(0L) );
         }
 
-        return new ArrayList();
+        return new ArrayList<Attachment>();
     }
 
     /**
@@ -578,4 +589,54 @@ public class AttachmentManager
         m_engine.getReferenceManager().clearPageEntries( att.getName() );
 
     }
+
+    /**
+     *  Validates the filename and makes sure it is legal.  It trims and splits
+     *  and replaces bad characters.
+     *  
+     *  @param filename
+     *  @return A validated name with annoying characters replaced.
+     *  @throws WikiException If the filename is not legal (e.g. empty)
+     */
+    static String validateFileName( String filename )
+        throws WikiException
+    {
+        if( filename == null || filename.trim().length() == 0 )
+        {
+            AttachmentServlet.log.error("Empty file name given.");
+    
+            throw new WikiException("Empty file name given.");
+        }
+    
+        //
+        //  Should help with IE 5.22 on OSX
+        //
+        filename = filename.trim();
+
+        // If file name ends with .jsp or .jspf, the user is being naughty!
+        if( filename.toLowerCase().endsWith( ".jsp" ) || filename.toLowerCase().endsWith(".jspf") )
+        {
+            log.info( "Attempt to upload a file with a .jsp/.jspf extension.  In certain cases this" +
+            		" can trigger unwanted security side effects, so we're preventing it." );
+            
+            throw new WikiException( "Unwanted file name." );
+        }
+    
+        //
+        //  Some browser send the full path info with the filename, so we need
+        //  to remove it here by simply splitting along slashes and then taking the path.
+        //
+        
+        String[] splitpath = filename.split( "[/\\\\]" );
+        filename = splitpath[splitpath.length-1];
+        
+        //
+        //  Remove any characters that might be a problem. Most
+        //  importantly - characters that might stop processing
+        //  of the URL.
+        //
+        filename = StringUtils.replaceChars( filename, "#?\"'", "____" );
+    
+        return filename;
+    }
 }
