@@ -1,31 +1,30 @@
-/*
-  JSPWiki - a JSP-based WikiWiki clone.
+/* 
+    JSPWiki - a JSP-based WikiWiki clone.
 
-  Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Lesser General Public License as published by
-  the Free Software Foundation; either version 2.1 of the License, or
-  (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
+ */
 package com.ecyrd.jspwiki.auth;
 
+import java.io.Serializable;
 import java.security.Permission;
 import java.security.Principal;
 import java.text.MessageFormat;
-import java.util.Map;
-import java.util.Properties;
-import java.util.ResourceBundle;
-import java.util.WeakHashMap;
+import java.util.*;
 
 import javax.mail.MessagingException;
 import javax.mail.internet.AddressException;
@@ -43,6 +42,8 @@ import com.ecyrd.jspwiki.auth.user.UserProfile;
 import com.ecyrd.jspwiki.event.WikiEventListener;
 import com.ecyrd.jspwiki.event.WikiEventManager;
 import com.ecyrd.jspwiki.event.WikiSecurityEvent;
+import com.ecyrd.jspwiki.filters.PageFilter;
+import com.ecyrd.jspwiki.filters.SpamFilter;
 import com.ecyrd.jspwiki.i18n.InternationalizationManager;
 import com.ecyrd.jspwiki.rpc.RPCCallable;
 import com.ecyrd.jspwiki.rpc.json.JSONRPCManager;
@@ -53,7 +54,6 @@ import com.ecyrd.jspwiki.workflow.*;
 
 /**
  * Provides a facade for obtaining user information.
- * @author Janne Jalkanen
  * @author Andrew Jaquith
  * @since 2.3
  */
@@ -84,8 +84,8 @@ public final class UserManager
 
     // private static final String  PROP_ACLMANAGER     = "jspwiki.aclManager";
 
-    /** Associateds wiki sessions with profiles */
-    private final Map        m_profiles     = new WeakHashMap();
+    /** Associates wiki sessions with profiles */
+    private final Map<WikiSession,UserProfile> m_profiles = new WeakHashMap<WikiSession,UserProfile>();
 
     /** The user database loads, manages and persists user identities */
     private UserDatabase     m_database;
@@ -104,6 +104,7 @@ public final class UserManager
      * @param engine the current wiki engine
      * @param props the wiki engine initialization properties
      */
+    @SuppressWarnings("deprecation")
     public final void initialize( WikiEngine engine, Properties props )
     {
         m_engine = engine;
@@ -114,7 +115,7 @@ public final class UserManager
         // TODO: it would be better if we did this in PageManager directly
         addWikiEventListener( engine.getPageManager() );
 
-        JSONRPCManager.registerGlobalObject( "users", new JSONUserModule(), new AllPermission(null) );
+        JSONRPCManager.registerGlobalObject( "users", new JSONUserModule(this), new AllPermission(null) );
     }
 
     /**
@@ -147,7 +148,7 @@ public final class UserManager
                                                           PROP_DATABASE );
 
             log.info("Attempting to load user database class "+dbClassName);
-            Class dbClass = ClassUtil.findClass( USERDATABASE_PACKAGE, dbClassName );
+            Class<?> dbClass = ClassUtil.findClass( USERDATABASE_PACKAGE, dbClassName );
             m_database = (UserDatabase) dbClass.newInstance();
             m_database.initialize( m_engine, m_engine.getWikiProperties() );
             log.info("UserDatabase initialized.");
@@ -206,7 +207,7 @@ public final class UserManager
     public final UserProfile getUserProfile( WikiSession session )
     {
         // Look up cached user profile
-        UserProfile profile = (UserProfile)m_profiles.get( session );
+        UserProfile profile = m_profiles.get( session );
         boolean newProfile = profile == null;
         Principal user = null;
 
@@ -216,7 +217,7 @@ public final class UserManager
             user = session.getUserPrincipal();
             try
             {
-                profile = m_database.find( user.getName() );
+                profile = getUserDatabase().find( user.getName() );
                 newProfile = false;
             }
             catch( NoSuchPrincipalException e )
@@ -226,7 +227,7 @@ public final class UserManager
 
         if ( newProfile )
         {
-            profile = m_database.newProfile();
+            profile = getUserDatabase().newProfile();
             if ( user != null )
             {
                 profile.setLoginName( user.getName() );
@@ -300,7 +301,7 @@ public final class UserManager
         UserProfile otherProfile;
         try
         {
-            otherProfile = m_database.findByLoginName( profile.getLoginName() );
+            otherProfile = getUserDatabase().findByLoginName( profile.getLoginName() );
             if ( otherProfile != null && !otherProfile.equals( oldProfile ) )
             {
                 throw new DuplicateUserException( "The login name '" + profile.getLoginName() + "' is already taken." );
@@ -311,7 +312,7 @@ public final class UserManager
         }
         try
         {
-            otherProfile = m_database.findByFullName( profile.getFullname() );
+            otherProfile = getUserDatabase().findByFullName( profile.getFullname() );
             if ( otherProfile != null && !otherProfile.equals( oldProfile ) )
             {
                 throw new DuplicateUserException( "The full name '" + profile.getFullname() + "' is already taken." );
@@ -383,11 +384,11 @@ public final class UserManager
             // If login name changed, rename it first
             if ( nameChanged && oldProfile != null && !oldProfile.getLoginName().equals( profile.getLoginName() ) )
             {
-                m_database.rename( oldProfile.getLoginName(), profile.getLoginName() );
+                getUserDatabase().rename( oldProfile.getLoginName(), profile.getLoginName() );
             }
 
             // Now, save the profile (userdatabase will take care of timestamps for us)
-            m_database.save( profile );
+            getUserDatabase().save( profile );
 
             if ( nameChanged )
             {
@@ -471,18 +472,35 @@ public final class UserManager
      * @param context the current wiki context
      * @param profile the supplied UserProfile
      */
+    @SuppressWarnings("unchecked")
     public final void validateProfile( WikiContext context, UserProfile profile )
     {
         boolean isNew = profile.isNew();
         WikiSession session = context.getWikiSession();
-        InputValidator validator = new InputValidator( SESSION_MESSAGES, session );
+        InputValidator validator = new InputValidator( SESSION_MESSAGES, context );
         ResourceBundle rb = context.getBundle( InternationalizationManager.CORE_BUNDLE );
 
+        //
+        //  Query the SpamFilter first
+        //
+        
+        List<PageFilter> ls = m_engine.getFilterManager().getFilterList();
+        for( PageFilter pf : ls )
+        {
+            if( pf instanceof SpamFilter )
+            {
+                if( ((SpamFilter)pf).isValidUserProfile( context, profile ) == false )
+                {
+                    session.addMessage( SESSION_MESSAGES, "Invalid userprofile" );
+                    return;
+                }
+                break;
+            }
+        }
+        
         // If container-managed auth and user not logged in, throw an error
-        // unless we're allowed to add profiles to the container
         if ( m_engine.getAuthenticationManager().isContainerAuthenticated()
-             && !context.getWikiSession().isAuthenticated()
-             && !m_database.isSharedWithContainer() )
+             && !context.getWikiSession().isAuthenticated() )
         {
             session.addMessage( SESSION_MESSAGES, rb.getString("security.error.createprofilebeforelogin") );
         }
@@ -520,7 +538,7 @@ public final class UserManager
         // It's illegal to use as a full name someone else's login name
         try
         {
-            otherProfile = m_database.find( fullName );
+            otherProfile = getUserDatabase().find( fullName );
             if ( otherProfile != null && !profile.equals( otherProfile ) && !fullName.equals( otherProfile.getFullname() ) )
             {
                 Object[] args = { fullName };
@@ -534,7 +552,7 @@ public final class UserManager
         // It's illegal to use as a login name someone else's full name
         try
         {
-            otherProfile = m_database.find( loginName );
+            otherProfile = getUserDatabase().find( loginName );
             if ( otherProfile != null && !profile.equals( otherProfile ) && !loginName.equals( otherProfile.getLoginName() ) )
             {
                 Object[] args = { loginName };
@@ -546,17 +564,22 @@ public final class UserManager
         { /* It's clean */ }
     }
 
+    /**
+     *  A helper method for returning all of the known WikiNames in this system.
+     *  
+     *  @return An Array of Principals
+     *  @throws WikiSecurityException If for reason the names cannot be fetched
+     */
     public Principal[] listWikiNames()
         throws WikiSecurityException
     {
-        return m_database.getWikiNames();
+        return getUserDatabase().getWikiNames();
     }
 
     /**
      * This is a database that gets used if nothing else is available. It does
      * nothing of note - it just mostly thorws NoSuchPrincipalExceptions if
      * someone tries to log in.
-     * @author Janne Jalkanen
      */
     public static class DummyUserDatabase extends AbstractUserDatabase
     {
@@ -565,6 +588,7 @@ public final class UserManager
          * No-op.
          * @throws WikiSecurityException never...
          */
+        @SuppressWarnings("deprecation")
         public void commit() throws WikiSecurityException
         {
             // No operation
@@ -613,6 +637,16 @@ public final class UserManager
             throw new NoSuchPrincipalException("No user profiles available");
         }
 
+        /**
+         * No-op; always throws <code>NoSuchPrincipalException</code>.
+         * @param uid the unique identifier to search for
+         * @return the user profile
+         * @throws NoSuchPrincipalException never...
+         */
+        public UserProfile findByUid( String uid ) throws NoSuchPrincipalException
+        {
+            throw new NoSuchPrincipalException("No user profiles available");
+        }
         /**
          * No-op; always throws <code>NoSuchPrincipalException</code>.
          * @param index the name to search for
@@ -644,15 +678,6 @@ public final class UserManager
         {
         }
 
-        /**
-         * No-op.
-         * @return <code>false</code>
-         */
-        public boolean isSharedWithContainer()
-        {
-            return false;
-        }
-
         /**
          * No-op; always throws <code>NoSuchPrincipalException</code>.
          * @param loginName the login name
@@ -688,6 +713,7 @@ public final class UserManager
      */
     public static class SaveUserProfileTask extends Task
     {
+        private static final long serialVersionUID = 6994297086560480285L;
         private final UserDatabase m_db;
         private final WikiEngine m_engine;
 
@@ -787,11 +813,25 @@ public final class UserManager
 
     /**
      *  Implements the JSON API for usermanager.
-     *
-     *  @author Janne Jalkanen
+     *  <p>
+     *  Even though this gets serialized whenever container shuts down/restarts,
+     *  this gets reinstalled to the session when JSPWiki starts.  This means
+     *  that it's not actually necessary to save anything.
      */
-    public final class JSONUserModule implements RPCCallable
+    public static final class JSONUserModule implements RPCCallable, Serializable
     {
+        private static final long serialVersionUID = 1L;
+        private volatile UserManager m_manager;
+        
+        /**
+         *  Create a new JSONUserModule.
+         *  @param mgr Manager
+         */
+        public JSONUserModule( UserManager mgr )
+        {
+            m_manager = mgr;
+        }
+        
         /**
          *  Directly returns the UserProfile object attached to an uid.
          *
@@ -802,12 +842,14 @@ public final class UserManager
         public UserProfile getUserInfo( String uid )
             throws NoSuchPrincipalException
         {
-            log.info("request "+uid);
-            UserProfile prof = m_database.find( uid );
+            if( m_manager != null )
+            {
+                UserProfile prof = m_manager.getUserDatabase().find( uid );
 
-            log.info("answer "+prof);
-
-            return prof;
+                return prof;
+            }
+            
+            throw new IllegalStateException("The manager is offline.");
         }
     }
-}
\ No newline at end of file
+}
