@@ -1,15 +1,22 @@
-/*
- * JSPWiki - a JSP-based WikiWiki clone. Copyright (C) 2001-2003 Janne Jalkanen
- * (Janne.Jalkanen@iki.fi) This program is free software; you can redistribute
- * it and/or modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of the
- * License, or (at your option) any later version. This program is distributed
- * in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
- * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU Lesser General Public License for more details. You should have
- * received a copy of the GNU Lesser General Public License along with this
- * program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA 02111-1307 USA
+/* 
+    JSPWiki - a JSP-based WikiWiki clone.
+
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.auth.authorize;
 
@@ -72,7 +79,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
     private GroupDatabase       m_groupDatabase    = null;
 
     /** Map with GroupPrincipals as keys, and Groups as values */
-    private final Map           m_groups           = new HashMap();
+    private final Map<Principal, Group>           m_groups           = new HashMap<Principal, Group>();
 
     /**
      * <p>
@@ -104,7 +111,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
      */
     public final Group getGroup( String name ) throws NoSuchPrincipalException
     {
-        Group group = (Group) m_groups.get( new GroupPrincipal( name ) );
+        Group group = m_groups.get( new GroupPrincipal( name ) );
         if ( group != null )
         {
             return group;
@@ -142,7 +149,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
                 dbClassName = XMLGroupDatabase.class.getName();
             }
             log.info( "Attempting to load group database class " + dbClassName );
-            Class dbClass = ClassUtil.findClass( "com.ecyrd.jspwiki.auth.authorize", dbClassName );
+            Class<?> dbClass = ClassUtil.findClass( "com.ecyrd.jspwiki.auth.authorize", dbClassName );
             m_groupDatabase = (GroupDatabase) dbClass.newInstance();
             m_groupDatabase.initialize( m_engine, m_engine.getWikiProperties() );
             log.info( "Group database initialized." );
@@ -189,7 +196,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
      */
     public final Principal[] getRoles()
     {
-        return (Principal[]) m_groups.keySet().toArray( new Principal[m_groups.size()] );
+        return m_groups.keySet().toArray( new Principal[m_groups.size()] );
     }
 
     /**
@@ -219,9 +226,8 @@ public final class GroupManager implements Authorizer, WikiEventListener
         Group[] groups = m_groupDatabase.groups();
         synchronized( m_groups )
         {
-            for( int i = 0; i < groups.length; i++ )
+            for( Group group : groups )
             {
-                Group group = groups[i];
                 // Add new group to cache; fire GROUP_ADD event
                 m_groups.put( group.getPrincipal(), group );
                 fireEvent( WikiSecurityEvent.GROUP_ADD, group );
@@ -267,17 +273,16 @@ public final class GroupManager implements Authorizer, WikiEventListener
         }
 
         // Get the group we're examining
-        Group group = (Group) m_groups.get( role );
+        Group group = m_groups.get( role );
         if ( group == null )
         {
             return false;
         }
 
         // Check each user principal to see if it belongs to the group
-        Principal[] principals = session.getPrincipals();
-        for ( int i = 0; i < principals.length; i++ )
+        for ( Principal principal : session.getPrincipals() )
         {
-            if ( AuthenticationManager.isUserPrincipal( principals[i] ) && group.isMember( principals[i] ) )
+            if ( AuthenticationManager.isUserPrincipal( principal ) && group.isMember( principal ) )
             {
                 return true;
             }
@@ -354,10 +359,9 @@ public final class GroupManager implements Authorizer, WikiEventListener
             group.setCreated( existingGroup.getCreated() );
             group.setModifier( existingGroup.getModifier() );
             group.setLastModified( existingGroup.getLastModified() );
-            Principal[] existingMembers = existingGroup.members();
-            for( int i = 0; i < existingMembers.length; i++ )
+            for( Principal existingMember : existingGroup.members() )
             {
-                group.add( existingMembers[i] );
+                group.add( existingMember );
             }
         }
         catch( NoSuchPrincipalException e )
@@ -374,9 +378,9 @@ public final class GroupManager implements Authorizer, WikiEventListener
         if ( members.length > 0 )
         {
             group.clear();
-            for( int i = 0; i < members.length; i++ )
+            for( String member : members )
             {
-                group.add( new WikiPrincipal( members[i] ) );
+                group.add( new WikiPrincipal( member ) );
             }
         }
 
@@ -451,7 +455,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
             throw new IllegalArgumentException( "Group cannot be null." );
         }
 
-        Group group = (Group) m_groups.get( new GroupPrincipal( index ) );
+        Group group = m_groups.get( new GroupPrincipal( index ) );
         if ( group == null )
         {
             throw new NoSuchPrincipalException( "Group " + index + " not found" );
@@ -512,7 +516,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
         // TODO: check for appropriate permissions
 
         // If group already exists, delete it; fire GROUP_REMOVE event
-        Group oldGroup = (Group) m_groups.get( group.getPrincipal() );
+        Group oldGroup = m_groups.get( group.getPrincipal() );
         if ( oldGroup != null )
         {
             fireEvent( WikiSecurityEvent.GROUP_REMOVE, oldGroup );
@@ -574,13 +578,12 @@ public final class GroupManager implements Authorizer, WikiEventListener
      */
     public final void validateGroup( WikiContext context, Group group )
     {
-        WikiSession session = context.getWikiSession();
-        InputValidator validator = new InputValidator( MESSAGES_KEY, session );
+        InputValidator validator = new InputValidator( MESSAGES_KEY, context );
 
         // Name cannot be null or one of the restricted names
         try
         {
-            checkGroupName( session, group.getName() );
+            checkGroupName( context, group.getName() );
         }
         catch( WikiSecurityException e )
         {
@@ -602,7 +605,7 @@ public final class GroupManager implements Authorizer, WikiEventListener
      */
     protected final String[] extractMembers( String memberLine )
     {
-        Set members = new HashSet();
+        Set<String> members = new HashSet<String>();
         if ( memberLine != null )
         {
             StringTokenizer tok = new StringTokenizer( memberLine, "\n" );
@@ -615,29 +618,24 @@ public final class GroupManager implements Authorizer, WikiEventListener
                 }
             }
         }
-        return (String[]) members.toArray( new String[members.size()] );
+        return members.toArray( new String[members.size()] );
     }
 
     /**
      * Checks if a String is blank or a restricted Group name, and if it is,
      * appends an error to the WikiSession's message list.
-     * @param session the wiki session
+     * @param context the wiki context
      * @param name the Group name to test
      * @throws WikiSecurityException if <code>session</code> is
      * <code>null</code> or the Group name is illegal
      * @see Group#RESTRICTED_GROUPNAMES
      */
-    protected final void checkGroupName( WikiSession session, String name ) throws WikiSecurityException
+    protected final void checkGroupName( WikiContext context, String name ) throws WikiSecurityException
     {
         //TODO: groups cannot have the same name as a user
 
-        if( session == null )
-        {
-            throw new WikiSecurityException( "Session cannot be null." );
-        }
-
         // Name cannot be null
-        InputValidator validator = new InputValidator( MESSAGES_KEY, session );
+        InputValidator validator = new InputValidator( MESSAGES_KEY, context );
         validator.validateNotNull( name, "Group name" );
 
         // Name cannot be one of the restricted names either
@@ -716,16 +714,14 @@ public final class GroupManager implements Authorizer, WikiEventListener
             int groupsChanged = 0;
             try
             {
-                Group[] groups = m_groupDatabase.groups();
-                for ( int i = 0; i < groups.length; i++ )
+                for ( Group group : m_groupDatabase.groups() )
                 {
                     boolean groupChanged = false;
-                    Group group = groups[i];
-                    for ( int j = 0; j < oldPrincipals.length; j++ )
+                    for ( Principal oldPrincipal : oldPrincipals )
                     {
-                        if ( group.isMember( oldPrincipals[j] ) )
+                        if ( group.isMember( oldPrincipal ) )
                         {
-                            group.remove( oldPrincipals[j] );
+                            group.remove( oldPrincipal );
                             group.add( newPrincipal );
                             groupChanged = true;
                         }
