@@ -1,15 +1,22 @@
-/*
- * JSPWiki - a JSP-based WikiWiki clone. Copyright (C) 2001-2003 Janne Jalkanen
- * (Janne.Jalkanen@iki.fi) This program is free software; you can redistribute
- * it and/or modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of the
- * License, or (at your option) any later version. This program is distributed
- * in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
- * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU Lesser General Public License for more details. You should have
- * received a copy of the GNU Lesser General Public License along with this
- * program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA 02111-1307 USA
+/* 
+    JSPWiki - a JSP-based WikiWiki clone.
+
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.auth.authorize;
 
@@ -128,9 +135,9 @@ public class WebContainerAuthorizer implements WebAuthorizer
         if ( m_containerRoles.length > 0 )
         {
             String roles = "";
-            for( int i = 0; i < m_containerRoles.length; i++ )
+            for( Role containerRole : m_containerRoles )
             {
-                roles = roles + m_containerRoles[i] + " ";
+                roles = roles + containerRole + " ";
             }
             log.info( " JSPWiki determined the web container manages these roles: " + roles );
         }
@@ -160,9 +167,10 @@ public class WebContainerAuthorizer implements WebAuthorizer
      * return <code>false</code>.
      * This method simply examines the WikiSession subject to see if it
      * possesses the desired Principal. We assume that the method
-     * {@link com.ecyrd.jspwiki.auth.AuthenticationManager#login(HttpServletRequest)}
-     * previously executed at user login time, and that it has injected
-     * the role Principals that were in force at login time.
+     * {@link com.ecyrd.jspwiki.ui.WikiServletFilter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)}
+     * previously executed, and that it has set the WikiSession
+     * subject correctly by logging in the user with the various login modules,
+     * in particular {@link com.ecyrd.jspwiki.auth.login.WebContainerLoginModule}}.
      * This is definitely a hack,
      * but it eliminates the need for WikiSession to keep dangling
      * references to the last WikiContext hanging around, just
@@ -193,11 +201,11 @@ public class WebContainerAuthorizer implements WebAuthorizer
      */
     public Principal findRole( String role )
     {
-        for( int i = 0; i < m_containerRoles.length; i++ )
+        for( Role containerRole : m_containerRoles )
         {
-            if ( m_containerRoles[i].getName().equals( role ) )
+            if ( containerRole.getName().equals( role ) )
             {
-                return m_containerRoles[i];
+                return containerRole;
             }
         }
         return null;
@@ -236,13 +244,13 @@ public class WebContainerAuthorizer implements WebAuthorizer
         selector = "//j:web-app/j:security-constraint[j:web-resource-collection/j:url-pattern=\"" + url + "\"]";
         xpath = XPath.newInstance( selector );
         xpath.addNamespace( "j", J2EE_SCHEMA_24_NAMESPACE );
-        List constraints = xpath.selectNodes( root );
+        List<?> constraints = xpath.selectNodes( root );
 
         // Get all constraints that match our Role pattern
         selector = "//j:web-app/j:security-constraint[j:auth-constraint/j:role-name=\"" + role.getName() + "\"]";
         xpath = XPath.newInstance( selector );
         xpath.addNamespace( "j", J2EE_SCHEMA_24_NAMESPACE );
-        List roles = xpath.selectNodes( root );
+        List<?> roles = xpath.selectNodes( root );
 
         // If we can't find either one, we must not be constrained
         if ( constraints.size() == 0 )
@@ -263,10 +271,10 @@ public class WebContainerAuthorizer implements WebAuthorizer
         }
 
         // If a constraint is contained in both lists, we must be constrained
-        for ( Iterator c = constraints.iterator(); c.hasNext(); )
+        for ( Iterator<?> c = constraints.iterator(); c.hasNext(); )
         {
             Element constraint = (Element)c.next();
-            for ( Iterator r = roles.iterator(); r.hasNext(); )
+            for ( Iterator<?> r = roles.iterator(); r.hasNext(); )
             {
                 Element roleConstraint = (Element)r.next();
                 if ( constraint.equals( roleConstraint ) )
@@ -308,7 +316,7 @@ public class WebContainerAuthorizer implements WebAuthorizer
      */
     public Principal[] getRoles()
     {
-        return (Principal[]) m_containerRoles.clone();
+        return m_containerRoles.clone();
     }
 
     /**
@@ -322,15 +330,15 @@ public class WebContainerAuthorizer implements WebAuthorizer
      */
     protected Role[] getRoles( Document webxml ) throws JDOMException
     {
-        Set roles = new HashSet();
+        Set<Role> roles = new HashSet<Role>();
         Element root = webxml.getRootElement();
 
         // Get roles referred to by constraints
         String selector = "//j:web-app/j:security-constraint/j:auth-constraint/j:role-name";
         XPath xpath = XPath.newInstance( selector );
         xpath.addNamespace( "j", J2EE_SCHEMA_24_NAMESPACE );
-        List nodes = xpath.selectNodes( root );
-        for( Iterator it = nodes.iterator(); it.hasNext(); )
+        List<?> nodes = xpath.selectNodes( root );
+        for( Iterator<?> it = nodes.iterator(); it.hasNext(); )
         {
             String role = ( (Element) it.next() ).getTextTrim();
             roles.add( new Role( role ) );
@@ -341,13 +349,13 @@ public class WebContainerAuthorizer implements WebAuthorizer
         xpath = XPath.newInstance( selector );
         xpath.addNamespace( "j", J2EE_SCHEMA_24_NAMESPACE );
         nodes = xpath.selectNodes( root );
-        for( Iterator it = nodes.iterator(); it.hasNext(); )
+        for( Iterator<?> it = nodes.iterator(); it.hasNext(); )
         {
             String role = ( (Element) it.next() ).getTextTrim();
             roles.add( new Role( role ) );
         }
 
-        return (Role[]) roles.toArray( new Role[roles.size()] );
+        return roles.toArray( new Role[roles.size()] );
     }
 
     /**
@@ -368,7 +376,6 @@ public class WebContainerAuthorizer implements WebAuthorizer
         SAXBuilder builder = new SAXBuilder();
         builder.setValidation( false );
         builder.setEntityResolver( new LocalEntityResolver() );
-        builder.setFeature("http://apache.org/xml/features/validation/schema", false);
         Document doc = null;
         if ( m_engine.getServletContext() == null )
         {
