@@ -1,21 +1,22 @@
 /* 
- JSPWiki - a JSP-based WikiWiki clone.
+    JSPWiki - a JSP-based WikiWiki clone.
 
- Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU Lesser General Public License as published by
- the Free Software Foundation; either version 2.1 of the License, or
- (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU Lesser General Public License for more details.
-
- You should have received a copy of the GNU Lesser General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.auth.user;
 
@@ -24,17 +25,12 @@ import java.security.Principal;
 import java.text.DateFormat;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
-import java.util.Date;
-import java.util.HashSet;
-import java.util.Properties;
-import java.util.Set;
+import java.util.*;
 
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
 
-import org.w3c.dom.Document;
-import org.w3c.dom.Element;
-import org.w3c.dom.NodeList;
+import org.w3c.dom.*;
 import org.xml.sax.SAXException;
 
 import com.ecyrd.jspwiki.NoRequiredPropertyException;
@@ -42,6 +38,7 @@ import com.ecyrd.jspwiki.WikiEngine;
 import com.ecyrd.jspwiki.auth.NoSuchPrincipalException;
 import com.ecyrd.jspwiki.auth.WikiPrincipal;
 import com.ecyrd.jspwiki.auth.WikiSecurityException;
+import com.ecyrd.jspwiki.util.Serializer;
 
 /**
  * <p>Manages {@link DefaultUserProfile} objects using XML files for persistence.
@@ -73,6 +70,8 @@ public class XMLUserDatabase extends AbstractUserDatabase
     
     private static final String DEFAULT_USERDATABASE = "userdatabase.xml";
 
+    private static final String ATTRIBUTES_TAG = "attributes";
+    
     private static final String CREATED           = "created";
     
     private static final String EMAIL             = "email";
@@ -83,8 +82,12 @@ public class XMLUserDatabase extends AbstractUserDatabase
 
     private static final String LAST_MODIFIED     = "lastModified";
     
+    private static final String LOCK_EXPIRY    = "lockExpiry";
+
     private static final String PASSWORD          = "password";
 
+    private static final String UID = "uid";
+
     private static final String USER_TAG          = "user";
 
     private static final String WIKI_NAME         = "wikiName";
@@ -184,6 +187,19 @@ public class XMLUserDatabase extends AbstractUserDatabase
         throw new NoSuchPrincipalException( "Not in database: " + index );
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public UserProfile findByUid( String uid ) throws NoSuchPrincipalException
+    {
+        UserProfile profile = findByAttribute( UID, uid );
+        if ( profile != null )
+        {
+            return profile;
+        }
+        throw new NoSuchPrincipalException( "Not in database: " + uid );
+    }
+
     /**
      * Looks up and returns the first {@link UserProfile}in the user database
      * that matches a profile having a given wiki name. If the user database
@@ -209,6 +225,7 @@ public class XMLUserDatabase extends AbstractUserDatabase
      * contain any profiles, this method will return a zero-length
      * array.
      * @return the WikiNames
+     * @throws WikiSecurityException In case things fail.
      */
     public Principal[] getWikiNames() throws WikiSecurityException
     {
@@ -216,7 +233,7 @@ public class XMLUserDatabase extends AbstractUserDatabase
         {
             throw new IllegalStateException( "FATAL: database does not exist" );
         }
-        Set principals = new HashSet();
+        SortedSet<Principal> principals = new TreeSet<Principal>();
         NodeList users = c_dom.getElementsByTagName( USER_TAG );
         for( int i = 0; i < users.getLength(); i++ )
         {
@@ -232,7 +249,7 @@ public class XMLUserDatabase extends AbstractUserDatabase
                 principals.add( principal );
             }
         }
-        return (Principal[])principals.toArray( new Principal[principals.size()] );
+        return principals.toArray( new Principal[principals.size()] );
     }
     
     /**
@@ -271,6 +288,7 @@ public class XMLUserDatabase extends AbstractUserDatabase
         log.info("XML user database at "+c_file.getAbsolutePath());
         
         buildDOM();
+        sanitizeDOM();
     }
     
     private void buildDOM()
@@ -344,7 +362,9 @@ public class XMLUserDatabase extends AbstractUserDatabase
             for( int i = 0; i < nodes.getLength(); i++ )
             {
                 Element user = (Element)nodes.item( i );
-                io.write( "<" + USER_TAG + " ");
+                io.write( "    <" + USER_TAG + " ");
+                io.write( UID );
+                io.write( "=\"" + user.getAttribute( UID ) + "\" " );
                 io.write( LOGIN_NAME );
                 io.write( "=\"" + user.getAttribute( LOGIN_NAME ) + "\" " );
                 io.write( WIKI_NAME );
@@ -359,7 +379,19 @@ public class XMLUserDatabase extends AbstractUserDatabase
                 io.write( "=\"" + user.getAttribute( CREATED ) + "\" " );
                 io.write( LAST_MODIFIED );
                 io.write( "=\"" + user.getAttribute( LAST_MODIFIED ) + "\" " );
-                io.write(" />\n");
+                io.write( LOCK_EXPIRY );
+                io.write( "=\"" + user.getAttribute( LOCK_EXPIRY ) + "\" " );
+                io.write( ">" );
+                NodeList attributes = user.getElementsByTagName( ATTRIBUTES_TAG );
+                for ( int j = 0; j < attributes.getLength(); j++ )
+                {
+                    Element attribute = (Element)attributes.item( j );
+                    String value = extractText( attribute );
+                    io.write( "\n        <" + ATTRIBUTES_TAG + ">" );
+                    io.write( value );
+                    io.write( "</" + ATTRIBUTES_TAG + ">" );
+                }
+                io.write("\n    </" +USER_TAG + ">\n");
             }
             io.write("</users>");
             io.close();
@@ -410,16 +442,6 @@ public class XMLUserDatabase extends AbstractUserDatabase
             }
         }
     }
-    
-    /**
-     * Determines whether the user database shares user/password data with the
-     * web container; always returns <code>false</code>.
-     * @see com.ecyrd.jspwiki.auth.user.UserDatabase#isSharedWithContainer()
-     */
-    public boolean isSharedWithContainer()
-    {
-        return false;
-    }
 
     /**
      * @see com.ecyrd.jspwiki.auth.user.UserDatabase#rename(String, String)
@@ -491,31 +513,46 @@ public class XMLUserDatabase extends AbstractUserDatabase
         String index = profile.getLoginName();
         NodeList users = c_dom.getElementsByTagName( USER_TAG );
         Element user = null;
-        boolean isNew = true;
         for( int i = 0; i < users.getLength(); i++ )
         {
             Element currentUser = (Element) users.item( i );
             if ( currentUser.getAttribute( LOGIN_NAME ).equals( index ) )
             {
                 user = currentUser;
-                isNew = false;
                 break;
             }
         }
+        
+        boolean isNew = false;
+        
         Date modDate = new Date( System.currentTimeMillis() );
-        if ( isNew )
+        if( user == null )
         {
+            // Create new user node
             profile.setCreated( modDate );
             log.info( "Creating new user " + index );
             user = c_dom.createElement( USER_TAG );
             c_dom.getDocumentElement().appendChild( user );
-            setAttribute( user, CREATED, c_format.format( modDate ) );
+            setAttribute( user, CREATED, c_format.format( profile.getCreated() ) );
+            isNew = true;
         }
+        else
+        {
+            // To update existing user node, delete old attributes first...
+            NodeList attributes = user.getElementsByTagName( ATTRIBUTES_TAG );
+            for ( int i = 0; i < attributes.getLength(); i++ )
+            {
+                user.removeChild( attributes.item( i ) );
+            }
+        }
+        
         setAttribute( user, LAST_MODIFIED, c_format.format( modDate ) );
         setAttribute( user, LOGIN_NAME, profile.getLoginName() );
         setAttribute( user, FULL_NAME, profile.getFullname() );
         setAttribute( user, WIKI_NAME, profile.getWikiName() );
         setAttribute( user, EMAIL, profile.getEmail() );
+        Date lockExpiry = profile.getLockExpiry();
+        setAttribute( user, LOCK_EXPIRY, lockExpiry == null ? "" : c_format.format( lockExpiry ) );
 
         // Hash and save the new password if it's different from old one
         String newPassword = profile.getPassword();
@@ -524,7 +561,24 @@ public class XMLUserDatabase extends AbstractUserDatabase
             String oldPassword = user.getAttribute( PASSWORD );
             if ( !oldPassword.equals( newPassword ) )
             {
-                setAttribute( user, PASSWORD, SHA_PREFIX + getHash( newPassword ) );
+                setAttribute( user, PASSWORD, getHash( newPassword ) );
+            }
+        }
+        
+        // Save the attributes as as Base64 string
+        if ( profile.getAttributes().size() > 0 )
+        {
+            try
+            {
+                String encodedAttributes = Serializer.serializeToBase64( profile.getAttributes() );
+                Element attributes = c_dom.createElement( ATTRIBUTES_TAG );
+                user.appendChild( attributes );
+                Text value = c_dom.createTextNode( encodedAttributes );
+                attributes.appendChild( value );
+            }
+            catch ( IOException e )
+            {
+                throw new WikiSecurityException( "Could not save user profile attribute. Reason: " + e.getMessage() );
             }
         }
 
@@ -541,7 +595,8 @@ public class XMLUserDatabase extends AbstractUserDatabase
 
     /**
      * Private method that returns the first {@link UserProfile}matching a
-     * &lt;user&gt; element's supplied attribute.
+     * &lt;user&gt; element's supplied attribute. This method will also
+     * set the UID if it has not yet been set.
      * @param matchAttribute
      * @param index
      * @return the profile, or <code>null</code> if not found
@@ -556,21 +611,60 @@ public class XMLUserDatabase extends AbstractUserDatabase
         checkForRefresh();
         
         NodeList users = c_dom.getElementsByTagName( USER_TAG );
+        
+        if( users == null ) return null;
+        
         for( int i = 0; i < users.getLength(); i++ )
         {
             Element user = (Element) users.item( i );
             if ( user.getAttribute( matchAttribute ).equals( index ) )
             {
-                UserProfile profile = new DefaultUserProfile();
+                UserProfile profile = newProfile();
+                
+                // Parse basic attributes
+                profile.setUid( user.getAttribute( UID ) );
+                if ( profile.getUid() == null )
+                {
+                    profile.setUid( generateUid( this ) );
+                }
                 profile.setLoginName( user.getAttribute( LOGIN_NAME ) );
                 profile.setFullname( user.getAttribute( FULL_NAME ) );
                 profile.setPassword( user.getAttribute( PASSWORD ) );
                 profile.setEmail( user.getAttribute( EMAIL ) );
+                
+                // Get created/modified timestamps
                 String created = user.getAttribute( CREATED );
                 String modified = user.getAttribute( LAST_MODIFIED );
-                
                 profile.setCreated( parseDate( profile, created ) );                  
                 profile.setLastModified( parseDate( profile, modified ) );                  
+                
+                // Is the profile locked?
+                String lockExpiry = user.getAttribute( LOCK_EXPIRY );
+                if ( lockExpiry == null || lockExpiry.length() == 0 )
+                {
+                    profile.setLockExpiry( null );
+                }
+                else
+                {
+                    profile.setLockExpiry( new Date( Long.parseLong( lockExpiry ) ) );
+                }
+                
+                // Extract all of the user's attributes (should only be one attributes tag, but you never know!)
+                NodeList attributes = user.getElementsByTagName( ATTRIBUTES_TAG );
+                for ( int j = 0; j < attributes.getLength(); j++ )
+                {
+                    Element attribute = (Element)attributes.item( j );
+                    String serializedMap = extractText( attribute );
+                    try
+                    {
+                        Map<String,? extends Serializable> map = Serializer.deserializeFromBase64( serializedMap );
+                        profile.getAttributes().putAll( map );
+                    }
+                    catch ( IOException e )
+                    {
+                        log.error( "Could not parse user profile attributes!", e );
+                    }
+                }
 
                 return profile;
             }
@@ -578,6 +672,29 @@ public class XMLUserDatabase extends AbstractUserDatabase
         return null;
     }
 
+    /**
+     * Extracts all of the text nodes that are immediate children of an Element.
+     * @param element the base element
+     * @return the text nodes that are immediate children of the base element, concatenated together
+     */
+    private String extractText( Element element )
+    {
+        String text = "";
+        if ( element.getChildNodes().getLength() > 0 )
+        {
+            NodeList children = element.getChildNodes();
+            for ( int k = 0; k < children.getLength(); k++ )
+            {
+                Node child = children.item( k );
+                if ( child.getNodeType() == Node.TEXT_NODE )
+                {
+                    text = text + ((Text)child).getData();
+                }
+            }
+        }
+        return text;
+    }
+
     /**
      *  Tries to parse a date using the default format - then, for backwards
      *  compatibility reasons, tries the platform default.
@@ -608,8 +725,64 @@ public class XMLUserDatabase extends AbstractUserDatabase
         }
         return null;
     }
+    
     /**
-     * Private method that sets an attibute value for a supplied DOM element.
+     * After loading the DOM, this method sanity-checks the dates in the DOM and makes
+     * sure they are formatted properly. This is sort-of hacky, but it should work.
+     */
+    private void sanitizeDOM()
+    {
+        if ( c_dom == null )
+        {
+            throw new IllegalStateException( "FATAL: database does not exist" );
+        }
+        
+        NodeList users = c_dom.getElementsByTagName( USER_TAG );
+        for( int i = 0; i < users.getLength(); i++ )
+        {
+            Element user = (Element) users.item( i );
+            
+            // Sanitize UID (and generate a new one if one does not exist)
+            String uid = user.getAttribute( UID ).trim();
+            if ( uid == null || uid.length() == 0 || "-1".equals( uid ) )
+            {
+                uid = String.valueOf( generateUid( this ) );
+                user.setAttribute( UID, uid );
+            }
+            
+            // Sanitize dates
+            String loginName = user.getAttribute( LOGIN_NAME );
+            String created = user.getAttribute( CREATED );
+            String modified = user.getAttribute( LAST_MODIFIED );
+            try
+            {
+                created = c_format.format( c_format.parse( created ) );
+                modified = c_format.format( c_format.parse( modified ) );
+                user.setAttribute( CREATED,  created );
+                user.setAttribute( LAST_MODIFIED,  modified );
+            }
+            catch( ParseException e )
+            {
+                try
+                {
+                    created = c_format.format( c_defaultFormat.parse( created ) );
+                    modified = c_format.format( c_defaultFormat.parse( modified ) );
+                    user.setAttribute( CREATED,  created );
+                    user.setAttribute( LAST_MODIFIED,  modified );
+                }
+                catch ( ParseException e2)
+                {
+                    log.warn("Could not parse 'created' or 'lastModified' "
+                        + "attribute for "
+                        + " profile '" + loginName + "'."
+                        + " It may have been tampered with." );
+                }            
+            }
+        }
+    }
+    
+    /**
+     * Private method that sets an attribute value for a supplied DOM element.
      * @param element the element whose attribute is to be set
      * @param attribute the name of the attribute to set
      * @param value the desired attribute value
