@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2006 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 
 package com.ecyrd.jspwiki.event;
@@ -153,10 +154,10 @@ public final class WikiEventManager
     private static WikiEventListener c_monitor = null;
 
     /* The Map of client object to WikiEventDelegate. */
-    private final Map m_delegates = new HashMap();
+    private final Map<Object, WikiEventDelegate> m_delegates = new HashMap<Object, WikiEventDelegate>();
 
     /* The Vector containing any preloaded WikiEventDelegates. */
-    private final Vector m_preloadCache = new Vector();
+    private final Vector<WikiEventDelegate> m_preloadCache = new Vector<WikiEventDelegate>();
 
     /* Singleton instance of the WikiEventManager. */
     private static WikiEventManager c_instance = null;
@@ -390,7 +391,7 @@ public final class WikiEventManager
                 // then see if any of the cached delegates match the class of the incoming client
                 for( int i = m_preloadCache.size()-1 ; i >= 0 ; i-- ) // start with most-recently added
                 {
-                    WikiEventDelegate delegate = (WikiEventDelegate)m_preloadCache.elementAt(i);
+                    WikiEventDelegate delegate = m_preloadCache.elementAt(i);
                     if( delegate.getClientClass() == null
                         || delegate.getClientClass().equals(client.getClass()) )
                     {
@@ -405,7 +406,7 @@ public final class WikiEventManager
                 }
             }
             // otherwise treat normally...
-            WikiEventDelegate delegate = (WikiEventDelegate)m_delegates.get( client );
+            WikiEventDelegate delegate = m_delegates.get( client );
             if( delegate == null )
             {
                 delegate = new WikiEventDelegate( client );
@@ -433,7 +434,7 @@ public final class WikiEventManager
     {
         /* A list of event listeners for this instance. */
 
-        private ArrayList m_listenerList = new ArrayList();
+        private ArrayList<WeakReference<WikiEventListener>> m_listenerList = new ArrayList<WeakReference<WikiEventListener>>();
 
         private Class  m_class  = null;
 
@@ -473,7 +474,7 @@ public final class WikiEventManager
         {
             synchronized( m_listenerList )
             {
-                TreeSet set = new TreeSet( new WikiEventListenerComparator() );
+                TreeSet<WikiEventListener> set = new TreeSet<WikiEventListener>( new WikiEventListenerComparator() );
 
                 for( Iterator i = m_listenerList.iterator(); i.hasNext(); )
                 {
@@ -500,7 +501,7 @@ public final class WikiEventManager
         {
             synchronized( m_listenerList )
             {
-                return m_listenerList.add( new WeakReference(listener) );
+                return m_listenerList.add( new WeakReference<WikiEventListener>(listener) );
             }
         }
 
@@ -577,7 +578,7 @@ public final class WikiEventManager
                     {
                         for( int i = 0; i < m_listenerList.size(); i++ )
                         {
-                            WeakReference w = (WeakReference)m_listenerList.get(i);
+                            WeakReference w = m_listenerList.get(i);
 
                             if( w.get() == null ) m_listenerList.remove(i--);
                         }
@@ -597,22 +598,14 @@ public final class WikiEventManager
 
     } // end inner class WikiEventDelegate
 
-    private static class WikiEventListenerComparator implements Comparator
+    private static class WikiEventListenerComparator implements Comparator<WikiEventListener>
     {
         // TODO: This method is a critical performance bottleneck
-        public int compare(Object arg0, Object arg1)
+        public int compare(WikiEventListener w0, WikiEventListener w1)
         {
-            if( arg0 instanceof WikiEventListener && arg1 instanceof WikiEventListener )
-            {
-                WikiEventListener w0 = (WikiEventListener) arg0;
-                WikiEventListener w1 = (WikiEventListener) arg1;
+            if( w1 == w0 || w0.equals(w1) ) return 0;
 
-                if( w1 == w0 || w0.equals(w1) ) return 0;
-
-                return w1.hashCode() - w0.hashCode();
-            }
-
-            throw new ClassCastException( arg1.getClass().getName() + " != " + arg0.getClass().getName() );
+            return w1.hashCode() - w0.hashCode();
         }
     }
 } // end com.ecyrd.jspwiki.event.WikiEventManager
