@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.filters;
 
@@ -23,6 +24,8 @@ import java.io.*;
 import java.util.*;
 
 import javax.servlet.http.HttpServletRequest;
+import javax.servlet.http.HttpServletResponse;
+import javax.servlet.jsp.PageContext;
 
 import net.sf.akismet.Akismet;
 
@@ -34,7 +37,9 @@ import org.apache.oro.text.regex.*;
 
 import com.ecyrd.jspwiki.*;
 import com.ecyrd.jspwiki.attachment.Attachment;
+import com.ecyrd.jspwiki.auth.user.UserProfile;
 import com.ecyrd.jspwiki.providers.ProviderException;
+import com.ecyrd.jspwiki.ui.EditorManager;
 
 /**
  *  This is Herb, the JSPWiki spamfilter that can also do choke modifications.
@@ -53,28 +58,79 @@ import com.ecyrd.jspwiki.providers.ProviderException;
  *    <li>akismet-apikey - The Akismet API key (see akismet.org)</li>
  *    <li>ignoreauthenticated - If set to "true", all authenticated users are ignored and never caught in SpamFilter</li>
  *    <li>captcha - Sets the captcha technology to use.  Current allowed values are "none" and "asirra".</li>
+ *    <li>strategy - Sets the filtering strategy to use.  If set to "eager", will stop at the first probable
+ *        match, and won't consider any other tests.  This is the default, as it's considerably lighter. If set to "score", will go through all of the tests
+ *        and calculates a score for the spam, which is then compared to a filter level value.
  *  </ul>
  *
+ *  <p>Please see the default editors/plain.jsp for examples on how the SpamFilter integrates
+ *  with the editor system.</p>
+ *  
  *  <p>Changes by admin users are ignored in any case.</p>
  *
  *  @since 2.1.112
- *  @author Janne Jalkanen
  */
 public class SpamFilter
     extends BasicPageFilter
 {
+    private static final String ATTR_SPAMFILTER_SCORE = "spamfilter.score";
+    private static final String REASON_REGEXP = "Regexp";
+    private static final String REASON_IP_BANNED_TEMPORARILY = "IPBannedTemporarily";
+    private static final String REASON_BOT_TRAP = "BotTrap";
+    private static final String REASON_AKISMET = "Akismet";
+    private static final String REASON_TOO_MANY_URLS = "TooManyUrls";
+    private static final String REASON_SIMILAR_MODIFICATIONS = "SimilarModifications";
+    private static final String REASON_TOO_MANY_MODIFICATIONS = "TooManyModifications";
+    private static final String REASON_UTF8_TRAP = "UTF8Trap";
+
     private static final String LISTVAR = "spamwords";
+    
+    /** The filter property name for specifying the page which contains the list of spamwords.
+     *  Value is <tt>{@value}</tt>. */
     public static final String  PROP_WORDLIST              = "wordlist";
+    
+    /** The filter property name for the page to which you are directed if Herb rejects your
+     *  edit.  Value is <tt>{@value}</tt>. */
     public static final String  PROP_ERRORPAGE             = "errorpage";
+    
+    /** The filter property name for specifying how many changes is any given IP address
+     *  allowed to do per minute.  Value is <tt>{@value}</tt>.
+     */
     public static final String  PROP_PAGECHANGES           = "pagechangesinminute";
+    
+    /** The filter property name for specifying how many similar changes are allowed
+     *  before a host is banned.  Value is <tt>{@value}</tt>.
+     */
     public static final String  PROP_SIMILARCHANGES        = "similarchanges";
+    
+    /** The filter property name for specifying how long a host is banned.  Value is <tt>{@value}</tt>.*/
     public static final String  PROP_BANTIME               = "bantime";
+    
+    /** The filter property name for the attachment containing the blacklist.  Value is <tt>{@value}</tt>.*/
     public static final String  PROP_BLACKLIST             = "blacklist";
+    
+    /** The filter property name for specifying how many URLs can any given edit contain.  
+     *  Value is <tt>{@value}</tt> */
     public static final String  PROP_MAXURLS               = "maxurls";
+    
+    /** The filter property name for specifying the Akismet API-key.  Value is <tt>{@value}</tt>. */
     public static final String  PROP_AKISMET_API_KEY       = "akismet-apikey";
+    
+    /** The filter property name for specifying whether authenticated users should be ignored. Value is <tt>{@value}</tt>. */
     public static final String  PROP_IGNORE_AUTHENTICATED  = "ignoreauthenticated";
+    
+    /** The filter property name for specifying which captcha technology should be used. Value is <tt>{@value}</tt>. */
     public static final String  PROP_CAPTCHA               = "captcha";
     
+    /** The filter property name for specifying which filter strategy should be used.  Value is <tt>{@value}</tt>. */
+    public static final String  PROP_FILTERSTRATEGY        = "strategy";
+
+    /** The string specifying the "eager" strategy. Value is <tt>{@value}</tt>. */
+    public static final String  STRATEGY_EAGER             = "eager";
+    
+    /** The string specifying the "score" strategy. Value is <tt>{@value}</tt>. */
+    public static final String  STRATEGY_SCORE             = "score";
+
     private static final String URL_REGEXP = "(http://|https://|mailto:)([A-Za-z0-9_/\\.\\+\\?\\#\\-\\@=&;]+)";
 
     private String          m_forbiddenWordsPage = "SpamFilterWordList";
@@ -84,19 +140,19 @@ public class SpamFilter
     private PatternMatcher  m_matcher = new Perl5Matcher();
     private PatternCompiler m_compiler = new Perl5Compiler();
 
-    private Collection      m_spamPatterns = null;
+    private Collection<Pattern> m_spamPatterns = null;
 
     private Date            m_lastRebuild = new Date( 0L );
 
-    static  Logger          spamlog = Logger.getLogger( "SpamLog" );
-    static  Logger          log = Logger.getLogger( SpamFilter.class );
+    private static  Logger  c_spamlog = Logger.getLogger( "SpamLog" );
+    private static  Logger  log = Logger.getLogger( SpamFilter.class );
 
 
-    private Vector          m_temporaryBanList = new Vector();
+    private Vector<Host>    m_temporaryBanList = new Vector<Host>();
 
     private int             m_banTime = 60; // minutes
 
-    private Vector          m_lastModifications = new Vector();
+    private Vector<Host>    m_lastModifications = new Vector<Host>();
 
     /**
      *  How many times a single IP address can change a page per minute?
@@ -119,12 +175,32 @@ public class SpamFilter
     private String          m_akismetAPIKey = null;
 
     private boolean         m_useCaptcha = false;
-    
+
+    /** The limit at which we consider something to be spam. */
+    private int             m_scoreLimit = 1;
+
     /**
      * If set to true, will ignore anyone who is in Authenticated role.
      */
     private boolean         m_ignoreAuthenticated = false;
 
+    private boolean         m_stopAtFirstMatch = true;
+
+    private static String   c_hashName;
+    private static long     c_lastUpdate;
+
+    /** The HASH_DELAY value is a maximum amount of time that an user can keep
+     *  a session open, because after the value has expired, we will invent a new
+     *  hash field name.  By default this is {@value} hours, which should be ample
+     *  time for someone.
+     */
+    private static final long HASH_DELAY = 24;
+
+
+    /**
+     *  {@inheritDoc}
+     */
+    @Override
     public void initialize( WikiEngine engine, Properties properties )
     {
         m_forbiddenWordsPage = properties.getProperty( PROP_WORDLIST,
@@ -153,9 +229,9 @@ public class SpamFilter
         m_ignoreAuthenticated = TextUtil.getBooleanProperty( properties,
                                                              PROP_IGNORE_AUTHENTICATED,
                                                              m_ignoreAuthenticated );
-        
+
         m_useCaptcha = properties.getProperty( PROP_CAPTCHA, "" ).equals("asirra");
-        
+
         try
         {
             m_urlPattern = m_compiler.compile( URL_REGEXP );
@@ -170,6 +246,10 @@ public class SpamFilter
                                                       PROP_AKISMET_API_KEY,
                                                       m_akismetAPIKey );
 
+        m_stopAtFirstMatch = TextUtil.getStringProperty( properties,
+                                                         PROP_FILTERSTRATEGY,
+                                                         STRATEGY_EAGER ).equals(STRATEGY_EAGER);
+
         log.info("# Spam filter initialized.  Temporary ban time "+m_banTime+
                  " mins, max page changes/minute: "+m_limitSinglePageChanges );
 
@@ -180,34 +260,38 @@ public class SpamFilter
     private static final int ACCEPT = 1;
     private static final int NOTE   = 2;
 
-    private String log( WikiContext ctx, int type, String source, String message )
+    private static String log( WikiContext ctx, int type, String source, String message )
     {
         message = TextUtil.replaceString( message, "\r\n", "\\r\\n" );
         message = TextUtil.replaceString( message, "\"", "\\\"" );
 
         String uid = getUniqueID();
 
-        String page = ctx.getPage().getName();
+        String page   = ctx.getPage().getName();
+        String reason = "UNKNOWN";
+        String addr   = ctx.getHttpRequest() != null ? ctx.getHttpRequest().getRemoteAddr() : "-";
 
         switch( type )
         {
             case REJECT:
-                spamlog.info("REJECTED "+source+" "+uid+" "+page+" "+message);
+                reason = "REJECTED";
                 break;
             case ACCEPT:
-                spamlog.info("ACCEPTED "+source+" "+uid+" "+page+" "+message);
+                reason = "ACCEPTED";
                 break;
             case NOTE:
-                spamlog.info("NOTE "+source+" "+uid+" "+page+" "+message);
+                reason = "NOTE";
                 break;
             default:
                 throw new InternalWikiException("Illegal type "+type);
         }
 
+        c_spamlog.info( reason+" "+source+" "+uid+" "+addr+" \""+page+"\" "+message );
+
         return uid;
     }
 
-
+    /** {@inheritDoc} */
     public String preSave( WikiContext context, String content )
         throws RedirectException
     {
@@ -223,21 +307,49 @@ public class SpamFilter
             checkPatternList(context, content, change);
         }
 
+        if( !m_stopAtFirstMatch )
+        {
+            Integer score = (Integer)context.getVariable(ATTR_SPAMFILTER_SCORE);
+
+            if( score != null && score.intValue() >= m_scoreLimit )
+            {
+                throw new RedirectException( "Herb says you got too many points",
+                                             getRedirectPage(context) );
+            }
+        }
+
         log( context, ACCEPT, "-", change );
         return content;
     }
 
+    private void checkStrategy( WikiContext context, String error, String message )
+        throws RedirectException
+    {
+        if( m_stopAtFirstMatch )
+        {
+            throw new RedirectException( message, getRedirectPage(context) );
+        }
+
+        Integer score = (Integer)context.getVariable( ATTR_SPAMFILTER_SCORE );
+
+        if( score != null )
+            score = score+1;
+        else
+            score = 1;
+
+        context.setVariable( ATTR_SPAMFILTER_SCORE, score );
+    }
     /**
      *  Parses a list of patterns and returns a Collection of compiled Pattern
      *  objects.
      *
      * @param source
      * @param list
-     * @return
+     * @return A Collection of the Patterns that were found from the lists.
      */
-    private Collection parseWordList( WikiPage source, String list )
+    private Collection<Pattern> parseWordList( WikiPage source, String list )
     {
-        ArrayList compiledpatterns = new ArrayList();
+        ArrayList<Pattern> compiledpatterns = new ArrayList<Pattern>();
 
         if( list != null )
         {
@@ -268,11 +380,11 @@ public class SpamFilter
      *  Pattern objects.
      *
      *  @param list
-     *  @return
+     *  @return The parsed blacklist patterns.
      */
-    private Collection parseBlacklist( String list )
+    private Collection<Pattern> parseBlacklist( String list )
     {
-        ArrayList compiledpatterns = new ArrayList();
+        ArrayList<Pattern> compiledpatterns = new ArrayList<Pattern>();
 
         if( list != null )
         {
@@ -378,10 +490,9 @@ public class SpamFilter
 
                 m_temporaryBanList.add( host );
 
-                String uid = log( context, REJECT, "TooManyModifications", change );
+                String uid = log( context, REJECT, REASON_TOO_MANY_MODIFICATIONS, change );
                 log.info( "SPAM:TooManyModifications ("+uid+"). Added host "+addr+" to temporary ban list for doing too many modifications/minute" );
-                throw new RedirectException( "Herb says you look like a spammer, and I trust Herb! (Incident code "+uid+")",
-                                             getRedirectPage(context) );
+                checkStrategy( context, REASON_TOO_MANY_MODIFICATIONS, "Herb says you look like a spammer, and I trust Herb! (Incident code "+uid+")" );
             }
 
             if( changeCounter >= m_limitSimilarChanges )
@@ -390,11 +501,10 @@ public class SpamFilter
 
                 m_temporaryBanList.add( host );
 
-                String uid = log( context, REJECT, "SimilarModifications", change );
+                String uid = log( context, REJECT, REASON_SIMILAR_MODIFICATIONS, change );
 
                 log.info("SPAM:SimilarModifications ("+uid+"). Added host "+addr+" to temporary ban list for doing too many similar modifications" );
-                throw new RedirectException( "Herb says you look like a spammer, and I trust Herb! (Incident code "+uid+")",
-                                             getRedirectPage(context) );
+                checkStrategy( context, REASON_SIMILAR_MODIFICATIONS, "Herb says you look like a spammer, and I trust Herb! (Incident code "+uid+")");
             }
 
             //
@@ -419,19 +529,24 @@ public class SpamFilter
 
                 m_temporaryBanList.add( host );
 
-                String uid = log( context, REJECT, "TooManyUrls", change );
+                String uid = log( context, REJECT, REASON_TOO_MANY_URLS, change );
 
                 log.info("SPAM:TooManyUrls ("+uid+"). Added host "+addr+" to temporary ban list for adding too many URLs" );
-                throw new RedirectException( "Herb says you look like a spammer, and I trust Herb! (Incident code "+uid+")",
-                                             getRedirectPage(context) );
+                checkStrategy( context, REASON_TOO_MANY_URLS, "Herb says you look like a spammer, and I trust Herb! (Incident code "+uid+")" );
             }
 
             //
             //  Check bot trap
             //
-            
+
             checkBotTrap( context, change );
-            
+
+            //
+            //  Check UTF-8 mangling
+            //
+
+            checkUTF8( context, change );
+
             //
             //  Do Akismet check.  This is good to be the last, because this is the most
             //  expensive operation.
@@ -509,12 +624,11 @@ public class SpamFilter
 
                     // m_temporaryBanList.add( host );
 
-                    String uid = log( context, REJECT, "Akismet", change );
+                    String uid = log( context, REJECT, REASON_AKISMET, change );
 
                     log.info("SPAM:Akismet ("+uid+"). Akismet thinks this change is spam; added host to temporary ban list.");
 
-                    throw new RedirectException("Akismet tells Herb you're a spammer, Herb trusts Akismet, and I trust Herb! (Incident code "+uid+")",
-                                                getRedirectPage(context) );
+                    checkStrategy( context, REASON_AKISMET, "Akismet tells Herb you're a spammer, Herb trusts Akismet, and I trust Herb! (Incident code "+uid+")");
                 }
             }
         }
@@ -523,42 +637,60 @@ public class SpamFilter
     /**
      *  Returns a static string which can be used to detect spambots which
      *  just wildly fill in all the fields.
-     *  
+     *
      *  @return A string
      */
     public static String getBotFieldName()
     {
-        return "submitauth";
+        return "submit_auth";
     }
-    
+
     /**
      *  This checks whether an invisible field is available in the request, and
      *  whether it's contents are suspected spam.
-     *  
+     *
      *  @param context
      *  @param change
-     * @throws RedirectException 
+     * @throws RedirectException
      */
     private void checkBotTrap( WikiContext context, String change ) throws RedirectException
     {
         HttpServletRequest request = context.getHttpRequest();
-        
+
         if( request != null )
         {
             String unspam = request.getParameter( getBotFieldName() );
             if( unspam != null && unspam.length() > 0 )
             {
-                String uid = log( context, REJECT, "BotTrap", change );
-                
+                String uid = log( context, REJECT, REASON_BOT_TRAP, change );
+
                 log.info("SPAM:BotTrap ("+uid+").  Wildly behaving bot detected.");
 
-                throw new RedirectException("Spamming attempt detected. (Incident code "+uid+")",
-                                            getRedirectPage(context) );
+                checkStrategy( context, REASON_BOT_TRAP, "Spamming attempt detected. (Incident code "+uid+")");
 
             }
         }
     }
-    
+
+    private void checkUTF8( WikiContext context, String change ) throws RedirectException
+    {
+        HttpServletRequest request = context.getHttpRequest();
+
+        if( request != null )
+        {
+            String utf8field = request.getParameter( "encodingcheck" );
+
+            if( utf8field != null && !utf8field.equals("\u3041") )
+            {
+                String uid = log( context, REJECT, REASON_UTF8_TRAP, change );
+
+                log.info("SPAM:UTF8Trap ("+uid+").  Wildly posting dumb bot detected.");
+
+                checkStrategy( context, REASON_UTF8_TRAP, "Spamming attempt detected. (Incident code "+uid+")");
+            }
+        }
+    }
+
     /**
      *  Goes through the ban list and cleans away any host which has expired from it.
      */
@@ -604,10 +736,9 @@ public class SpamFilter
                 {
                     long timeleft = (host.getReleaseTime() - now) / 1000L;
 
-                    log( context, REJECT, "IPBannedTemporarily", change );
+                    log( context, REJECT, REASON_IP_BANNED_TEMPORARILY, change );
 
-                    throw new RedirectException( "You have been temporarily banned from modifying this wiki. ("+timeleft+" seconds of ban left)",
-                                                 getRedirectPage(context) );
+                    checkStrategy( context, REASON_IP_BANNED_TEMPORARILY, "You have been temporarily banned from modifying this wiki. ("+timeleft+" seconds of ban left)");
                 }
             }
         }
@@ -659,7 +790,7 @@ public class SpamFilter
                 m_lastRebuild = new Date();
 
                 m_spamPatterns = parseWordList( source,
-                                                (String)source.getAttribute( LISTVAR ) );
+                                                (source != null) ? (String)source.getAttribute( LISTVAR ) : null );
 
                 log.info("Spam filter reloaded - recognizing "+m_spamPatterns.size()+" patterns from page "+m_forbiddenWordsPage);
 
@@ -671,7 +802,7 @@ public class SpamFilter
 
                     FileUtil.copyContents( new InputStreamReader(in,"UTF-8"), out );
 
-                    Collection blackList = parseBlacklist( out.toString() );
+                    Collection<Pattern> blackList = parseBlacklist( out.toString() );
 
                     log.info("...recognizing additional "+blackList.size()+" patterns from blacklist "+m_blacklist);
 
@@ -692,7 +823,7 @@ public class SpamFilter
 
     /**
      *  Does a check against a known pattern list.
-     *  
+     *
      *  @param context
      *  @param content
      *  @param change
@@ -709,10 +840,11 @@ public class SpamFilter
             return;
         }
 
-        for( Iterator i = m_spamPatterns.iterator(); i.hasNext(); )
-        {
-            Pattern p = (Pattern) i.next();
+        if( context.getHttpRequest() != null )
+            change += context.getHttpRequest().getRemoteAddr();
 
+        for( Pattern p : m_spamPatterns )
+        {
             // log.debug("Attempting to match page contents with "+p.getPattern());
 
             if( m_matcher.contains( change, p ) )
@@ -720,12 +852,11 @@ public class SpamFilter
                 //
                 //  Spam filter has a match.
                 //
-                String uid = log( context, REJECT, "Regexp("+p.getPattern()+")", change);
+                String uid = log( context, REJECT, REASON_REGEXP+"("+p.getPattern()+")", change);
 
                 log.info("SPAM:Regexp ("+uid+"). Content matches the spam filter '"+p.getPattern()+"'");
 
-                throw new RedirectException( "Herb says '"+p.getPattern()+"' is a bad spam word and I trust Herb! (Incident code "+uid+")",
-                                             getRedirectPage(context) );
+                checkStrategy( context, REASON_REGEXP, "Herb says '"+p.getPattern()+"' is a bad spam word and I trust Herb! (Incident code "+uid+")");
             }
         }
     }
@@ -737,7 +868,7 @@ public class SpamFilter
      *  @param newText
      *  @return Empty string, if there is no change.
      */
-    private String getChange( WikiContext context, String newText )
+    private static String getChange( WikiContext context, String newText )
     {
         WikiPage page = context.getPage();
         StringBuffer change = new StringBuffer();
@@ -801,10 +932,10 @@ public class SpamFilter
     }
 
     /**
-     *  Returns true, if this user should be ignored.
-     *  
+     *  Returns true, if this user should be ignored.  For example, admin users.
+     *
      * @param context
-     * @return
+     * @return True, if this users should be ignored.
      */
     private boolean ignoreThisUser(WikiContext context)
     {
@@ -828,10 +959,10 @@ public class SpamFilter
 
     /**
      *  Returns a random string of six uppercase characters.
-     *  
+     *
      *  @return A random string
      */
-    private String getUniqueID()
+    private static String getUniqueID()
     {
         StringBuffer sb = new StringBuffer();
         Random rand = new Random();
@@ -849,7 +980,7 @@ public class SpamFilter
     /**
      *  Returns a page to which we shall redirect, based on the current value
      *  of the "captcha" parameter.
-     *  
+     *
      *  @param ctx WikiContext
      *  @return An URL to redirect to
      */
@@ -857,15 +988,155 @@ public class SpamFilter
     {
         if( m_useCaptcha )
             return ctx.getURL( WikiContext.NONE, "Captcha.jsp", "page="+ctx.getEngine().encodeName(ctx.getPage().getName()) );
-        
+
         return ctx.getURL( WikiContext.VIEW, m_errorPage );
     }
 
+    /**
+     *  Checks whether the UserProfile matches certain checks.
+     *
+     *  @param profile The profile to check
+     *  @param context The WikiContext
+     *  @return False, if this userprofile is suspect and should not be allowed to be added.
+     *  @since 2.6.1
+     */
+    public boolean isValidUserProfile( WikiContext context, UserProfile profile )
+    {
+        try
+        {
+            checkPatternList( context, profile.getEmail(), profile.getEmail() );
+            checkPatternList( context, profile.getFullname(), profile.getFullname() );
+            checkPatternList( context, profile.getLoginName(), profile.getLoginName() );
+        }
+        catch( RedirectException e )
+        {
+            log.info("Detected attempt to create a spammer user account (see above for rejection reason)");
+            return false;
+        }
+
+        return true;
+    }
+
+    /**
+     *  This method is used to calculate an unique code when submitting the page
+     *  to detect edit conflicts.  It currently incorporates the last-modified
+     *  date of the page, and the IP address of the submitter.
+     *
+     *  @param page The WikiPage under edit
+     *  @param request The HTTP Request
+     *  @since 2.6
+     *  @return A hash value for this page and session
+     */
+    public static final String getSpamHash( WikiPage page, HttpServletRequest request )
+    {
+        long lastModified = 0;
+
+        if( page.getLastModified() != null )
+            lastModified = page.getLastModified().getTime();
+
+        long remote = request.getRemoteAddr().hashCode();
+
+        return Long.toString( lastModified ^ remote );
+    }
+
+    /**
+     *  Returns the name of the hash field to be used in this request.
+     *  The value is unique per session, and once the session has expired,
+     *  you cannot edit anymore.
+     *
+     *  @param request The page request
+     *  @return The name to be used in the hash field
+     *  @since  2.6
+     */
+
+    public static final String getHashFieldName( HttpServletRequest request )
+    {
+        String hash = null;
+
+        if( request.getSession() != null )
+        {
+            hash = (String)request.getSession().getAttribute("_hash");
+
+            if( hash == null )
+            {
+                hash = c_hashName;
+
+                request.getSession().setAttribute( "_hash", hash );
+            }
+        }
+
+        if( c_hashName == null || c_lastUpdate < (System.currentTimeMillis() - HASH_DELAY*60*60*1000) )
+        {
+            c_hashName = getUniqueID().toLowerCase();
+
+            c_lastUpdate = System.currentTimeMillis();
+        }
+
+        return hash != null ? hash : c_hashName;
+    }
+
 
     /**
-     *  A local class for storing host information.
+     *  This method checks if the hash value is still valid, i.e. if it exists at all. This
+     *  can occur in two cases: either this is a spam bot which is not adaptive, or it is
+     *  someone who has been editing one page for too long, and their session has expired.
+     *  <p>
+     *  This method puts a redirect to the http response field to page "SessionExpired"
+     *  and logs the incident in the spam log (it may or may not be spam, but it's rather likely
+     *  that it is).
      *
-     *  @author jalkanen
+     *  @param context The WikiContext
+     *  @param pageContext The JSP PageContext.
+     *  @return True, if hash is okay.  False, if hash is not okay, and you need to redirect.
+     *  @throws IOException If redirection fails
+     *  @since 2.6
+     */
+    public static final boolean checkHash( WikiContext context, PageContext pageContext )
+        throws IOException
+    {
+        String hashName = getHashFieldName( (HttpServletRequest)pageContext.getRequest() );
+
+        if( pageContext.getRequest().getParameter(hashName) == null )
+        {
+            if( pageContext.getAttribute( hashName ) == null )
+            {
+                String change = getChange( context, EditorManager.getEditedText( pageContext ) );
+
+                log( context, REJECT, "MissingHash", change );
+
+                String redirect = context.getURL(WikiContext.VIEW,"SessionExpired");
+                ((HttpServletResponse)pageContext.getResponse()).sendRedirect( redirect );
+
+                return false;
+            }
+        }
+
+        return true;
+    }
+
+    /**
+     *  This helper method adds all the input fields to your editor that the SpamFilter requires
+     *  to check for spam.  This <i>must</i> be in your editor form if you intend to use
+     *  the SpamFilter.
+     *  
+     *  @param pageContext The PageContext
+     *  @return A HTML string which contains input fields for the SpamFilter.
+     */
+    public static final String insertInputFields( PageContext pageContext )
+    {
+        WikiContext ctx = WikiContext.findContext(pageContext);
+        WikiEngine engine = ctx.getEngine();
+
+        StringBuffer sb = new StringBuffer();
+        if (engine.getContentEncoding().equals("UTF-8"))
+        {
+            sb.append("<input name='encodingcheck' type='hidden' value='\u3041' />\n");
+        }
+
+        return sb.toString();
+    }
+    /**
+     *  A local class for storing host information.
      *
      *  @since
      */
