@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.htmltowiki;
 
@@ -41,19 +42,54 @@ public class HtmlStringToWikiTranslator
 
     private static final String CYBERNEKO_PARSER = "org.cyberneko.html.parsers.SAXParser";
 
+    /**
+     *  Create a new translator.
+     */
     public HtmlStringToWikiTranslator()
     {}
 
+    /**
+     *  Translates text from HTML into WikiMarkup without a WikiContext (meaning
+     *  some things perhaps cannot be translated).  Uses the default configuration.
+     *  
+     *  @param html HTML text to translate
+     *  @return WikiMarkup
+     *  
+     *  @throws JDOMException If parsing fails
+     *  @throws IOException For other kinds of errors.
+     */
     public String translate( String html ) throws JDOMException, IOException
     {
         return translate( html, new XHtmlToWikiConfig() );
     }
 
+    /**
+     *  Translates text from HTML into WikiMarkup with a WikiContext.  The translation
+     *  accuracy is better.  Uses the default configuration.
+     *  
+     *  @param html HTML text to translate
+     *  @param wikiContext The WikiContext to use.
+     *  @return WikiMarkup
+     *  
+     *  @throws JDOMException If parsing fails
+     *  @throws IOException For other kinds of errors.
+     */
     public String translate( String html, WikiContext wikiContext ) throws JDOMException, IOException
     {
         return translate( html, new XHtmlToWikiConfig( wikiContext ) );
     }
 
+    /**
+     *  Translates text from HTML into WikiMarkup using a specified configuration.
+     *  
+     *  @param html HTML text to translate
+     *  @param config The configuration to use.
+     *  @return WikiMarkup
+     *  
+     *  @throws JDOMException If parsing fails
+     *  @throws IOException For other kinds of errors.
+     */
+
     public String translate( String html, XHtmlToWikiConfig config ) throws JDOMException, IOException
     {
         Element element = htmlStringToElement( html );
@@ -63,7 +99,7 @@ public class HtmlStringToWikiTranslator
     }
 
     /**
-     * use NekoHtml to parse HTML like well formed XHTML
+     * Use NekoHtml to parse HTML like well formed XHTML
      * 
      * @param html
      * @return xhtml jdom root element (node "HTML")
@@ -78,6 +114,12 @@ public class HtmlStringToWikiTranslator
         return element;
     }
 
+    /**
+     *  A static helper method to create HTML from an Element.
+     *  
+     *  @param element The element to get HTML from.
+     *  @return HTML
+     */
     public static String element2String( Element element )
     {
         Document document = new Document( element );
