@@ -1,24 +1,26 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.parser;
 
+import java.io.UnsupportedEncodingException;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.text.SimpleDateFormat;
@@ -32,17 +34,20 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 /**
- * Translates Creole markp to JSPWiki markup. Simple translator uses regular expressions.
- * See http://www.wikicreole.org for the WikiCreole spec. 
- * 
- * This translator can be configured through properties defined in
- * jspwiki.properties starting with "creole.*". See the 
- * jspwiki.properties file for an explanation of the properties
- * 
- * @author Steffen Schramm 
- * @author Hanno Eichelberger 
+ * <p>Translates Creole markp to JSPWiki markup. Simple translator uses regular expressions.
+ * See http://www.wikicreole.org for the WikiCreole spec.</p>
+ *
+ * <p>This translator can be configured through properties defined in
+ * jspwiki.properties starting with "creole.*". See the
+ * jspwiki.properties file for an explanation of the properties</p>
+ *
+ * <p><b>WARNING</b>: This is an experimental feature, and known to be
+ * broken.  Use at your own risk.</o>
+ *
+ * @author Steffen Schramm
+ * @author Hanno Eichelberger
  * @author Christoph Sauer
- * 
+ *
  * @see <a href="http://www.wikicreole.org/">Wiki Creole Spec</a>
  */
 public class CreoleToJSPWikiTranslator
@@ -59,13 +64,17 @@ public class CreoleToJSPWikiTranslator
     // [{$creolepagefilter.creoleversion}]
     // [{$creolepagefilter.linebreak}] -> bloglike/wikilike
 
-    public static String VAR_VERSION = "1.0.2";
+    /** The version of the filter. */
+    public static final String VAR_VERSION = "1.0.3";
 
-    public static String VAR_CREOLE_VERSION = "1.0";
+    /** The version of Creole that this filter supports. */
+    public static final String VAR_CREOLE_VERSION = "1.0";
 
-    public static String VAR_LINEBREAK_BLOGLIKE = "bloglike";
+    /** The linebreak style "bloglike". */
+    public static final String VAR_LINEBREAK_BLOGLIKE = "bloglike";
 
-    public static String VAR_LINEBREAK_C2LIKE = "c2like";
+    /** The linebreak style "c2like". */
+    public static final String VAR_LINEBREAK_C2LIKE = "c2like";
 
     private static final String CREOLE_BOLD = "\\*\\*((?s:.)*?)(\\*\\*|(\n\n|\r\r|\r\n\r\n))";
 
@@ -121,9 +130,9 @@ public class CreoleToJSPWikiTranslator
 
     private static final String PREFORMATTED_PROTECTED = "\\Q{{{\\E.*?\\Q}}}\\E";
 
-    private static final String CREOLE_LINEBREAKS = "([^\\s\\\\])(\r\n|\r|\n)+(?=[^\\s\\*#])";
+    //private static final String CREOLE_LINEBREAKS = "([^\\s\\\\])(\r\n|\r|\n)+(?=[^\\s\\*#])";
 
-    private static final String JSPWIKI_LINEBREAKS = "$1\\\\\\\\$2";
+    //private static final String JSPWIKI_LINEBREAKS = "$1\\\\\\\\$2";
 
     private static final String CREOLE_TABLE = "(\n|\r|\r\n|^)(\\|[^\n\r]*)\\|(\\t| )*(\n|\r|\r\n|$)";
 
@@ -158,10 +167,18 @@ public class CreoleToJSPWikiTranslator
 
     private static final String ESCAPE_PROTECTED = "~(\\*\\*|~|//|-|#|\\{\\{|}}|\\\\|~\\[~~[|]]|----|=|\\|)";
 
-    private static final Map protectionMap = new HashMap();
+    private static Map<String, String> c_protectionMap = new HashMap<String, String>();
 
-    public ArrayList hashList = new ArrayList();
+    private        ArrayList<String> m_hashList = new ArrayList<String>();
 
+    /**
+     *  I have no idea what this method does.  Could someone please tell me?
+     *  
+     * @param wikiProps A property set
+     * @param content The content to translate?
+     * @param username The username in the signature?
+     * @return Probably some translated content.
+     */
     public String translateSignature(Properties wikiProps, final String content, String username)
     {
 
@@ -190,26 +207,37 @@ public class CreoleToJSPWikiTranslator
         Calendar cal = Calendar.getInstance();
         result = translateElement(result, SIGNATURE_AND_DATE, "-- [[" + username + "]], " + df.format(cal.getTime()));
         result = translateElement(result, SIGNATURE, "-- [[" + username + "]]");
-        result = unprotectMarkup(result);
+        result = unprotectMarkup(result, false);
         return result;
     }
 
-    /** Translates Creole markup to JSPWiki markup */
+    /** 
+     *  Translates Creole markup to JSPWiki markup 
+     *  
+     *  @param wikiProps A set of Wiki Properties
+     *  @param content Creole markup
+     *  @return Wiki markup
+     */
     public String translate(Properties wikiProps, final String content)
     {
-        String tmp = wikiProps.getProperty("creole.blogLineBreaks");
         boolean blogLineBreaks = false;
+        /*
+        // BROKEN, breaks on different platforms.
+        String tmp = wikiProps.getProperty("creole.blogLineBreaks");
         if (tmp != null)
         {
             if (tmp.trim().equals("true"))
                 blogLineBreaks = true;
         }
-
+        */
         String imagePlugin = wikiProps.getProperty("creole.imagePlugin.name");
 
         String result = content;
-        result = result.replace("\r\n", "\n");
-        result = result.replace("\r", "\n");
+        //
+        // Breaks on OSX.  It is never a good idea to tamper with the linebreaks.  JSPWiki always
+        // stores linebreaks as \r\n, regardless of the platform.
+        //result = result.replace("\r\n", "\n");
+        //result = result.replace("\r", "\n");
 
         /* Now protect the rest */
         result = protectMarkup(result);
@@ -238,15 +266,16 @@ public class CreoleToJSPWikiTranslator
         result = translateElement(result, CREOLE_TABLE, JSPWIKI_TABLE);
         result = replaceArea(result, TABLE_HEADER_PROTECTED, "\\|=([^\\|]*)=|\\|=([^\\|]*)", "||$1$2");
 
+        /*
         if (blogLineBreaks)
         {
             result = translateElement(result, CREOLE_LINEBREAKS, JSPWIKI_LINEBREAKS);
         }
-
-        result = unprotectMarkup(result);
+        */
+        result = unprotectMarkup(result, true);
 
         result = translateVariables(result, blogLineBreaks);
-        result = result.replace("\n", System.getProperty("line.separator"));
+        //result = result.replace("\n", System.getProperty("line.separator"));
         return result;
     }
 
@@ -298,6 +327,13 @@ public class CreoleToJSPWikiTranslator
             }
             counter = 0;
         }
+        
+        // Fixes testExtensions5
+        if( content.endsWith( "\n" ) && result.charAt( result.length()-1 ) != '\n' ) 
+        {
+            result.append( '\n' );
+        }
+        
         return result.toString();
     }
 
@@ -313,19 +349,20 @@ public class CreoleToJSPWikiTranslator
     /**
      * Undoes the protection. This is done by replacing the md5 hashes by the
      * original markup.
-     * 
+     *
      * @see #protectMarkup(String)
      */
-    private String unprotectMarkup(String content)
+    private String unprotectMarkup(String content,boolean replacePlugins)
     {
-        Object[] it = this.hashList.toArray();
+        Object[] it = this.m_hashList.toArray();
 
         for (int i = it.length - 1; i >= 0; i--)
         {
             String hash = (String) it[i];
-            String protectedMarkup = (String) protectionMap.get(hash);
+            String protectedMarkup = c_protectionMap.get(hash);
             content = content.replace(hash, protectedMarkup);
-            if (protectedMarkup.length() < 3 || (protectedMarkup.length() > 2 && !protectedMarkup.substring(0, 3).equals("{{{")))
+            if ((protectedMarkup.length() < 3 || (protectedMarkup.length() > 2 &&
+                !protectedMarkup.substring(0, 3).equals("{{{")))&&replacePlugins)
                 content = translateElement(content, CREOLE_PLUGIN, JSPWIKI_PLUGIN);
 
         }
@@ -343,14 +380,14 @@ public class CreoleToJSPWikiTranslator
      * This protection is a simple method to keep the regular expressions for
      * the other markup simple. Internally the protection is done by replacing
      * the protected markup with the the md5 hash of the markup.
-     * 
+     *
      * @param content
-     * @return
+     * @return The content with protection
      */
     private String protectMarkup(String content)
     {
-        protectionMap.clear();
-        this.hashList = new ArrayList();
+        c_protectionMap.clear();
+        m_hashList = new ArrayList<String>();
         content = protectMarkup(content, PREFORMATTED_PROTECTED, "", "");
         content = protectMarkup(content, URL_PROTECTED, "", "");
         content = protectMarkup(content, ESCAPE_PROTECTED, "", "");
@@ -361,29 +398,29 @@ public class CreoleToJSPWikiTranslator
         return content;
     }
 
-    public ArrayList readPlaceholderProperties(Properties wikiProps)
+    private ArrayList readPlaceholderProperties(Properties wikiProps)
     {
         Set keySet = wikiProps.keySet();
         Object[] keys = keySet.toArray();
-        ArrayList result = new ArrayList();
+        ArrayList<String[]> result = new ArrayList<String[]>();
 
-        for (int i = 0; i < keys.length; i++)
+        for( int i = 0; i < keys.length; i++ )
         {
             String key = keys[i] + "";
-            String value = wikiProps.getProperty(keys[i] + "");
-            if ((key).indexOf("creole.imagePlugin.para.%") > -1)
+            String value = wikiProps.getProperty( keys[i] + "" );
+            if( key.indexOf( "creole.imagePlugin.para.%" ) > -1 )
             {
                 String[] pair = new String[2];
-                pair[0] = key.replaceAll("creole.imagePlugin.para.%", "");
+                pair[0] = key.replaceAll( "creole.imagePlugin.para.%", "" );
                 pair[1] = value;
-                result.add(pair);
+                result.add( pair );
             }
         }
         return result;
     }
 
-    public String replaceImageArea(Properties wikiProps, String content, String markupRegex, String replaceContent, int groupPos,
-                                   String imagePlugin)
+    private String replaceImageArea(Properties wikiProps, String content, String markupRegex, String replaceContent, int groupPos,
+                                    String imagePlugin)
     {
         Matcher matcher = Pattern.compile(markupRegex, Pattern.MULTILINE | Pattern.DOTALL).matcher(content);
         String contentCopy = content;
@@ -447,7 +484,7 @@ public class CreoleToJSPWikiTranslator
         return contentCopy;
     }
 
-    public String replaceArea(String content, String markupRegex, String replaceSource, String replaceTarget)
+    private String replaceArea(String content, String markupRegex, String replaceSource, String replaceTarget)
     {
         Matcher matcher = Pattern.compile(markupRegex, Pattern.MULTILINE | Pattern.DOTALL).matcher(content);
         String contentCopy = content;
@@ -466,7 +503,7 @@ public class CreoleToJSPWikiTranslator
 
     /**
      * Protects a specific markup
-     * 
+     *
      * @see #protectMarkup(String)
      */
     private String protectMarkup(String content, String markupRegex, String replaceSource, String replaceTarget)
@@ -481,14 +518,20 @@ public class CreoleToJSPWikiTranslator
             {
                 MessageDigest digest = MessageDigest.getInstance("MD5");
                 digest.reset();
-                digest.update(protectedMarkup.getBytes());
+                digest.update(protectedMarkup.getBytes("UTF-8"));
                 String hash = bytesToHash(digest.digest());
                 matcher.appendReplacement(result, hash);
-                protectionMap.put(hash, protectedMarkup);
-                this.hashList.add(hash);
+                c_protectionMap.put(hash, protectedMarkup);
+                m_hashList.add(hash);
             }
             catch (NoSuchAlgorithmException e)
             {
+                // FIXME: Should log properly
+                e.printStackTrace();
+            }
+            catch (UnsupportedEncodingException e)
+            {
+                // FIXME: Auto-generated catch block
                 e.printStackTrace();
             }
         }
