@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.parser;
 
@@ -52,7 +53,6 @@ import com.ecyrd.jspwiki.render.RenderingManager;
  *  heart and soul of JSPWiki : make sure you test properly anything that is added,
  *  or else it breaks down horribly.
  *
- *  @author Janne Jalkanen
  *  @since  2.4
  */
 public class JSPWikiMarkupParser
@@ -96,18 +96,18 @@ public class JSPWikiMarkupParser
     private boolean        m_isPreBlock   = false;
 
     /** Contains style information, in multiple forms. */
-    private Stack          m_styleStack   = new Stack();
+    private Stack<Boolean> m_styleStack   = new Stack<Boolean>();
 
      // general list handling
     private int            m_genlistlevel = 0;
-    private StringBuffer   m_genlistBulletBuffer = new StringBuffer(10);  // stores the # and * pattern
+    private StringBuilder  m_genlistBulletBuffer = new StringBuilder(10);  // stores the # and * pattern
     private boolean        m_allowPHPWikiStyleLists = true;
 
 
     private boolean        m_isOpenParagraph = false;
 
     /** Keeps image regexp Patterns */
-    private List           m_inlineImagePatterns;
+    private List<Pattern>  m_inlineImagePatterns;
 
     /** Parser for extended link functionality. */
     private LinkParser     m_linkParser = new LinkParser();
@@ -115,10 +115,13 @@ public class JSPWikiMarkupParser
     private PatternMatcher m_inlineMatcher = new Perl5Matcher();
 
     /** Keeps track of any plain text that gets put in the Text nodes */
-    private StringBuffer   m_plainTextBuf = new StringBuffer(20);
+    private StringBuilder  m_plainTextBuf = new StringBuilder(20);
 
     private Element        m_currentElement;
 
+    /** Keep track of duplicate header names.  */
+    private Map<String, Integer>   m_titleSectionCounter = new HashMap<String, Integer>();
+
     /**
      *  This property defines the inline image pattern.  It's current value
      *  is jspwiki.translatorReader.inlinePattern
@@ -163,7 +166,7 @@ public class JSPWikiMarkupParser
 
     private PatternCompiler        m_compiler = new Perl5Compiler();
 
-    static final String WIKIWORD_REGEX = "(^|[[:^alnum:]]+)([[:upper:]]+[[:lower:]]+[[:upper:]]+[[:alnum:]]*|(http://|https://|mailto:)([A-Za-z0-9_/\\.\\+\\?\\#\\-\\@=&;]+))";
+    static final String WIKIWORD_REGEX = "(^|[[:^alnum:]]+)([[:upper:]]+[[:lower:]]+[[:upper:]]+[[:alnum:]]*|(http://|https://|mailto:)([A-Za-z0-9_/\\.\\+\\?\\#\\-\\@=&;~%]+))";
 
     private PatternMatcher         m_camelCaseMatcher = new Perl5Matcher();
     private Pattern                m_camelCasePattern;
@@ -185,7 +188,7 @@ public class JSPWikiMarkupParser
      *  This array is sorted during class load, so you can just dump
      *  here whatever you want in whatever order you want.
      */
-    static final String[] c_externalLinks = {
+    static final String[] EXTERNAL_LINKS = {
         "http:", "ftp:", "https:", "mailto:",
         "news:", "file:", "rtsp:", "mms:", "ldap:",
         "gopher:", "nntp:", "telnet:", "wais:",
@@ -221,15 +224,18 @@ public class JSPWikiMarkupParser
      *  This Comparator is used to find an external link from c_externalLinks.  It
      *  checks if the link starts with the other arraythingie.
      */
-    private static Comparator c_startingComparator = new StartingComparator();
+    private static Comparator<String> c_startingComparator = new StartingComparator();
 
     static
     {
-        Arrays.sort( c_externalLinks );
+        Arrays.sort( EXTERNAL_LINKS );
     }
 
     /**
      *  Creates a markup parser.
+     *  
+     *  @param context The WikiContext which controls the parsing
+     *  @param in Where the data is read from.
      */
     public JSPWikiMarkupParser( WikiContext context, Reader in )
     {
@@ -243,20 +249,21 @@ public class JSPWikiMarkupParser
      */
 
     // FIXME: parsers should be pooled for better performance.
+    @SuppressWarnings("unchecked")
     private void initialize()
     {
         PatternCompiler compiler         = new GlobCompiler();
-        List            compiledpatterns;
+        List<Pattern>   compiledpatterns;
 
         //
         //  We cache compiled patterns in the engine, since their creation is
         //  really expensive
         //
-        compiledpatterns = (List)m_engine.getAttribute( INLINE_IMAGE_PATTERNS );
+        compiledpatterns = (List<Pattern>)m_engine.getAttribute( INLINE_IMAGE_PATTERNS );
 
         if( compiledpatterns == null )
         {
-            compiledpatterns = new ArrayList(20);
+            compiledpatterns = new ArrayList<Pattern>(20);
             Collection ptrns = getImagePatterns( m_engine );
 
             //
@@ -381,13 +388,15 @@ public class JSPWikiMarkupParser
     /**
      *  Figure out which image suffixes should be inlined.
      *  @return Collection of Strings with patterns.
+     *  
+     *  @param engine The WikiEngine from which the patterns are read.
      */
 
     // FIXME: Does not belong here; should be elsewhere
     public static Collection getImagePatterns( WikiEngine engine )
     {
         Properties props    = engine.getWikiProperties();
-        ArrayList  ptrnlist = new ArrayList();
+        ArrayList<String>  ptrnlist = new ArrayList<String>();
 
         for( Enumeration e = props.propertyNames(); e.hasMoreElements(); )
         {
@@ -510,6 +519,8 @@ public class JSPWikiMarkupParser
         {
             type = EMPTY;
         }
+        ResourceBundle rb = m_context.getBundle(InternationalizationManager.CORE_BUNDLE);
+        Object[] args = { link };
 
         switch(type)
         {
@@ -519,7 +530,7 @@ public class JSPWikiMarkupParser
 
             case EDIT:
                 el = createAnchor( EDIT, m_context.getURL(WikiContext.EDIT,link), text, "" );
-                el.setAttribute("title","Create '"+link+"'");
+                el.setAttribute("title", MessageFormat.format( rb.getString( "markupparser.link.create" ), args ) );
                 break;
 
             case EMPTY:
@@ -601,6 +612,7 @@ public class JSPWikiMarkupParser
                     el.setAttribute("alt","(info)");
 
                     el = new Element("a").setAttribute("href",infolink).addContent(el);
+                    el.setAttribute("class","infolink");
                 }
                 else
                 {
@@ -637,19 +649,21 @@ public class JSPWikiMarkupParser
      *  Figures out if a link is an off-site link.  This recognizes
      *  the most common protocols by checking how it starts.
      *
+     *  @param link The link to check.
+     *  @return true, if this is a link outside of this wiki.
      *  @since 2.4
      */
 
     public static boolean isExternalLink( String link )
     {
-        int idx = Arrays.binarySearch( c_externalLinks, link,
+        int idx = Arrays.binarySearch( EXTERNAL_LINKS, link,
                                        c_startingComparator );
 
         //
         //  We need to check here once again; otherwise we might
         //  get a match for something like "h".
         //
-        if( idx >= 0 && link.startsWith(c_externalLinks[idx]) ) return true;
+        if( idx >= 0 && link.startsWith(EXTERNAL_LINKS[idx]) ) return true;
 
         return false;
     }
@@ -737,7 +751,11 @@ public class JSPWikiMarkupParser
 
 
     /**
-     *  Writes HTML for error message.
+     *  Writes HTML for error message.  Does not add it to the document, you
+     *  have to do it yourself.
+     *  
+     *  @param error The error string.
+     *  @return An Element containing the error.
      */
 
     public static Element makeError( String error )
@@ -766,7 +784,7 @@ public class JSPWikiMarkupParser
             //  calling makeCamelCaseLink() is to call this routine.
             //
 
-            m_plainTextBuf = new StringBuffer(20);
+            m_plainTextBuf = new StringBuilder(20);
 
             try
             {
@@ -864,7 +882,7 @@ public class JSPWikiMarkupParser
                 // Sometimes it's possible that illegal XML chars is added to the data.
                 // Here we make sure it does not stop parsing.
                 //
-                m_currentElement.addContent( makeError(e.getMessage()) );
+                m_currentElement.addContent( makeError(cleanupSuspectData( e.getMessage() )) );
             }
         }
 
@@ -876,11 +894,11 @@ public class JSPWikiMarkupParser
      *  entities that are already escaped).
      *
      *  @param buf
-     *  @return
+     *  @return An escaped string.
      */
     private String escapeHTMLEntities(String buf)
     {
-        StringBuffer tmpBuf = new StringBuffer( buf.length() + 20 );
+        StringBuilder tmpBuf = new StringBuilder( buf.length() + 20 );
 
         for( int i = 0; i < buf.length(); i++ )
         {
@@ -900,19 +918,47 @@ public class JSPWikiMarkupParser
             }
             else if( ch == '&' )
             {
-                for( int j = (i < buf.length()-1 ) ? i+1 : i; j < buf.length(); j++ )
+                //
+                //  If the following is an XML entity reference (&#.*;) we'll
+                //  leave it as it is; otherwise we'll replace it with an &amp;
+                //
+                
+                boolean isEntity = false;
+                StringBuilder entityBuf = new StringBuilder();
+                
+                if( i < buf.length() -1 )
                 {
-                    int ch2 = buf.charAt(j);
-                    if( ch2 == ';' )
+                    for( int j = i; j < buf.length(); j++ )
                     {
-                        tmpBuf.append(ch);
-                        break;
-                    }
-                    if( ch2 != '#' && !Character.isLetterOrDigit( (char)ch2) )
-                    {
-                        tmpBuf.append("&amp;"); break;
+                        char ch2 = buf.charAt(j);
+                        
+                        if( Character.isLetterOrDigit( ch2 ) || (ch2 == '#' && j == i+1) || ch2 == ';' || ch2 == '&' )
+                        {
+                            entityBuf.append(ch2);
+                            
+                            if( ch2 == ';' )
+                            {
+                                isEntity = true;
+                                break;
+                            }
+                        }
+                        else
+                        {
+                            break;
+                        }
                     }
                 }
+                
+                if( isEntity ) 
+                {
+                    tmpBuf.append( entityBuf );
+                    i = i + entityBuf.length() - 1;
+                }
+                else 
+                {
+                    tmpBuf.append("&amp;");
+                }
+                
             }
             else
             {
@@ -999,7 +1045,7 @@ public class JSPWikiMarkupParser
     private String readUntil( String endChars )
         throws IOException
     {
-        StringBuffer sb = new StringBuffer( 80 );
+        StringBuilder sb = new StringBuilder( 80 );
         int ch = nextToken();
 
         while( ch != -1 )
@@ -1034,7 +1080,7 @@ public class JSPWikiMarkupParser
     private String readWhile( String endChars )
         throws IOException
     {
-        StringBuffer sb = new StringBuffer( 80 );
+        StringBuilder sb = new StringBuilder( 80 );
         int ch = nextToken();
 
         while( ch != -1 )
@@ -1077,18 +1123,34 @@ public class JSPWikiMarkupParser
      *  Modifies the "hd" parameter to contain proper values.  Because
      *  an "id" tag may only contain [a-zA-Z0-9:_-], we'll replace the
      *  % after url encoding with '_'.
+     *  <p>
+     *  Counts also duplicate headings (= headings with similar name), and
+     *  attaches a counter.
      */
-    // FIXME: This method should probably be public and in an util class somewhere
     private String makeHeadingAnchor( String baseName, String title, Heading hd )
     {
         hd.m_titleText = title;
         title = MarkupParser.wikifyLink( title );
+
         hd.m_titleSection = m_engine.encodeName(title);
+        
+        if( m_titleSectionCounter.containsKey( hd.m_titleSection ) )
+        {
+            Integer count = m_titleSectionCounter.get( hd.m_titleSection );
+            count = count + 1;
+            m_titleSectionCounter.put( hd.m_titleSection, count );
+            hd.m_titleSection += "-" + count;
+        }
+        else
+        {
+            m_titleSectionCounter.put( hd.m_titleSection, 1 );
+        }
+
         hd.m_titleAnchor = "section-"+m_engine.encodeName(baseName)+
                            "-"+hd.m_titleSection;
-
         hd.m_titleAnchor = hd.m_titleAnchor.replace( '%', '_' );
         hd.m_titleAnchor = hd.m_titleAnchor.replace( '/', '_' );
+        
         return hd.m_titleAnchor;
     }
 
@@ -1116,11 +1178,12 @@ public class JSPWikiMarkupParser
     }
 
     /**
-     *  Returns XHTML for the start of the heading.  Also sets the
-     *  line-end emitter.
-     *  @param level
+     *  Returns XHTML for the heading. 
+     *  
+     *  @param level The level of the heading.  @see Heading
      *  @param title the title for the heading
      *  @param hd a List to which heading should be added
+     *  @return An Element containing the heading
      */
     public Element makeHeading( int level, String title, Heading hd )
     {
@@ -1215,7 +1278,7 @@ public class JSPWikiMarkupParser
      *  What a crappy problem.
      *
      * @param url
-     * @return
+     * @return An anchor Element containing the link.
      */
     private Element makeDirectURILink( String url )
     {
@@ -1294,7 +1357,7 @@ public class JSPWikiMarkupParser
 
         if( !m_parseAccessRules ) return m_currentElement;
         Acl acl;
-        WikiPage          page = m_context.getPage();
+        WikiPage          page = m_context.getRealPage();
         // UserDatabase      db = m_context.getEngine().getUserDatabase();
 
         if( ruleLine.startsWith( "{" ) )
@@ -1380,7 +1443,7 @@ public class JSPWikiMarkupParser
     {
         ResourceBundle rb = m_context.getBundle(InternationalizationManager.CORE_BUNDLE);
         
-        StringBuffer sb = new StringBuffer(linktext.length()+80);
+        StringBuilder sb = new StringBuilder(linktext.length()+80);
 
         if( isAccessRule( linktext ) )
         {
@@ -1399,10 +1462,16 @@ public class JSPWikiMarkupParser
                 PluginContent pluginContent = m_engine.getPluginManager().parsePluginLine( m_context,
                                                                                            linktext,
                                                                                            pos );
+                //
+                //  This might sometimes fail, especially if there is something which looks
+                //  like a plugin invocation but is really not.
+                //
+                if( pluginContent != null )
+                {
+                    addElement( pluginContent );
 
-                addElement( pluginContent );
-
-                pluginContent.executeParse( m_context );
+                    pluginContent.executeParse( m_context );
+                }
             }
             catch( PluginException e )
             {
@@ -1550,7 +1619,7 @@ public class JSPWikiMarkupParser
                     String matchedLink;
                     if( (matchedLink = linkExists( linkref )) != null )
                     {
-                        String sectref = "section-"+m_engine.encodeName(matchedLink)+"-"+namedSection;
+                        String sectref = "section-"+m_engine.encodeName(matchedLink)+"-"+wikifyLink(namedSection);
                         sectref = sectref.replace('%', '_');
                         makeLink( READ, matchedLink, linktext, sectref, link.getAttributes() );
                     }
@@ -1873,11 +1942,11 @@ public class JSPWikiMarkupParser
      *  Reads the stream until the next EOL or EOF.  Note that it will also read the
      *  EOL from the stream.
      */
-    private StringBuffer readUntilEOL()
+    private StringBuilder readUntilEOL()
         throws IOException
     {
         int ch;
-        StringBuffer buf = new StringBuffer( 256 );
+        StringBuilder buf = new StringBuilder( 256 );
 
         while( true )
         {
@@ -2046,7 +2115,7 @@ public class JSPWikiMarkupParser
              for( ; m_genlistlevel > numEqualBullets; m_genlistlevel-- )
              {
                  popElement( getListType( m_genlistBulletBuffer.charAt(m_genlistlevel-1) ) );
-                 if( m_genlistlevel > 0 )
+                 if( m_genlistlevel > numBullets )
                  {
                      popElement("li");
                  }
@@ -2110,7 +2179,7 @@ public class JSPWikiMarkupParser
     private Element handleOpenbracket()
         throws IOException
     {
-        StringBuffer sb = new StringBuffer(40);
+        StringBuilder sb = new StringBuilder(40);
         int pos = getPosition();
         int ch = nextToken();
         boolean isPlugin = false;
@@ -2207,7 +2276,7 @@ public class JSPWikiMarkupParser
     private String readBraceContent( char opening, char closing )
         throws IOException
     {
-        StringBuffer sb = new StringBuffer(40);
+        StringBuilder sb = new StringBuilder(40);
         int braceLevel = 1;
         int ch;
         while(( ch = nextToken() ) != -1 )
@@ -2237,7 +2306,7 @@ public class JSPWikiMarkupParser
     /**
      *  Handles constructs of type %%(style) and %%class
      * @param newLine
-     * @return
+     * @return An Element containing the div or span, depending on the situation.
      * @throws IOException
      */
     private Element handleDiv( boolean newLine )
@@ -2285,7 +2354,7 @@ public class JSPWikiMarkupParser
 
                 try
                 {
-                    Boolean isSpan = (Boolean)m_styleStack.pop();
+                    Boolean isSpan = m_styleStack.pop();
 
                     if( isSpan == null )
                     {
@@ -2312,12 +2381,26 @@ public class JSPWikiMarkupParser
             //
             //  Check if there is an attempt to do something nasty
             //
-            style = StringEscapeUtils.unescapeHtml(style);
-            if( style != null && style.indexOf("javascript:") != -1 )
+            
+            try
             {
-                log.debug("Attempt to output javascript within CSS:"+style);
+                style = StringEscapeUtils.unescapeHtml(style);
+                if( style != null && style.indexOf("javascript:") != -1 )
+                {
+                    log.debug("Attempt to output javascript within CSS:"+style);
+                    ResourceBundle rb = m_context.getBundle(InternationalizationManager.CORE_BUNDLE);
+                    return addElement( makeError( rb.getString( "markupparser.error.javascriptattempt" ) ) );
+                }
+            }
+            catch( NumberFormatException e )
+            {
+                //
+                //  If there are unknown entities, we don't want the parser to stop.
+                //
                 ResourceBundle rb = m_context.getBundle(InternationalizationManager.CORE_BUNDLE);
-                return addElement( makeError( rb.getString( "markupparser.error.javascriptattempt" ) ) );
+                Object[] args = { e.getMessage() };
+                String msg = MessageFormat.format( rb.getString( "markupparser.error.parserfailure"), args );
+                return addElement( makeError( msg ) );
             }
 
             //
@@ -2552,8 +2635,22 @@ public class JSPWikiMarkupParser
                 m_istable = false;
             }
 
-            int skip = parseToken( ch );
-
+            int skip = IGNORE;
+            
+            //
+            //  Do the actual parsing and catch any errors.
+            //
+            try
+            {
+                skip = parseToken( ch );
+            }
+            catch( IllegalDataException e )
+            {
+                log.info("Page "+m_context.getPage().getName()+" contains data which cannot be added to DOM tree: "+e.getMessage());
+                
+                makeError("Error: "+cleanupSuspectData(e.getMessage()) );
+            }
+            
             //
             //   The idea is as follows:  If the handler method returns
             //   an element (el != null), it is assumed that it has been
@@ -2584,18 +2681,43 @@ public class JSPWikiMarkupParser
         popElement("domroot");
     }
 
-    public static final int CHARACTER = 0;
-    public static final int ELEMENT   = 1;
-    public static final int IGNORE    = 2;
+    private String cleanupSuspectData( String s )
+    {
+        StringBuilder sb = new StringBuilder( s.length() );
+        
+        for( int i = 0; i < s.length(); i++ )
+        {
+            char c = s.charAt(i);
+            
+            if( Verifier.isXMLCharacter( c ) ) sb.append( c );
+            else sb.append( "0x"+Integer.toString(c,16).toUpperCase() );
+        }
+        
+        return sb.toString();
+    }
+    
+    /** The token is a plain character. */
+    protected static final int CHARACTER = 0;
+    
+    /** The token is a wikimarkup element. */
+    protected static final int ELEMENT   = 1;
+    
+    /** The token is to be ignored. */
+    protected static final int IGNORE    = 2;
 
     /**
      *  Return CHARACTER, if you think this was a plain character; ELEMENT, if
      *  you think this was a wiki markup element, and IGNORE, if you think
      *  we should ignore this altogether.
+     *  <p>
+     *  To add your own MarkupParser, you can override this method, but it
+     *  is recommended that you call super.parseToken() as well to gain advantage
+     *  of JSPWiki's own markup.  You can call it at the start of your own
+     *  parseToken() or end - it does not matter.
      *
-     * @param ch
+     * @param ch The character under investigation
      * @return {@link #ELEMENT}, {@link #CHARACTER} or {@link #IGNORE}.
-     * @throws IOException
+     * @throws IOException If parsing fails.
      */
     protected int parseToken( int ch )
         throws IOException
@@ -2762,6 +2884,13 @@ public class JSPWikiMarkupParser
         return el != null ? ELEMENT : CHARACTER;
     }
 
+    /**
+     *  Parses the entire document from the Reader given in the constructor or
+     *  set by {@link #setInputReader(Reader)}.
+     *  
+     *  @return A WikiDocument, ready to be passed to the renderer.
+     *  @throws IOException If parsing cannot be accomplished.
+     */
     public WikiDocument parse()
         throws IOException
     {
@@ -2771,18 +2900,10 @@ public class JSPWikiMarkupParser
         Element rootElement = new Element("domroot");
 
         d.setRootElement( rootElement );
-        try
-        {
-            fillBuffer( rootElement );
 
-            paragraphify(rootElement);
+        fillBuffer( rootElement );
 
-        }
-        catch( IllegalDataException e )
-        {
-            log.error("Page "+m_context.getName()+" contained something that cannot be added in the DOM tree",e);
-            throw new IOException("Illegal page data: "+e.getMessage());
-        }
+        paragraphify(rootElement);
 
         return d;
     }
@@ -2801,7 +2922,7 @@ public class JSPWikiMarkupParser
 
         if( rootElement.getChild("p") != null )
         {
-            ArrayList ls = new ArrayList();
+            ArrayList<Content> ls = new ArrayList<Content>();
             int idxOfFirstContent = 0;
             int count = 0;
 
@@ -2852,17 +2973,12 @@ public class JSPWikiMarkupParser
      *  returns null.  Otherwise just like the normal Comparator
      *  for strings.
      *
-     *  @author jalkanen
-     *
      *  @since
      */
-    private static class StartingComparator implements Comparator
+    private static class StartingComparator implements Comparator<String>
     {
-        public int compare( Object arg0, Object arg1 )
+        public int compare( String s1, String s2 )
         {
-            String s1 = (String)arg0;
-            String s2 = (String)arg1;
-
             if( s1.length() > s2.length() )
             {
                 if( s1.startsWith(s2) && s2.length() > 1 ) return 0;
