@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2006 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 
 package com.ecyrd.jspwiki.parser;
@@ -269,14 +270,10 @@ public class LinkParser
                 catch( ParseException pe )
                 {
                     log.warn("syntax error parsing link attributes '"+attribs+"': " + pe.getMessage());
-                //  throw new RuntimeException("syntax error parsing link attributes '"
-                //          +attribs+"': " + pe.getMessage()); // TEMP
                 }
                 catch( NoSuchElementException nse )
                 {
                     log.warn("expected more tokens while parsing link attributes '" + attribs + "'");
-                //  throw new RuntimeException("expected more tokens while parsing link attributes '"
-                //          + attribs + "'"); // TEMP
                 }
             }
 
@@ -284,8 +281,6 @@ public class LinkParser
         catch( Exception e )
         {
             log.warn( e.getClass().getName() + " thrown by link parser: " + e.getMessage() );
-        //  throw new RuntimeException( e.getClass().getName() + " thrown by link parser: "
-        //          + e.getMessage() );
         }
 
         return link;
@@ -349,10 +344,10 @@ public class LinkParser
      */
     public static class Link
     {
-        private String m_text;
-        private String m_ref = null;
-        private int    m_interwikiPoint = -1;
-        private List   m_attribs = null;
+        private String            m_text;
+        private String            m_ref = null;
+        private int               m_interwikiPoint = -1;
+        private List<Attribute>   m_attribs = null;
 
         /**
          *  Create a new Link with text but no reference.
@@ -416,7 +411,6 @@ public class LinkParser
                 throw new ParseException("null link reference value");
             }
             m_ref = ref;
-            m_interwikiPoint = m_ref.indexOf(':');
         }
 
         /**
@@ -448,6 +442,10 @@ public class LinkParser
          */
         public boolean isInterwikiLink()
         {
+            if( !hasReference() ) m_ref = m_text;
+
+            m_interwikiPoint = m_ref.indexOf(':');
+
             return m_interwikiPoint != -1;
         }
 
@@ -462,9 +460,12 @@ public class LinkParser
          */
         public String getExternalWiki()
         {
-            return m_interwikiPoint != -1
-                    ? m_ref.substring( 0, m_interwikiPoint )
-                    : null ;
+            if( isInterwikiLink() )
+            {
+                return m_ref.substring( 0, m_interwikiPoint );
+            }
+            
+            return null;
         }
 
         /** 
@@ -478,9 +479,12 @@ public class LinkParser
          */
         public String getExternalWikiPage()
         {
-            return m_interwikiPoint != -1
-                    ? m_ref.substring( m_interwikiPoint+1 )
-                    : null ;
+            if( isInterwikiLink() )
+            {
+                return m_ref.substring( m_interwikiPoint+1 );
+            }
+            
+            return null;
         }
 
         /**
@@ -504,7 +508,7 @@ public class LinkParser
         {
             if( m_attribs == null )
             {
-                m_attribs = new ArrayList();
+                m_attribs = new ArrayList<Attribute>();
             }
             m_attribs.add(attr);
         }
