@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2004 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.plugin;
 
@@ -38,26 +39,37 @@ import com.ecyrd.jspwiki.parser.MarkupParser;
 /**
  *  Provides a handler for bug reports.  Still under construction.
  *
+ *  <p>Parameters : </p>
  *  <ul>
- *   <li>"title" = title of the bug.  This is required.  If it is empty (as in "")
- *       it is a signal to the handler to return quietly.</li>
+ *  <li><b>title</b> -  title of the bug, this is required.  If it is empty (as in "")  it is a signal to the handler to return quietly.</li>
+ *  <li><b>description</b> - description of the bug.</li>
+ *  <li><b>version</b> - version</li>
+ *  <li><b>map</b> - I have no idea </li>
+ *  <li><b>page</b> - The name of the page to be created for this bug report </li>
  *  </ul>
  *
- *  @author Janne Jalkanen
  */
 public class BugReportHandler
     implements WikiPlugin
 {
     private static Logger log = Logger.getLogger( BugReportHandler.class );
 
-    public static final String TITLE          = "title";
-    public static final String DESCRIPTION    = "description";
-    public static final String VERSION        = "version";
-    public static final String MAPPINGS       = "map";
-    public static final String PAGE           = "page";
+    /** Parameter name for setting the title.  Value is <tt>{@value}</tt>. */
+    public static final String PARAM_TITLE          = "title";
+    /** Parameter name for setting the description.  Value is <tt>{@value}</tt>. */
+    public static final String PARAM_DESCRIPTION    = "description";
+    /** Parameter name for setting the version.  Value is <tt>{@value}</tt>. */
+    public static final String PARAM_VERSION        = "version";
+    /** Parameter name for setting the map.  Value is <tt>{@value}</tt>. */
+    public static final String PARAM_MAPPINGS       = "map";
+    /** Parameter name for setting the page.  Value is <tt>{@value}</tt>. */
+    public static final String PARAM_PAGE           = "page";
 
-    public static final String DEFAULT_DATEFORMAT = "dd-MMM-yyyy HH:mm:ss zzz";
+    private static final String DEFAULT_DATEFORMAT = "dd-MMM-yyyy HH:mm:ss zzz";
 
+    /**
+     *  {@inheritDoc}
+     */
     public String execute( WikiContext context, Map params )
         throws PluginException
     {
@@ -68,9 +80,9 @@ public class BugReportHandler
         SimpleDateFormat format = new SimpleDateFormat( DEFAULT_DATEFORMAT );
         ResourceBundle rb = context.getBundle(WikiPlugin.CORE_PLUGINS_RESOURCEBUNDLE);
 
-        title       = (String) params.get( TITLE );
-        description = (String) params.get( DESCRIPTION );
-        version     = (String) params.get( VERSION );
+        title       = (String) params.get( PARAM_TITLE );
+        description = (String) params.get( PARAM_DESCRIPTION );
+        version     = (String) params.get( PARAM_VERSION );
 
         Principal wup = context.getCurrentUser();
 
@@ -85,7 +97,7 @@ public class BugReportHandler
         if( description == null ) description = "";
         if( version == null ) version = "unknown";
 
-        Properties mappings = parseMappings( (String) params.get( MAPPINGS ) );
+        Properties mappings = parseMappings( (String) params.get( PARAM_MAPPINGS ) );
 
         //
         //  Start things
@@ -101,9 +113,9 @@ public class BugReportHandler
             //
             //  Outputting of basic data
             //
-            out.println("|"+mappings.getProperty(TITLE,"Title")+"|"+title);
+            out.println("|"+mappings.getProperty(PARAM_TITLE,"Title")+"|"+title);
             out.println("|"+mappings.getProperty("date","Date")+"|"+format.format(d));
-            out.println("|"+mappings.getProperty(VERSION,"Version")+"|"+version);
+            out.println("|"+mappings.getProperty(PARAM_VERSION,"Version")+"|"+version);
             if( submitter != null )
             {
                 out.println("|"+mappings.getProperty("submitter","Submitter")+
@@ -117,11 +129,11 @@ public class BugReportHandler
             {
                 Map.Entry entry = (Map.Entry) i.next();
 
-                if( entry.getKey().equals( TITLE ) ||
-                    entry.getKey().equals( DESCRIPTION ) ||
-                    entry.getKey().equals( VERSION ) ||
-                    entry.getKey().equals( MAPPINGS ) ||
-                    entry.getKey().equals( PAGE ) ||
+                if( entry.getKey().equals( PARAM_TITLE ) ||
+                    entry.getKey().equals( PARAM_DESCRIPTION ) ||
+                    entry.getKey().equals( PARAM_VERSION ) ||
+                    entry.getKey().equals( PARAM_MAPPINGS ) ||
+                    entry.getKey().equals( PARAM_PAGE ) ||
                     entry.getKey().toString().startsWith("_") )
                 {
                     // Ignore this
@@ -151,7 +163,7 @@ public class BugReportHandler
             //  Now create a new page for this bug report
             //
             String pageName = findNextPage( context, title,
-                                            (String)params.get( PAGE ) );
+                                            (String)params.get( PARAM_PAGE ) );
 
             WikiPage newPage = new WikiPage( context.getEngine(), pageName );
             WikiContext newContext = (WikiContext)context.clone();
