@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.plugin;
 
@@ -28,24 +29,46 @@ import java.util.*;
 /**
  *  Inserts page contents.  Muchos thanks to Scott Hurlbert for the initial code.
  *
+ *  <p>Parameters : </p>
+ *  <ul>
+ *  <li><b>page</b> - the name of the page to be inserted</li>
+ *  <li><b>style</b> - the style to use</li>
+ *  <li><b>maxlength</b> - the maximum length of the page to be inserted (page contents)</li>
+ *  <li><b>class</b> - the class to use</li>
+ *  <li><b>section</b> - the section of the page that has to be inserted (separated by "----"</li>
+ *  <li><b>default</b> - the text to insert if the requested page does not exist</li>
+ *  </ul>
+ *  
  *  @since 2.1.37
  *  @author Scott Hurlbert
- *  @author Janne Jalkanen
  */
 public class InsertPage
     implements WikiPlugin
 {
+    /** Parameter name for setting the page.  Value is <tt>{@value}</tt>. */
     public static final String PARAM_PAGENAME  = "page";
+    /** Parameter name for setting the style.  Value is <tt>{@value}</tt>. */
     public static final String PARAM_STYLE     = "style";
+    /** Parameter name for setting the maxlength.  Value is <tt>{@value}</tt>. */
     public static final String PARAM_MAXLENGTH = "maxlength";
+    /** Parameter name for setting the class.  Value is <tt>{@value}</tt>. */
     public static final String PARAM_CLASS     = "class";
+    /** Parameter name for setting the section.  Value is <tt>{@value}</tt>. */
     public static final String PARAM_SECTION   = "section";
+    /** Parameter name for setting the default.  Value is <tt>{@value}</tt>. */
     public static final String PARAM_DEFAULT   = "default";
 
     private static final String DEFAULT_STYLE = "";
 
+    /** This attribute is stashed in the WikiContext to make sure that we don't
+     *  have circular references.
+     */
     public static final String ATTR_RECURSE    = "com.ecyrd.jspwiki.plugin.InsertPage.recurseCheck";
     
+    /**
+     *  {@inheritDoc}
+     */
+    @SuppressWarnings("unchecked")
     public String execute( WikiContext context, Map params )
         throws PluginException
     {
@@ -77,7 +100,7 @@ public class InsertPage
                 //  Check for recursivity
                 //
                 
-                List previousIncludes = (List)context.getVariable( ATTR_RECURSE );
+                List<String> previousIncludes = (List)context.getVariable( ATTR_RECURSE );
                 
                 if( previousIncludes != null )
                 {
@@ -88,7 +111,7 @@ public class InsertPage
                 }
                 else
                 {
-                    previousIncludes = new ArrayList();
+                    previousIncludes = new ArrayList<String>();
                 }
                
                 previousIncludes.add( page.getName() );
