@@ -1,21 +1,22 @@
 /*
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.plugin;
 
@@ -40,6 +41,8 @@ import com.ecyrd.jspwiki.*;
 import com.ecyrd.jspwiki.auth.AuthorizationManager;
 import com.ecyrd.jspwiki.auth.permissions.PagePermission;
 import com.ecyrd.jspwiki.parser.PluginContent;
+import com.ecyrd.jspwiki.preferences.Preferences;
+import com.ecyrd.jspwiki.preferences.Preferences.TimeFormat;
 import com.ecyrd.jspwiki.providers.ProviderException;
 
 /**
@@ -48,17 +51,17 @@ import com.ecyrd.jspwiki.providers.ProviderException;
  *  <p>%p - Page name</p>
  *  <p>Parameters:</p>
  *  <ul>
- *    <li>page - which page is used to do the blog; default is the current page.</li>
- *    <li>entryFormat - how to display the date on pages, using the J2SE SimpleDateFormat
+ *    <li><b>page</b> - which page is used to do the blog; default is the current page.</li>
+ *    <li><b>entryFormat</b> - how to display the date on pages, using the J2SE SimpleDateFormat
  *       syntax. Defaults to the current locale's DateFormat.LONG format
  *       for the date, and current locale's DateFormat.SHORT for the time.
  *       Thus, for the US locale this will print dates similar to
  *       this: September 4, 2005 11:54 PM</li>
- *    <li>days - how many days the weblog aggregator should show.  If set to
+ *    <li><b>days</b> - how many days the weblog aggregator should show.  If set to
  *      "all", shows all pages.</li>
- *    <li>pageformat - What the entry pages should look like.</li>
- *    <li>startDate - Date when to start.  Format is "ddMMyy."</li>
- *    <li>maxEntries - How many entries to show at most.</li>
+ *    <li><b>pageformat</b> - What the entry pages should look like.</li>
+ *    <li><b>startDate</b> - Date when to start.  Format is "ddMMyy."</li>
+ *    <li><b>maxEntries</b> - How many entries to show at most.</li>
  *  </ul>
  *  <p>The "days" and "startDate" can also be sent in HTTP parameters,
  *  and the names are "weblog.days" and "weblog.startDate", respectively.</p>
@@ -75,31 +78,47 @@ public class WeblogPlugin
     implements WikiPlugin, ParserStagePlugin
 {
     private static Logger     log = Logger.getLogger(WeblogPlugin.class);
-    private static final DateFormat DEFAULT_ENTRYFORMAT
-                                = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.SHORT);
-    private static final Pattern headingPattern;
+    private static final Pattern HEADINGPATTERN;
 
     /** How many days are considered by default.  Default value is {@value} */
-    public static final int     DEFAULT_DAYS = 7;
-    public static final String  DEFAULT_PAGEFORMAT = "%p_blogentry_";
+    private static final int     DEFAULT_DAYS = 7;
+    private static final String  DEFAULT_PAGEFORMAT = "%p_blogentry_";
 
-    public static final String  DEFAULT_DATEFORMAT = "ddMMyy";
+    /** The default date format used in the blog entry page names. */
+    public static final String   DEFAULT_DATEFORMAT = "ddMMyy";
 
+    /** Parameter name for the startDate.  Value is <tt>{@value}</tt>. */
     public static final String  PARAM_STARTDATE    = "startDate";
+    /** Parameter name for the entryFormat.  Value is <tt>{@value}</tt>. */
     public static final String  PARAM_ENTRYFORMAT  = "entryFormat";
+    /** Parameter name for the days.  Value is <tt>{@value}</tt>. */
     public static final String  PARAM_DAYS         = "days";
+    /** Parameter name for the allowComments.  Value is <tt>{@value}</tt>. */
     public static final String  PARAM_ALLOWCOMMENTS = "allowComments";
+    /** Parameter name for the maxEntries.  Value is <tt>{@value}</tt>. */
     public static final String  PARAM_MAXENTRIES   = "maxEntries";
+    /** Parameter name for the page.  Value is <tt>{@value}</tt>. */
     public static final String  PARAM_PAGE         = "page";
 
+    /** The attribute which is stashed to the WikiPage attributes to check if a page
+     *  is a weblog or not. You may check for its presence.
+     */
     public static final String  ATTR_ISWEBLOG      = "weblogplugin.isweblog";
 
     static
     {
         // This is a pretty ugly, brute-force regex. But it will do for now...
-        headingPattern = Pattern.compile("(<h[1-4].*>)(.*)(</h[1-4]>)", Pattern.CASE_INSENSITIVE);
+        HEADINGPATTERN = Pattern.compile("(<h[1-4].*>)(.*)(</h[1-4]>)", Pattern.CASE_INSENSITIVE);
     }
 
+    /**
+     *  Create an entry name based on the blogname, a date, and an entry number.
+     *  
+     *  @param pageName Name of the blog
+     *  @param date The date (in ddMMyy format)
+     *  @param entryNum The entry number.
+     *  @return A formatted page name.
+     */
     public static String makeEntryPage( String pageName,
                                         String date,
                                         String entryNum )
@@ -107,16 +126,33 @@ public class WeblogPlugin
         return TextUtil.replaceString(DEFAULT_PAGEFORMAT,"%p",pageName)+date+"_"+entryNum;
     }
 
+    /**
+     *  Return just the basename for entires without date and entry numebr.
+     *  
+     *  @param pageName The name of the blog.
+     *  @return A formatted name.
+     */
     public static String makeEntryPage( String pageName )
     {
         return TextUtil.replaceString(DEFAULT_PAGEFORMAT,"%p",pageName);
     }
 
+    /**
+     *  Returns the entry page without the entry number.
+     *  
+     *  @param pageName Blog name.
+     *  @param date The date.
+     *  @return A base name for the blog entries.
+     */
     public static String makeEntryPage( String pageName, String date )
     {
         return TextUtil.replaceString(DEFAULT_PAGEFORMAT,"%p",pageName)+date;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
+    @SuppressWarnings("unchecked")
     public String execute( WikiContext context, Map params )
         throws PluginException
     {
@@ -148,7 +184,7 @@ public class WeblogPlugin
 
         if( ( params.get(PARAM_ENTRYFORMAT)) == null )
         {
-            entryFormat = DEFAULT_ENTRYFORMAT;
+            entryFormat = Preferences.getDateFormat( context, TimeFormat.DATETIME );
         }
         else
         {
@@ -225,10 +261,10 @@ public class WeblogPlugin
 
         try
         {
-            List blogEntries = findBlogEntries( engine.getPageManager(),
-                                                weblogName,
-                                                startTime.getTime(),
-                                                stopTime.getTime() );
+            List<WikiPage> blogEntries = findBlogEntries( engine.getPageManager(),
+                                                          weblogName,
+                                                          startTime.getTime(),
+                                                          stopTime.getTime() );
 
             Collections.sort( blogEntries, new PageDateComparator() );
 
@@ -294,7 +330,7 @@ public class WeblogPlugin
 
         // Extract the first h1/h2/h3 as title, and replace with null
         buffer.append("<div class=\"weblogentrytitle\">\n");
-        Matcher matcher = headingPattern.matcher( html );
+        Matcher matcher = HEADINGPATTERN.matcher( html );
         if ( matcher.find() )
         {
             String title = matcher.group(2);
@@ -392,7 +428,7 @@ public class WeblogPlugin
         throws ProviderException
     {
         Collection everyone = mgr.getAllPages();
-        ArrayList  result = new ArrayList();
+        ArrayList<WikiPage> result = new ArrayList<WikiPage>();
 
         baseName = makeEntryPage( baseName );
         SimpleDateFormat fmt = new SimpleDateFormat(DEFAULT_DATEFORMAT);
@@ -451,18 +487,15 @@ public class WeblogPlugin
     /**
      *  Reverse comparison.
      */
-    private static class PageDateComparator implements Comparator
+    private static class PageDateComparator implements Comparator<WikiPage>
     {
-        public int compare( Object o1, Object o2 )
+        public int compare( WikiPage page1, WikiPage page2 )
         {
-            if( o1 == null || o2 == null )
+            if( page1 == null || page2 == null )
             {
                 return 0;
             }
 
-            WikiPage page1 = (WikiPage)o1;
-            WikiPage page2 = (WikiPage)o2;
-
             return page2.getLastModified().compareTo( page1.getLastModified() );
         }
     }
