@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.providers;
 
@@ -78,6 +79,8 @@ public class BasicAttachmentProvider
 {
     private WikiEngine         m_engine;
     private String             m_storageDir;
+    
+    /** The property name for where the attachments should be stored.  Value is <tt>{@value}</tt>. */
     public static final String PROP_STORAGEDIR = "jspwiki.basicAttachmentProvider.storageDir";
     
     /*
@@ -85,15 +88,24 @@ public class BasicAttachmentProvider
      * since 2.5.96
      */
     private Pattern            m_disableCache = null;
+    
+    /** The property name for specifying which attachments are not cached.  Value is <tt>{@value}</tt>. */
     public static final String PROP_DISABLECACHE = "jspwiki.basicAttachmentProvider.disableCache";
 
+    /** The name of the property file. */
     public static final String PROPERTY_FILE   = "attachment.properties";
 
+    /** The default extension for the page attachment directory name. */
     public static final String DIR_EXTENSION   = "-att";
+    
+    /** The default extension for the attachment directory. */
     public static final String ATTDIR_EXTENSION = "-dir";
     
     static final Logger log = Logger.getLogger( BasicAttachmentProvider.class );
 
+    /**
+     *  {@inheritDoc}
+     */
     public void initialize( WikiEngine engine, Properties properties ) 
         throws NoRequiredPropertyException,
                IOException
@@ -248,6 +260,9 @@ public class BasicAttachmentProvider
      *  Returns the file extension.  For example "test.png" returns "png".
      *  <p>
      *  If file has no extension, will return "bin"
+     *  
+     *  @param filename The file name to check
+     *  @return The extension.  If no extension is found, returns "bin".
      */
     protected static String getFileExtension( String filename )
     {
@@ -306,6 +321,9 @@ public class BasicAttachmentProvider
         return props;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void putAttachmentData( Attachment att, InputStream data )
         throws ProviderException,
                IOException
@@ -343,7 +361,7 @@ public class BasicAttachmentProvider
 
             if( author == null )
             {
-                author = "unknown";
+                author = "unknown"; // FIXME: Should be localized, but cannot due to missing WikiContext
             }
 
             props.setProperty( versionNumber+".author", author );
@@ -367,6 +385,9 @@ public class BasicAttachmentProvider
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public String getProviderInfo()
     {
         return "";
@@ -403,6 +424,9 @@ public class BasicAttachmentProvider
         return f;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public InputStream getAttachmentData( Attachment att )
         throws IOException,
                ProviderException
@@ -422,10 +446,13 @@ public class BasicAttachmentProvider
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public Collection listAttachments( WikiPage page )
         throws ProviderException
     {
-        Collection result = new ArrayList();
+        Collection<Attachment> result = new ArrayList<Attachment>();
 
         File dir = findPageDir( page.getName() );
 
@@ -495,11 +522,17 @@ public class BasicAttachmentProvider
         return result;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public Collection findAttachments( QueryItem[] query )
     {
         return null;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     // FIXME: Very unoptimized.
     public List listAllChanged( Date timestamp )
         throws ProviderException
@@ -511,7 +544,7 @@ public class BasicAttachmentProvider
             throw new ProviderException("Specified attachment directory "+m_storageDir+" does not exist!");
         }
 
-        ArrayList list = new ArrayList();
+        ArrayList<Attachment> list = new ArrayList<Attachment>();
 
         String[] pagesWithAttachments = attDir.list( new AttachmentFilter() );
 
@@ -538,6 +571,9 @@ public class BasicAttachmentProvider
         return list;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public Attachment getAttachmentInfo( WikiPage page, String name, int version )
         throws ProviderException
     {
@@ -600,9 +636,12 @@ public class BasicAttachmentProvider
         return att;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public List getVersionHistory( Attachment att )
     {
-        ArrayList list = new ArrayList();
+        ArrayList<Attachment> list = new ArrayList<Attachment>();
 
         try
         {
@@ -628,13 +667,18 @@ public class BasicAttachmentProvider
         return list;
     }
 
-
+    /**
+     *  {@inheritDoc}
+     */
     public void deleteVersion( Attachment att )
         throws ProviderException
     {
         // FIXME: Does nothing yet.
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void deleteAttachment( Attachment att )
         throws ProviderException
     {
@@ -656,6 +700,9 @@ public class BasicAttachmentProvider
     public static class AttachmentFilter
         implements FilenameFilter
     {
+        /**
+         *  {@inheritDoc}
+         */
         public boolean accept( File dir, String name )
         {
             return name.endsWith( DIR_EXTENSION );
@@ -668,12 +715,19 @@ public class BasicAttachmentProvider
     public static class AttachmentVersionFilter
         implements FilenameFilter
     {
+        /**
+         *  {@inheritDoc}
+         */
         public boolean accept( File dir, String name )
         {
             return !name.equals( PROPERTY_FILE );
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
+
     public void moveAttachmentsForPage( String oldParent, String newParent )
         throws ProviderException
     {
