@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.providers;
 
@@ -32,7 +33,6 @@ import com.ecyrd.jspwiki.*;
  *  who insist on using Windows or other software which makes assumptions
  *  on the files contents based on its name.
  *
- *  @author Janne Jalkanen
  */
 public class FileSystemProvider
     extends AbstractFileProvider
@@ -43,6 +43,9 @@ public class FileSystemProvider
      */
     public static final String PROP_EXT = ".properties";
 
+    /**
+     *  {@inheritDoc}
+     */
     public void putPageText( WikiPage page, String text )        
         throws ProviderException
     {
@@ -129,6 +132,9 @@ public class FileSystemProvider
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public WikiPage getPageInfo( String page, int version )
         throws ProviderException
     {
@@ -150,6 +156,9 @@ public class FileSystemProvider
         return p;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void deletePage(String pageName) throws ProviderException
     {
         super.deletePage(pageName);
@@ -160,6 +169,9 @@ public class FileSystemProvider
         if( file.exists() ) file.delete();
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void movePage( String from,
                           String to )
         throws ProviderException
