@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.providers;
 
@@ -55,7 +56,6 @@ import com.ecyrd.jspwiki.*;
  *  who insist on using Windows or other software which makes assumptions
  *  on the files contents based on its name.
  *
- *  @author Janne Jalkanen
  */
 public class VersioningFileProvider
     extends AbstractFileProvider
@@ -63,16 +63,43 @@ public class VersioningFileProvider
 {
     private static final Logger     log = Logger.getLogger(VersioningFileProvider.class);
    
+    /** Name of the directory where the old versions are stored. */
     public static final String      PAGEDIR      = "OLD";
+    
+    /** Name of the property file which stores the metadata. */
     public static final String      PROPERTYFILE = "page.properties";
 
     private CachedProperties        m_cachedProperties;
     
+    /**
+     *  {@inheritDoc}
+     */
     public void initialize( WikiEngine engine, Properties properties )
         throws NoRequiredPropertyException,
                IOException
     {
         super.initialize( engine, properties );
+        // some additional sanity checks :
+        File oldpages = new File(getPageDirectory(), PAGEDIR);
+        if (!oldpages.exists())
+        {
+            if (!oldpages.mkdirs())
+            {
+                throw new IOException("Failed to create page version directory " + oldpages.getAbsolutePath());
+            }
+        }
+        else
+        {
+            if (!oldpages.isDirectory())
+            {
+                throw new IOException("Page version directory is not a directory: " + oldpages.getAbsolutePath());
+            }
+            if (!oldpages.canWrite())
+            {
+                throw new IOException("Page version directory is not writable: " + oldpages.getAbsolutePath());
+            }
+        }
+        log.info("Using directory " + oldpages.getAbsolutePath() + " for storing old versions of pages");
     }
 
     /**
@@ -194,7 +221,7 @@ public class VersioningFileProvider
             //
             //   The profiler showed that when calling the history of a page the propertyfile
             //   was read just as much times as there were versions of that file. The loading
-            //   of a propertyfile is a cpu-intensive jobs. So now hold on to the last propertyfile
+            //   of a propertyfile is a cpu-intensive job. So now hold on to the last propertyfile
             //   read because the next method will with a high probability ask for the same propertyfile.
             //   The time it took to show a historypage with 267 versions dropped with 300%. 
             //
@@ -291,6 +318,9 @@ public class VersioningFileProvider
         return requestedVersion;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public synchronized String getPageText( String page, int version )
         throws ProviderException
     {
@@ -373,6 +403,9 @@ public class VersioningFileProvider
            2         Main.txt (2)  1.txt
            3         Main.txt (3)  1.txt, 2.txt
     */
+    /**
+     *  {@inheritDoc}
+     */
     public synchronized void putPageText( WikiPage page, String text )
         throws ProviderException
     {
@@ -465,6 +498,9 @@ public class VersioningFileProvider
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public WikiPage getPageInfo( String page, int version )
         throws ProviderException
     {
@@ -548,6 +584,9 @@ public class VersioningFileProvider
         return p;
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public boolean pageExists( String pageName, int version )
     {
         File dir = findOldPageDir( pageName );
@@ -568,12 +607,13 @@ public class VersioningFileProvider
     }
 
     /**
-     *  FIXME: Does not get user information.
+     *  {@inheritDoc}
      */
+     // FIXME: Does not get user information.
     public List getVersionHistory( String page )
     throws ProviderException
     {
-        ArrayList list = new ArrayList();
+        ArrayList<WikiPage> list = new ArrayList<WikiPage>();
 
         int latest = findLatestVersion( page );
 
@@ -596,6 +636,9 @@ public class VersioningFileProvider
      *  Removes the relevant page directory under "OLD" -directory as well,
      *  but does not remove any extra subdirectories from it.  It will only
      *  touch those files that it thinks to be WikiPages.
+     *  
+     *  @param page {@inheritDoc}
+     *  @throws {@inheritDoc}
      */
     // FIXME: Should log errors.
     public void deletePage( String page )
@@ -625,6 +668,9 @@ public class VersioningFileProvider
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void deleteVersion( String page, int version )
         throws ProviderException
     {
@@ -717,11 +763,14 @@ public class VersioningFileProvider
         }
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     // FIXME: This is kinda slow, we should need to do this only once.
     public Collection getAllPages() throws ProviderException
     {
         Collection pages = super.getAllPages();
-        Collection returnedPages = new ArrayList();
+        Collection<WikiPage> returnedPages = new ArrayList<WikiPage>();
         
         for( Iterator i = pages.iterator(); i.hasNext(); )
         {
@@ -735,11 +784,17 @@ public class VersioningFileProvider
         return returnedPages;
     }
     
+    /**
+     *  {@inheritDoc}
+     */
     public String getProviderInfo()
     {
         return "";
     }
 
+    /**
+     *  {@inheritDoc}
+     */
     public void movePage( String from,
                           String to )
         throws ProviderException
