@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.providers;
 
@@ -41,13 +42,17 @@ import com.ecyrd.jspwiki.attachment.Attachment;
  *  Input/OutputStreams.
  *
  *  @author Erik Bunn
- *  @author Janne Jalkanen
  */
 public interface WikiAttachmentProvider
     extends WikiProvider
 {
     /**
      *  Put new attachment data.
+     *  
+     *  @param att Attachment object to add new data to
+     *  @param data The stream from which the provider should read the data
+     *  @throws IOException If writing fails
+     *  @throws ProviderException If there are other errors.
      */
     public void putAttachmentData( Attachment att, InputStream data )
         throws ProviderException,
@@ -55,6 +60,12 @@ public interface WikiAttachmentProvider
 
     /**
      *  Get attachment data.
+     *  
+     *  @param att The attachment
+     *  @return An InputStream which you contains the raw data of the object. It's your
+     *          responsibility to close it.
+     *  @throws ProviderException If the attachment cannot be found
+     *  @throws IOException If the attachment cannot be opened
      */
 
     public InputStream getAttachmentData( Attachment att )
@@ -64,7 +75,9 @@ public interface WikiAttachmentProvider
     /**
      *  Lists all attachments attached to a page.
      *
+     *  @param page The page to list the attachments from.
      *  @return A collection of Attachment objects.  May be empty, but never null.
+     *  @throws ProviderException If something goes wrong when listing the attachments.
      */
 
     public Collection listAttachments( WikiPage page )
@@ -72,6 +85,8 @@ public interface WikiAttachmentProvider
 
     /**
      * Finds attachments based on the query.
+     * @param query An array of QueryItem objects to search for
+     * @return A Collection of Attachment objects.  May be empty, but never null.
      */
     public Collection findAttachments( QueryItem[] query );
 
@@ -89,12 +104,19 @@ public interface WikiAttachmentProvider
      *
      *  @param timestamp List all files from this date onward.
      *  @return A List of Attachment objects, in most-recently-changed first order.
+     *  @throws ProviderException If something goes wrong.
      */
     public List listAllChanged( Date timestamp )
         throws ProviderException;
 
     /**
      *  Returns info about an attachment.
+     *  
+     *  @param page The parent page
+     *  @param name The name of the attachment
+     *  @param version The version of the attachment (it's okay to use WikiPage.LATEST_VERSION to find the latest one)
+     *  @return An attachment object
+     *  @throws ProviderException If the attachment cannot be found or some other error occurs.
      */
     public Attachment getAttachmentInfo( WikiPage page, String name, int version )
         throws ProviderException;
@@ -102,6 +124,9 @@ public interface WikiAttachmentProvider
     /**
      *  Returns version history.  Each element should be
      *  an Attachment.
+     *  
+     *  @param att The attachment for which to find the version history for.
+     *  @return A List of Attachment objects.
      */
     public List getVersionHistory( Attachment att );
 
