@@ -1,22 +1,23 @@
 /*
-JSPWiki - a JSP-based WikiWiki clone.
+    JSPWiki - a JSP-based WikiWiki clone.
 
-Copyright (C) 2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or
-(at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.    
+ */
 package com.ecyrd.jspwiki.search;
 
 import java.io.*;
@@ -54,7 +55,6 @@ import com.ecyrd.jspwiki.providers.WikiPageProvider;
 import com.ecyrd.jspwiki.util.ClassUtil;
 import com.ecyrd.jspwiki.util.WatchDog;
 import com.ecyrd.jspwiki.util.WikiBackgroundThread;
-import com.opensymphony.oscache.util.StringUtil;
 
 /**
  *  Interface for the search providers that handle searching the Wiki
@@ -92,7 +92,7 @@ public class LuceneSearchProvider implements SearchProvider
 
     private String           m_luceneDirectory = null;
     private int              m_updateCount = 0;
-    protected Vector         m_updates = new Vector(); // Vector because multi-threaded.
+    protected Vector<Object[]> m_updates = new Vector<Object[]>(); // Vector because multi-threaded.
 
     /** Maximum number of fragments from search matches. */
     private static final int MAX_FRAGMENTS = 3;
@@ -273,8 +273,13 @@ public class LuceneSearchProvider implements SearchProvider
     /**
      *  Fetches the attachment content from the repository.
      *  Content is flat text that can be used for indexing/searching or display
+     *  
+     *  @param attachmentName Name of the attachment.
+     *  @param version The version of the attachment.
+     *  
+     *  @return the content of the Attachment as a String.
      */
-    private String getAttachmentContent( String attachmentName, int version )
+    protected String getAttachmentContent( String attachmentName, int version )
     {
         AttachmentManager mgr = m_engine.getAttachmentManager();
 
@@ -301,7 +306,7 @@ public class LuceneSearchProvider implements SearchProvider
      * FIXME This is a very simple implementation of some text-based attachment, mainly used for testing.
      * This should be replaced /moved to Attachment search providers or some other 'plugable' wat to search attachments
      */
-    private String getAttachmentContent( Attachment att )
+    protected String getAttachmentContent( Attachment att )
     {
         AttachmentManager mgr = m_engine.getAttachmentManager();
         //FIXME: Add attachment plugin structure
@@ -400,21 +405,23 @@ public class LuceneSearchProvider implements SearchProvider
     }
 
     /**
-     *  Indexes page using the given IndexWriter
+     *  Indexes page using the given IndexWriter.
      *
      *  @param page WikiPage
      *  @param text Page text to index
      *  @param writer The Lucene IndexWriter to use for indexing
+     *  @return the created index Document
      *  @throws IOException If there's an indexing problem
-     *
      */
-    protected void luceneIndexPage( WikiPage page, String text, IndexWriter writer )
+    protected Document luceneIndexPage( WikiPage page, String text, IndexWriter writer )
         throws IOException
     {
+        if( log.isDebugEnabled() ) log.debug( "Indexing "+page.getName()+"..." );
+        
         // make a new, empty document
         Document doc = new Document();
 
-        if( text == null ) return;
+        if( text == null ) return doc;
 
         // Raw name is the keyword we'll use to refer to this document for updates.
         Field field = new Field(LUCENE_ID, page.getName(), Field.Store.YES, Field.Index.UN_TOKENIZED);
@@ -466,6 +473,8 @@ public class LuceneSearchProvider implements SearchProvider
             log.error("Failed to get attachments for page", e);
         }
         writer.addDocument(doc);
+
+        return doc;
     }
 
     /**
@@ -548,7 +557,7 @@ public class LuceneSearchProvider implements SearchProvider
         throws ProviderException
     {
         Searcher  searcher = null;
-        ArrayList list     = null;
+        ArrayList<SearchResult> list = null;
         Highlighter highlighter = null;
 
         try
@@ -578,7 +587,7 @@ public class LuceneSearchProvider implements SearchProvider
 
             Hits hits = searcher.search(luceneQuery);
 
-            list = new ArrayList(hits.length());
+            list = new ArrayList<SearchResult>(hits.length());
             for ( int curr = 0; curr < hits.length(); curr++ )
             {
                 Document doc = hits.doc(curr);
@@ -609,7 +618,8 @@ public class LuceneSearchProvider implements SearchProvider
 
                     }
 
-                    SearchResult result = new SearchResultImpl( page, score, fragments );                    list.add(result);
+                    SearchResult result = new SearchResultImpl( page, score, fragments );     
+                    list.add(result);
                 }
                 else
                 {
@@ -669,7 +679,7 @@ public class LuceneSearchProvider implements SearchProvider
      */
     private static final class LuceneUpdater extends WikiBackgroundThread
     {
-        protected static final int INDEX_DELAY    = 1;
+        protected static final int INDEX_DELAY    = 5;
         protected static final int INITIAL_DELAY = 60;
         private final LuceneSearchProvider m_provider;
 
@@ -713,7 +723,7 @@ public class LuceneSearchProvider implements SearchProvider
             {
                 while( m_provider.m_updates.size() > 0 )
                 {
-                    Object[] pair = ( Object[] ) m_provider.m_updates.remove(0);
+                    Object[] pair = m_provider.m_updates.remove(0);
                     WikiPage page = ( WikiPage ) pair[0];
                     String text = ( String ) pair[1];
                     m_provider.updateLuceneIndex(page, text);
