@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.tags;
 
@@ -26,10 +27,11 @@ import org.apache.commons.lang.StringUtils;
 import com.ecyrd.jspwiki.providers.ProviderException;
 
 /**
- *  Includes body, if the request context matches.
+ *  Includes body, if the request context matches.  To understand more about
+ *  RequestContexts, please look at the WikiContext class.
  *
- *  @author Janne Jalkanen
  *  @since 2.0
+ *  @see com.ecyrd.jspwiki.WikiContext
  */
 public class CheckRequestContextTag
     extends WikiTagBase
@@ -39,7 +41,10 @@ public class CheckRequestContextTag
     private String m_context;
     private String[] m_contextList = {};
 
-
+    /**
+     *  {@inheritDoc}
+     */
+    @Override
     public void initTag()
     {
         super.initTag();
@@ -47,11 +52,21 @@ public class CheckRequestContextTag
         m_contextList = new String[0];
     }
     
+    /**
+     *  Returns the context.
+     *  
+     *  @return Return the context.
+     */
     public String getContext()
     {
         return m_context;
     }
 
+    /**
+     *  Set the context to check for.
+     *  
+     *  @param arg One of the RequestsContexts.
+     */
     public void setContext( String arg )
     {
         m_context = arg;
@@ -59,6 +74,10 @@ public class CheckRequestContextTag
         m_contextList = StringUtils.split(arg,'|');
     }
 
+    /**
+     *  {@inheritDoc}
+     */
+    @Override
     public final int doWikiStartTag()
         throws IOException,
                ProviderException
