@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.tags;
 
@@ -24,6 +25,8 @@ import java.text.SimpleDateFormat;
 import java.io.IOException;
 
 import com.ecyrd.jspwiki.WikiPage;
+import com.ecyrd.jspwiki.preferences.Preferences;
+import com.ecyrd.jspwiki.preferences.Preferences.TimeFormat;
 
 /**
  *  Writes the modification date of the page, formatted
@@ -34,7 +37,6 @@ import com.ecyrd.jspwiki.WikiPage;
  *       This is exactly like in "java.text.SimpleDateFormat".
  *  </UL>
  *
- *  @author Janne Jalkanen
  *  @since 2.0
  */
 
@@ -57,9 +59,6 @@ public class PageDateTag
 
     public String getFormat()
     {
-        if( m_format == null )
-            return DEFAULT_FORMAT;
-
         return m_format;
     }
 
@@ -82,7 +81,12 @@ public class PageDateTag
             //
             if( d != null )
             {
-                SimpleDateFormat fmt = new SimpleDateFormat( getFormat() );
+                SimpleDateFormat fmt;
+                
+                if( m_format == null )
+                    fmt = Preferences.getDateFormat( m_wikiContext, TimeFormat.DATETIME );
+                else
+                    fmt = new SimpleDateFormat( m_format );
 
                 pageContext.getOut().write( fmt.format( d ) );
             }
