@@ -1,21 +1,22 @@
-/*
+/* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2007 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.ui;
 
@@ -76,16 +77,16 @@ public final class CommandResolver
     private static final String PROP_SPECIALPAGE = "jspwiki.specialPage.";
 
     /** Private map with request contexts as keys, Commands as values */
-    private static final Map    CONTEXTS;
+    private static final Map<String, Command>    CONTEXTS;
 
     /** Private map with JSPs as keys, Commands as values */
-    private static final Map    JSPS;
+    private static final Map<String, Command>    JSPS;
 
     /** Store the JSP-to-Command and context-to-Command mappings */
     static
     {
-        CONTEXTS = new HashMap();
-        JSPS = new HashMap();
+        CONTEXTS = new HashMap<String, Command>();
+        JSPS = new HashMap<String, Command>();
         Command[] commands = AbstractCommand.allCommands();
         for( int i = 0; i < commands.length; i++ )
         {
@@ -102,7 +103,7 @@ public final class CommandResolver
     private final boolean       m_matchEnglishPlurals;
 
     /** Stores special page names as keys, and Commands as values. */
-    private final Map           m_specialPages;
+    private final Map<String, Command> m_specialPages;
 
     /**
      * Constructs a CommandResolver for a given WikiEngine. This constructor
@@ -114,7 +115,7 @@ public final class CommandResolver
     public CommandResolver( WikiEngine engine, Properties properties )
     {
         m_engine = engine;
-        m_specialPages = new HashMap();
+        m_specialPages = new HashMap<String, Command>();
 
         // Skim through the properties and look for anything with
         // the "special page" prefix. Create maps that allow us
@@ -132,7 +133,7 @@ public final class CommandResolver
                 {
                     specialPage = specialPage.trim();
                     jsp = jsp.trim();
-                    Command command = (Command) JSPS.get( jsp );
+                    Command command = JSPS.get( jsp );
                     if ( command == null )
                     {
                         Command redirect = RedirectCommand.REDIRECT;
@@ -159,7 +160,7 @@ public final class CommandResolver
      */
     public static Command findCommand( String context )
     {
-        Command command = (Command) CONTEXTS.get( context );
+        Command command = CONTEXTS.get( context );
         if ( command == null )
         {
             throw new IllegalArgumentException( "Unsupported wiki context: " + context + "." );
@@ -214,7 +215,7 @@ public final class CommandResolver
         // Can we find a special-page command matching the extracted page?
         if ( pageName != null )
         {
-            command = (AbstractCommand) m_specialPages.get( pageName );
+            command = m_specialPages.get( pageName );
         }
 
         // If we haven't found a matching command yet, extract the JSP path
@@ -226,7 +227,7 @@ public final class CommandResolver
             // Otherwise: use the default context
             if ( command == null )
             {
-                command = (AbstractCommand) CONTEXTS.get( defaultContext );
+                command = CONTEXTS.get( defaultContext );
                 if ( command == null )
                 {
                     throw new IllegalArgumentException( "Wiki context " + defaultContext + " is illegal." );
@@ -264,6 +265,7 @@ public final class CommandResolver
         if ( command instanceof GroupCommand )
         {
             String groupName = request.getParameter( "group" );
+            groupName = TextUtil.replaceEntities( groupName );
             if ( groupName != null && groupName.length() > 0 )
             {
                 GroupPrincipal group = new GroupPrincipal( groupName );
@@ -355,7 +357,7 @@ public final class CommandResolver
      */
     public final String getSpecialPageReference( String page )
     {
-        Command command = (Command) m_specialPages.get( page );
+        Command command = m_specialPages.get( page );
 
         if ( command != null )
         {
@@ -408,7 +410,7 @@ public final class CommandResolver
         // Ok, see if we match against our standard list of JSPs
         if ( jsp.length() > 0 && JSPS.containsKey( jsp ) )
         {
-            return (Command)JSPS.get( jsp );
+            return JSPS.get( jsp );
         }
 
         return null;
@@ -489,7 +491,16 @@ public final class CommandResolver
 
         if ( rev != null )
         {
-            version = Integer.parseInt( rev );
+            try
+            {
+                version = Integer.parseInt( rev );
+            }
+            catch( NumberFormatException e )
+            {
+                // This happens a lot with bots or other guys who are trying
+                // to test if we are vulnerable to e.g. XSS attacks.  We catch
+                // it here so that the admin does not get tons of mail.
+            }
         }
 
         wikipage = m_engine.getPage( page, version );
