@@ -1,22 +1,23 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
+ */
 package com.ecyrd.jspwiki.url;
 
 import java.util.Properties;
@@ -30,7 +31,6 @@ import com.ecyrd.jspwiki.WikiEngine;
  *  A specific URL constructor that returns easy-to-grok URLs for
  *  VIEW and ATTACH contexts, but goes through JSP pages otherwise.
  * 
- *  @author jalkanen
  *
  *  @since 2.2
  */
@@ -100,10 +100,10 @@ public class ShortViewURLConstructor
      *   Since we're only called from WikiServlet, where we get the VIEW requests,
      *   we can safely return this.
      *   
-     *   @param {@inheritDoc}
-     *   @return {@inheritDoc}
+     * @param request The HTTP Request that was used to end up in this page.
+     * @return always returns "Wiki.jsp"
      */
-    public String getForwardPage( HttpServletRequest req )
+    public String getForwardPage( HttpServletRequest request )
     {        
         return "Wiki.jsp";
     }
