@@ -1,21 +1,22 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.util;
 
@@ -32,6 +33,7 @@ import javax.naming.NamingException;
 
 import org.apache.log4j.Logger;
 
+import com.ecyrd.jspwiki.TextUtil;
 import com.ecyrd.jspwiki.WikiEngine;
 
 /**
@@ -53,8 +55,8 @@ import com.ecyrd.jspwiki.WikiEngine;
  * this is already taken care of. However, when using JNDI-supplied
  * Session factories, these should be moved, <em>not copied</em>, to a classpath location
  * where the JARs can be shared by both the JSPWiki webapp and the container. For example,
- * Tomcat 5 provides the directory <code><var>$CATALINA_HOME></var>/common/lib</code>
- * for storage of shared JARs; move <code>mail.jar</code> and <code>activation</code>
+ * Tomcat 5 provides the directory <code><var>$CATALINA_HOME</var>/common/lib</code>
+ * for storage of shared JARs; move <code>mail.jar</code> and <code>activation.jar</code>
  * there instead of keeping them in <code>/WEB-INF/lib</code>.</p>
  * <strong>JavaMail configuration</strong>
  * <p>Regardless of the method used for supplying JavaMail sessions (JNDI container-managed
@@ -197,7 +199,9 @@ public final class MailUtil
 
     private static final String TRUE = "true";
 
-    public static final String PROP_MAIL_AUTH = "mail.smtp.auth";
+    private static boolean c_useJndi = true;
+
+    private static final String PROP_MAIL_AUTH = "mail.smtp.auth";
 
     protected static final Logger log = Logger.getLogger(MailUtil.class);
 
@@ -209,6 +213,10 @@ public final class MailUtil
 
     protected static final String DEFAULT_MAIL_TIMEOUT         = "5000";
 
+    protected static final String DEFAULT_MAIL_CONN_TIMEOUT    = "5000";
+
+    protected static final String DEFAULT_SENDER               = "jspwiki@localhost";
+
     protected static final String PROP_MAIL_JNDI_NAME          = "jspwiki.mail.jndiname";
 
     protected static final String PROP_MAIL_HOST               = "mail.smtp.host";
@@ -229,6 +237,8 @@ public final class MailUtil
 
     protected static final String PROP_MAIL_STARTTLS           = "mail.smtp.starttls.enable";
 
+    private static String c_fromAddress = null;
+    
     /**
      *  Private constructor prevents instantiation.
      */
@@ -237,7 +247,7 @@ public final class MailUtil
     }
 
     /**
-     * <p>Sends an e-mail to a specified receiver using a  JavaMail Session supplied
+     * <p>Sends an e-mail to a specified receiver using a JavaMail Session supplied
      * by a JNDI mail session factory (preferred) or a locally initialized
      * session based on properties in <code>jspwiki.properties</code>.
      * See the top-level JavaDoc for this class for a description of
@@ -256,88 +266,117 @@ public final class MailUtil
      * @param to the receiver
      * @param subject the subject line of the message
      * @param content the contents of the mail message, as plain text
+     * @throws AddressException If the address is invalid
+     * @throws MessagingException If the message cannot be sent.
      */
-    public static void sendMessage( WikiEngine engine, String to, String subject, String content )
+    public static void sendMessage(WikiEngine engine, String to, String subject, String content)
         throws AddressException, MessagingException
-    {
-        String from = engine.getWikiProperties().getProperty( PROP_MAIL_SENDER ).trim();
-        sendMessage( engine, to, from, subject, content );
-    }
-
-    /**
-     * <p>Sends an e-mail to a specified receiver from a specified sender, using a
-     * JavaMail Session supplied by a JNDI mail session factory (preferred) or
-     * a locally initialized session based on properties in
-     * <code>jspwiki.properties</code>. See the top-level JavaDoc for this
-     * class for a description of required properties and their
-     * default values.</p>
-     * <p>The e-mail addresses used for the <code>to</code> and <code>from</code>
-     * parameters must be in RFC822 format, as described in the JavaDoc for
-     * {@link javax.mail.internet.InternetAddress} and more fully at
-     * <a href="http://www.freesoft.org/CIE/RFC/822/index.htm">http://www.freesoft.org/CIE/RFC/822/index.htm</a>.
-     * In other words, e-mail addresses should look like this:</p>
-     * <blockquote><code>Snoop Dog &lt;snoop.dog@shizzle.net&gt;<br/>
-     * snoop.dog@shizzle.net</code></blockquote>
-     * <p>Note that the first form allows a "friendly" user name to be supplied
-     * in addition to the actual e-mail address.</p>
-     *
-     * @param engine the WikiEngine for the current wiki
-     * @param to the receiver
-     * @param from the address the email will be from
-     * @param subject the subject line of the message
-     * @param content the contents of the mail message, as plain text
-     */
-    public static void sendMessage(WikiEngine engine, String to, String from, String subject, String content)
-        throws MessagingException
     {
         Properties props = engine.getWikiProperties();
-        String jndiName = props.getProperty( PROP_MAIL_JNDI_NAME, DEFAULT_MAIL_JNDI_NAME ).trim();
-        Session session = null;
-        boolean useJndi = false;
-
-        // Try getting the Session from the JNDI factory first
-        try
-        {
-            session = getJNDIMailSession( jndiName );
-            useJndi = true;
-        }
-        catch ( NamingException e )
-        {
-            // Oops! JNDI factory must not be set up
-        }
-
-        // JNDI failed; so, get the Session from the standalone factory
-        if ( session == null )
-        {
-            session = getStandaloneMailSession( props );
-        }
+        Session session = getMailSession(engine);
+        getSenderEmailAddress(session, props);
 
         try
         {
             // Create and address the message
-            Message msg = new MimeMessage( session );
-            msg.setFrom( new InternetAddress( from ) );
-            msg.setRecipients( Message.RecipientType.TO, InternetAddress.parse( to, false ) );
-            msg.setSubject( subject );
-            msg.setText( content );
-            msg.setSentDate( new Date() );
+            MimeMessage msg = new MimeMessage(session);
+            msg.setFrom(new InternetAddress(c_fromAddress));
+            msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to, false));
+            msg.setSubject(subject);
+            msg.setText(content, "UTF-8");
+            msg.setSentDate(new Date());
 
             // Send and log it
-            Transport.send( msg );
-            if ( log.isInfoEnabled() )
+            Transport.send(msg);
+            if (log.isInfoEnabled())
             {
-                log.info( "Sent e-mail to=" + to + ", subject=\"" + subject
-                    + "\", jndi=" + ( useJndi ? TRUE : FALSE ) );
+                log.info("Sent e-mail to=" + to + ", subject=\"" + subject + "\", used "
+                         + (c_useJndi ? "JNDI" : "standalone") + " mail session.");
             }
         }
-        catch ( MessagingException e )
+        catch (MessagingException e)
         {
-            log.error( e );
+            log.error(e);
             throw e;
         }
     }
+    
+    // --------- JavaMail Session Helper methods  --------------------------------
 
-    // --------- JavaMail Session Helper methods ---------------------------------
+    /**
+     * Gets the Sender's email address from JNDI Session if available, otherwise
+     * from the jspwiki.properties or lastly the default value.
+     * @param pSession <code>Session</code>
+     * @param pProperties <code>Properties</code>
+     * @return <code>String</code>
+     */
+    protected static String getSenderEmailAddress(Session pSession, Properties pProperties)
+    {
+        if( c_fromAddress == null )
+        {
+            // First, attempt to get the email address from the JNDI Mail
+            // Session.
+            if( pSession != null && c_useJndi )
+            {
+                c_fromAddress = pSession.getProperty( MailUtil.PROP_MAIL_SENDER );
+            }
+            // If unsuccessful, get the email address from the properties or
+            // default.
+            if( c_fromAddress == null )
+            {
+                c_fromAddress = pProperties.getProperty( PROP_MAIL_SENDER, DEFAULT_SENDER ).trim();
+                if( log.isDebugEnabled() )
+                    log.debug( "Attempt to get the sender's mail address from the JNDI mail session failed, will use \""
+                               + c_fromAddress + "\" (configured via jspwiki.properties or the internal default)." );
+            }
+            else
+            {
+                if( log.isDebugEnabled() )
+                    log.debug( "Attempt to get the sender's mail address from the JNDI mail session was successful (" + c_fromAddress
+                               + ")." );
+            }
+        }
+        return c_fromAddress;
+    }
+
+    /**
+     * Returns the Mail Session from either JNDI or creates a stand-alone.
+     * @param engine a <code>WikiEngine</code>
+     * @return <code>Session</code>
+     */
+    private static Session getMailSession(WikiEngine engine)
+    {
+        Session result = null;
+        Properties props = engine.getWikiProperties();
+        String jndiName = props.getProperty(PROP_MAIL_JNDI_NAME, DEFAULT_MAIL_JNDI_NAME).trim();
+
+        if (c_useJndi)
+        {
+            // Try getting the Session from the JNDI factory first
+            if ( log.isDebugEnabled() )
+                log.debug("Try getting a mail session via JNDI name \"" + jndiName + "\".");
+            try
+            {
+                result = getJNDIMailSession(jndiName);
+            }
+            catch (NamingException e)
+            {
+                // Oops! JNDI factory must not be set up
+                c_useJndi = false;
+                if ( log.isInfoEnabled() )
+                    log.info("Unable to get a mail session via JNDI, will use custom settings at least until next startup.");
+            }
+        }
+
+        // JNDI failed; so, get the Session from the standalone factory
+        if (result == null)
+        {
+            if ( log.isDebugEnabled() )
+                log.debug("Getting a standalone mail session configured by jspwiki.properties and/or internal default values.");
+            result = getStandaloneMailSession(props);
+        }
+        return result;
+    }
 
     /**
      * Returns a stand-alone JavaMail Session by looking up the correct
@@ -356,6 +395,10 @@ public final class MailUtil
         String port     = props.getProperty( PROP_MAIL_PORT, DEFAULT_MAIL_PORT );
         String account  = props.getProperty( PROP_MAIL_ACCOUNT );
         String password = props.getProperty( PROP_MAIL_PASSWORD );
+        String timeout  = props.getProperty( PROP_MAIL_TIMEOUT, DEFAULT_MAIL_TIMEOUT);
+        String conntimeout = props.getProperty( PROP_MAIL_CONNECTION_TIMEOUT, DEFAULT_MAIL_CONN_TIMEOUT );
+        boolean starttls = TextUtil.getBooleanProperty( props, PROP_MAIL_STARTTLS, true);
+        
         boolean useAuthentication = account != null && account.length() > 0;
 
         Properties mailProps = new Properties();
@@ -363,9 +406,9 @@ public final class MailUtil
         // Set JavaMail properties
         mailProps.put( PROP_MAIL_HOST, host );
         mailProps.put( PROP_MAIL_PORT, port );
-        mailProps.put( PROP_MAIL_TIMEOUT, DEFAULT_MAIL_TIMEOUT );
-        mailProps.put( PROP_MAIL_CONNECTION_TIMEOUT, DEFAULT_MAIL_TIMEOUT );
-        mailProps.put( PROP_MAIL_STARTTLS, TRUE );
+        mailProps.put( PROP_MAIL_TIMEOUT, timeout );
+        mailProps.put( PROP_MAIL_CONNECTION_TIMEOUT, conntimeout );
+        mailProps.put( PROP_MAIL_STARTTLS, starttls ? TRUE : FALSE );
 
         // Add SMTP authentication if required
         Session session = null;
@@ -383,7 +426,9 @@ public final class MailUtil
 
         if ( log.isDebugEnabled() )
         {
-            String mailServer = host + ":" + port + ", auth=" + ( useAuthentication ? TRUE : FALSE );
+            String mailServer = host + ":" + port + ", account=" + account + ", password not displayed, timeout="
+            + timeout + ", connectiontimeout=" + conntimeout + ", starttls.enable=" + starttls
+            + ", use authentication=" + ( useAuthentication ? TRUE : FALSE );
             log.debug( "JavaMail session obtained from standalone mail factory: " + mailServer );
         }
         return session;
@@ -408,12 +453,12 @@ public final class MailUtil
         }
         catch( NamingException e )
         {
-            log.error( "JavaMail initialization error: " + e.getMessage() );
+            log.warn( "JNDI mail session initialization error: " + e.getMessage() );
             throw e;
         }
         if ( log.isDebugEnabled() )
         {
-            log.debug( "JavaMail session obtained from JNDI mail factory: " + jndiName );
+            log.debug( "mail session obtained from JNDI mail factory: " + jndiName );
         }
         return session;
     }
@@ -444,6 +489,7 @@ public final class MailUtil
 
         /**
          * Returns the password used to authenticate to the SMTP server.
+         * @return <code>PasswordAuthentication</code>
          */
         public PasswordAuthentication getPasswordAuthentication()
         {
