@@ -1,24 +1,26 @@
 /* 
     JSPWiki - a JSP-based WikiWiki clone.
 
-    Copyright (C) 2001-2002 Janne Jalkanen (Janne.Jalkanen@iki.fi)
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 2.1 of the License, or
-    (at your option) any later version.
+       http://www.apache.org/licenses/LICENSE-2.0
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
  */
 package com.ecyrd.jspwiki.workflow;
 
+import java.io.Serializable;
 import java.security.Principal;
 import java.util.*;
 
@@ -134,7 +136,7 @@ import com.ecyrd.jspwiki.event.WorkflowEvent;
  * array is retrieved via {@link #getMessageArguments()}; the first two array
  * elements will always be these: a String representing work flow owner's name,
  * and a String representing the current actor's name. Workflow participants
- * can add to this array by invoking {@link #addMessageArgument(Object)}.</li>
+ * can add to this array by invoking {@link #addMessageArgument(Serializable)}.</li>
  * </ul>
  * <h2>Example</h2>
  * <p>
@@ -197,10 +199,12 @@ import com.ecyrd.jspwiki.event.WorkflowEvent;
  *
  * @author Andrew Jaquith
  */
-public class Workflow
+public class Workflow implements Serializable
 {
+    private static final long serialVersionUID = 5228149040690660032L;
+
     /** Time value: the start or end time has not been set. */
-    public static final Date TIME_NOT_SET = new Date(0);
+    public static final Date TIME_NOT_SET = new Date( 0 );
 
     /** ID value: the workflow ID has not been set. */
     public static final int ID_NOT_SET = 0;
@@ -224,7 +228,7 @@ public class Workflow
     public static final int CREATED = -2;
 
     /** Lazily-initialized attribute map. */
-    private Map m_attributes;
+    private Map<String, Object> m_attributes;
 
     /** The initial Step for this Workflow. */
     private Step m_firstStep;
@@ -232,7 +236,7 @@ public class Workflow
     /** Flag indicating whether the Workflow has started yet. */
     private boolean m_started;
 
-    private final LinkedList m_history;
+    private final LinkedList<Step> m_history;
 
     private int m_id;
 
@@ -240,7 +244,7 @@ public class Workflow
 
     private final Principal m_owner;
 
-    private final List m_messageArgs;
+    private final List<Serializable> m_messageArgs;
 
     private int m_state;
 
@@ -267,11 +271,11 @@ public class Workflow
         super();
         m_attributes = null;
         m_currentStep = null;
-        m_history = new LinkedList();
+        m_history = new LinkedList<Step>();
         m_id = ID_NOT_SET;
         m_key = messageKey;
         m_manager = null;
-        m_messageArgs = new ArrayList();
+        m_messageArgs = new ArrayList<Serializable>();
         m_owner = owner;
         m_started = false;
         m_state = CREATED;
@@ -291,27 +295,27 @@ public class Workflow
     public final synchronized void abort()
     {
         // Check corner cases: previous abort or completion
-        if (m_state == ABORTED)
+        if ( m_state == ABORTED )
         {
-            throw new IllegalStateException("The workflow has already been aborted.");
+            throw new IllegalStateException( "The workflow has already been aborted." );
         }
-        if (m_state == COMPLETED)
+        if ( m_state == COMPLETED )
         {
-            throw new IllegalStateException("The workflow has already completed.");
+            throw new IllegalStateException( "The workflow has already completed." );
         }
 
-        if (m_currentStep != null)
+        if ( m_currentStep != null )
         {
-            if (m_manager != null && m_currentStep instanceof Decision)
+            if ( m_manager != null && m_currentStep instanceof Decision )
             {
                 Decision d = (Decision)m_currentStep;
-                m_manager.getDecisionQueue().remove(d);
+                m_manager.getDecisionQueue().remove( d );
             }
-            m_currentStep.setOutcome(Outcome.STEP_ABORT);
-            m_history.addLast(m_currentStep);
+            m_currentStep.setOutcome( Outcome.STEP_ABORT );
+            m_history.addLast( m_currentStep );
         }
         m_state = ABORTED;
-        fireEvent(WorkflowEvent.ABORTED);
+        fireEvent( WorkflowEvent.ABORTED );
         cleanup();
     }
 
@@ -324,14 +328,14 @@ public class Workflow
      * an IllegalArgumentException.
      * @param obj the object to add
      */
-    public final void addMessageArgument(Object obj)
+    public final void addMessageArgument( Serializable obj )
     {
-        if (obj instanceof String || obj instanceof Date || obj instanceof Number)
+        if ( obj instanceof String || obj instanceof Date || obj instanceof Number )
         {
-            m_messageArgs.add(obj);
+            m_messageArgs.add( obj );
             return;
         }
-        throw new IllegalArgumentException("Message arguments must be of type String, Date or Number.");
+        throw new IllegalArgumentException( "Message arguments must be of type String, Date or Number." );
     }
 
     /**
@@ -342,7 +346,7 @@ public class Workflow
      */
     public final synchronized Principal getCurrentActor()
     {
-        if (m_currentStep == null)
+        if ( m_currentStep == null )
         {
             return null;
         }
@@ -379,13 +383,13 @@ public class Workflow
      *            the name of the attribute
      * @return the value
      */
-    public final synchronized Object getAttribute(String attr)
+    public final synchronized Object getAttribute( String attr )
     {
-        if (m_attributes == null)
+        if ( m_attributes == null )
         {
             return null;
         }
-        return m_attributes.get(attr);
+        return m_attributes.get( attr );
     }
 
     /**
@@ -398,10 +402,10 @@ public class Workflow
      */
     public final Date getEndTime()
     {
-        if (isCompleted())
+        if ( isCompleted() )
         {
-            Step last = (Step) m_history.getLast();
-            if (last != null)
+            Step last = m_history.getLast();
+            if ( last != null )
             {
                 return last.getEndTime();
             }
@@ -434,19 +438,19 @@ public class Workflow
      * </ul>
      * <p>
      * Workflow and Step subclasses are free to append items to this collection
-     * with {@link #addMessageArgument(Object)}.
+     * with {@link #addMessageArgument(Serializable)}.
      * </p>
      *
      * @return the array of message arguments
      */
-    public final Object[] getMessageArguments()
+    public final Serializable[] getMessageArguments()
     {
-        List args = new ArrayList();
-        args.add(m_owner.getName());
+        List<Serializable> args = new ArrayList<Serializable>();
+        args.add( m_owner.getName() );
         Principal actor = getCurrentActor();
-        args.add(actor == null ? "-" : actor.getName());
-        args.addAll(m_messageArgs);
-        return args.toArray(new Object[args.size()]);
+        args.add( actor == null ? "-" : actor.getName() );
+        args.addAll( m_messageArgs );
+        return args.toArray( new Serializable[args.size()] );
     }
 
     /**
@@ -505,7 +509,7 @@ public class Workflow
      */
     public final List getHistory()
     {
-        return Collections.unmodifiableList(m_history);
+        return Collections.unmodifiableList( m_history );
     }
 
     /**
@@ -554,7 +558,7 @@ public class Workflow
      */
     public final Step getPreviousStep()
     {
-        return previousStep(m_currentStep);
+        return previousStep( m_currentStep );
     }
 
     /**
@@ -568,12 +572,12 @@ public class Workflow
      */
     public final synchronized void restart() throws WikiException
     {
-        if (m_state != WAITING)
+        if ( m_state != WAITING )
         {
-            throw new IllegalStateException("Workflow is not paused; cannot restart.");
+            throw new IllegalStateException( "Workflow is not paused; cannot restart." );
         }
         m_state = RUNNING;
-        fireEvent(WorkflowEvent.RUNNING);
+        fireEvent( WorkflowEvent.RUNNING );
 
         // Process current step
         try
@@ -588,23 +592,23 @@ public class Workflow
     }
 
     /**
-     * Temporarily associates an Object with this Workflow, as a named attribute, for the
-     * duration of workflow execution. The passed Object can be anything required by
-     * an executing Step. Note that when the workflow completes or aborts, all
-     * attributes will be cleared.
+     * Temporarily associates an object with this Workflow, as a named attribute, for the
+     * duration of workflow execution. The passed object can be anything required by
+     * an executing Step, although it <em>should</em> be serializable. Note that when the workflow
+     * completes or aborts, all attributes will be cleared.
      *
      * @param attr
      *            the attribute name
      * @param obj
      *            the value
      */
-    public final synchronized void setAttribute(String attr, Object obj)
+    public final synchronized void setAttribute(String attr, Object obj )
     {
-        if (m_attributes == null)
+        if ( m_attributes == null )
         {
-            m_attributes = new HashMap();
+            m_attributes = new HashMap<String, Object>();
         }
-        m_attributes.put(attr, obj);
+        m_attributes.put( attr, obj );
     }
 
     /**
@@ -627,7 +631,7 @@ public class Workflow
      * @param id
      *            the unique identifier
      */
-    public final synchronized void setId(int id)
+    public final synchronized void setId( int id )
     {
         this.m_id = id;
     }
@@ -638,10 +642,10 @@ public class Workflow
      * @param manager
      *            the workflow manager
      */
-    public final synchronized void setWorkflowManager(WorkflowManager manager)
+    public final synchronized void setWorkflowManager( WorkflowManager manager )
     {
         m_manager = manager;
-        addWikiEventListener(manager);
+        addWikiEventListener( manager );
     }
 
     /**
@@ -655,21 +659,21 @@ public class Workflow
      */
     public final synchronized void start() throws WikiException
     {
-        if (m_state == ABORTED)
+        if ( m_state == ABORTED )
         {
-            throw new IllegalStateException("Workflow cannot be started; it has already been aborted.");
+            throw new IllegalStateException( "Workflow cannot be started; it has already been aborted." );
         }
-        if (m_started)
+        if ( m_started )
         {
-            throw new IllegalStateException("Workflow has already started.");
+            throw new IllegalStateException( "Workflow has already started." );
         }
         m_started = true;
         m_state = RUNNING;
-        fireEvent(WorkflowEvent.RUNNING);
+        fireEvent( WorkflowEvent.RUNNING );
 
         // Mark the first step as the current one & add to history
         m_currentStep = m_firstStep;
-        m_history.add(m_currentStep);
+        m_history.add( m_currentStep );
 
         // Process current step
         try
@@ -691,12 +695,12 @@ public class Workflow
      */
     public final synchronized void waitstate()
     {
-        if (m_state != RUNNING)
+        if ( m_state != RUNNING )
         {
-            throw new IllegalStateException("Workflow is not running; cannot pause.");
+            throw new IllegalStateException( "Workflow is not running; cannot pause." );
         }
         m_state = WAITING;
-        fireEvent(WorkflowEvent.WAITING);
+        fireEvent( WorkflowEvent.WAITING );
     }
 
     /**
@@ -720,7 +724,7 @@ public class Workflow
         if ( !isCompleted() )
         {
             m_state = COMPLETED;
-            fireEvent(WorkflowEvent.COMPLETED);
+            fireEvent( WorkflowEvent.COMPLETED );
             cleanup();
         }
     }
@@ -735,8 +739,8 @@ public class Workflow
      */
     protected final Step previousStep(Step step)
     {
-        int index = m_history.indexOf(step);
-        return index < 1 ? null : (Step) m_history.get(index - 1);
+        int index = m_history.indexOf( step );
+        return index < 1 ? null : m_history.get( index - 1 );
     }
 
     /**
@@ -749,29 +753,29 @@ public class Workflow
      */
     protected final void processCurrentStep() throws WikiException
     {
-        while (m_currentStep != null)
+        while ( m_currentStep != null )
         {
 
             // Start and execute the current step
-            if (!m_currentStep.isStarted())
+            if ( !m_currentStep.isStarted() )
             {
                 m_currentStep.start();
             }
             try
             {
                 Outcome result = m_currentStep.execute();
-                if (Outcome.STEP_ABORT.equals(result))
+                if ( Outcome.STEP_ABORT.equals( result ) )
                 {
                     abort();
                     break;
                 }
 
-                if (!m_currentStep.isCompleted())
+                if ( !m_currentStep.isCompleted() )
                 {
-                    m_currentStep.setOutcome(result);
+                    m_currentStep.setOutcome( result );
                 }
             }
-            catch (WikiException e)
+            catch ( WikiException e )
             {
                 throw e;
             }
@@ -779,22 +783,22 @@ public class Workflow
             // Get the execution Outcome; if not complete, pause workflow and
             // exit
             Outcome outcome = m_currentStep.getOutcome();
-            if (!outcome.isCompletion())
+            if ( !outcome.isCompletion() )
             {
                 waitstate();
                 break;
             }
 
             // Get the next Step; if null, we're done
-            Step nextStep = m_currentStep.getSuccessor(outcome);
-            if (nextStep == null)
+            Step nextStep = m_currentStep.getSuccessor( outcome );
+            if ( nextStep == null )
             {
                 complete();
                 break;
             }
 
             // Add the next step to Workflow history, and mark as current
-            m_history.add(nextStep);
+            m_history.add( nextStep );
             m_currentStep = nextStep;
         }
 
@@ -809,9 +813,9 @@ public class Workflow
      * @param listener
      *            the event listener
      */
-    public final synchronized void addWikiEventListener(WikiEventListener listener)
+    public final synchronized void addWikiEventListener( WikiEventListener listener )
     {
-        WikiEventManager.addWikiEventListener(this, listener);
+        WikiEventManager.addWikiEventListener( this, listener );
     }
 
     /**
@@ -821,9 +825,9 @@ public class Workflow
      * @param listener
      *            the event listener
      */
-    public final synchronized void removeWikiEventListener(WikiEventListener listener)
+    public final synchronized void removeWikiEventListener( WikiEventListener listener )
     {
-        WikiEventManager.removeWikiEventListener(this, listener);
+        WikiEventManager.removeWikiEventListener( this, listener );
     }
 
     /**
@@ -833,11 +837,11 @@ public class Workflow
      * @param type
      *            the event type to be fired
      */
-    protected final void fireEvent(int type)
+    protected final void fireEvent( int type )
     {
-        if (WikiEventManager.isListening(this))
+        if ( WikiEventManager.isListening( this ) )
         {
-            WikiEventManager.fireEvent(this, new WorkflowEvent(this, type));
+            WikiEventManager.fireEvent( this, new WorkflowEvent( this, type ) );
         }
     }
 
