@@ -1,18 +1,23 @@
 
 package com.ecyrd.jspwiki;
 
-import junit.framework.*;
-import java.io.*;
-import java.util.*;
+import java.util.Properties;
 
-import org.apache.log4j.*;
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
 
-import com.ecyrd.jspwiki.providers.*;
+import org.apache.log4j.PropertyConfigurator;
+
+import com.ecyrd.jspwiki.PageManager;
+import com.ecyrd.jspwiki.providers.CachingProvider;
 
 public class PageManagerTest extends TestCase
 {
     Properties props = new Properties();
 
+    TestEngine engine;
+
     public PageManagerTest( String s )
     {
         super( s );
@@ -23,6 +28,7 @@ public class PageManagerTest extends TestCase
     {
         props.load( TestEngine.findTestProperties() );
         PropertyConfigurator.configure(props);
+        engine = new TestEngine(props);
     }
 
     public void tearDown()
@@ -33,7 +39,7 @@ public class PageManagerTest extends TestCase
         throws Exception
     {
         props.setProperty( "jspwiki.usePageCache", "true" );
-        PageManager m = new PageManager( props );
+        PageManager m = new PageManager( engine, props );
 
         assertTrue( m.getProvider() instanceof CachingProvider );
     }
@@ -42,7 +48,7 @@ public class PageManagerTest extends TestCase
         throws Exception
     {
         props.setProperty( "jspwiki.usePageCache", "false" );
-        PageManager m = new PageManager( props );
+        PageManager m = new PageManager( engine, props );
 
         assertTrue( !(m.getProvider() instanceof CachingProvider) );
     }
