@@ -902,4 +902,86 @@ public class WikiEngineTest extends TestCase
         
         assertEquals( "has wrong referrer", "NewNameTestPage", pages.iterator().next() );        
     }
+    
+    public void testChangeNoteOldVersion2() throws Exception
+    {
+        WikiPage p = new WikiPage( m_engine, NAME1 );
+    
+        WikiContext context = new WikiContext(m_engine,p);
+
+        context.getPage().setAttribute( WikiPage.CHANGENOTE, "Test change" );
+        
+        m_engine.saveText( context, "test" );
+
+        for( int i = 0; i < 5; i++ )
+        {
+            WikiPage p2 = (WikiPage)m_engine.getPage( NAME1 ).clone();
+            p2.removeAttribute(WikiPage.CHANGENOTE);
+
+            context.setPage( p2 );
+
+            m_engine.saveText( context, "test"+i );
+        }
+
+        WikiPage p3 = m_engine.getPage( NAME1, -1 );
+    
+        assertEquals( null, p3.getAttribute(WikiPage.CHANGENOTE) );
+    }
+    
+    public void testCreatePage() throws Exception 
+    {
+        String text = "Foobar.\r\n";
+        String name = "mrmyxpltz";
+        
+        assertEquals( "page should not exist right now",
+                      false,
+                      m_engine.pageExists( name ) );
+
+        m_engine.saveText( name, text );
+
+        assertEquals( "page does not exist",
+                      true,
+                      m_engine.pageExists( name ) );
+    }
+    
+    public void testCreateEmptyPage() throws Exception 
+    {
+        String text = "";
+        String name = "mrmxyzptlk";
+        
+        assertEquals( "page should not exist right now",
+                      false,
+                      m_engine.pageExists( name ) );
+
+        m_engine.saveText( name, text );
+
+        assertEquals( "page should not exist right now neither",
+                      false,
+                      m_engine.pageExists( name ) );
+    }
+    
+    public void testSaveExistingPageWithEmptyContent() throws Exception 
+    {
+        String text = "Foobar.\r\n";
+        String name = NAME1;
+        
+        m_engine.saveText( name, text );
+
+        assertEquals( "page does not exist",
+                      true,
+                      m_engine.pageExists( name ) );
+        
+        // saveText uses normalizePostData to assure it conforms to certain rules
+        assertEquals( "wrong content",
+                      TextUtil.normalizePostData( text ),
+                      m_engine.getText( name ) );
+        
+        m_engine.saveText( name, "" );
+        
+        assertEquals( "wrong content",
+                      TextUtil.normalizePostData( "" ), 
+                      m_engine.getText( name ) );
+        
+    }
+
 }
