@@ -15,14 +15,14 @@ import com.ecyrd.jspwiki.event.WikiSecurityEvent;
 public class SecurityEventTrap implements WikiEventListener
 {
     private WikiSecurityEvent m_lastEvent = null;
-    private List              m_events    = new ArrayList();
+    private List<WikiSecurityEvent> m_events    = new ArrayList<WikiSecurityEvent>();
 
     public void actionPerformed( WikiEvent event )
     {
         if ( event instanceof WikiSecurityEvent )
         {
             m_lastEvent = (WikiSecurityEvent)event;
-            m_events.add( event );
+            m_events.add( (WikiSecurityEvent)event );
         }
         else
         {
@@ -42,7 +42,7 @@ public class SecurityEventTrap implements WikiEventListener
     
     public WikiSecurityEvent[] events() 
     {
-        return (WikiSecurityEvent[])m_events.toArray(new WikiSecurityEvent[m_events.size()]);
+        return m_events.toArray(new WikiSecurityEvent[m_events.size()]);
     }
 
 }
