@@ -1,3 +1,23 @@
+/* 
+    JSPWiki - a JSP-based WikiWiki clone.
+
+    Licensed to the Apache Software Foundation (ASF) under one
+    or more contributor license agreements.  See the NOTICE file
+    distributed with this work for additional information
+    regarding copyright ownership.  The ASF licenses this file
+    to you under the Apache License, Version 2.0 (the
+    "License"); you may not use this file except in compliance
+    with the License.  You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing,
+    software distributed under the License is distributed on an
+    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+    KIND, either express or implied.  See the License for the
+    specific language governing permissions and limitations
+    under the License.  
+ */
 package com.ecyrd.jspwiki.parser;
 
 import java.io.FileInputStream;
@@ -5,7 +25,9 @@ import java.text.SimpleDateFormat;
 import java.util.Calendar;
 import java.util.Properties;
 
+import junit.framework.Test;
 import junit.framework.TestCase;
+import junit.framework.TestSuite;
 
 public class CreoleToJSPWikiTranslatorTest extends TestCase
 {
@@ -478,10 +500,10 @@ public class CreoleToJSPWikiTranslatorTest extends TestCase
             "//Comment\r\n" +
             "}]\r\n" +
             "\r\n" +
-            "[http://www.xyz.com/]";
+            "[http://www.xyz.com/]\r\n";
         
-        System.out.println(src);
-        System.out.println(translate(src));
+        //System.out.println(src);
+        //System.out.println(translate(src));
         
         assertEquals(target, translate(src));
     }
@@ -818,4 +840,8 @@ public class CreoleToJSPWikiTranslatorTest extends TestCase
         return translator.translate(new Properties(), src);
     }
 
+    public static Test suite()
+    {
+        return new TestSuite( CreoleToJSPWikiTranslatorTest.class );
+    }
 }
