@@ -24,7 +24,7 @@ import com.ecyrd.jspwiki.render.XHTMLRenderer;
 public class JSPWikiMarkupParserTest extends TestCase
 {
     Properties props = new Properties();
-    Vector     created = new Vector();
+    Vector<String>     created = new Vector<String>();
 
     static final String PAGE_NAME = "testpage";
 
@@ -182,7 +182,7 @@ public class JSPWikiMarkupParserTest extends TestCase
 
         String src = "This should be a [here|HyperLink#heading]";
 
-        assertEquals( "This should be a <a class=\"wikipage\" href=\"/Wiki.jsp?page=HyperLink#section-HyperLink-heading\">here</a>",
+        assertEquals( "This should be a <a class=\"wikipage\" href=\"/Wiki.jsp?page=HyperLink#section-HyperLink-Heading\">here</a>",
                       translate(src) );
     }
 
@@ -193,10 +193,21 @@ public class JSPWikiMarkupParserTest extends TestCase
 
         String src = "This should be a [HyperLink#heading]";
 
-        assertEquals( "This should be a <a class=\"wikipage\" href=\"/Wiki.jsp?page=HyperLink#section-HyperLink-heading\">HyperLink#heading</a>",
+        assertEquals( "This should be a <a class=\"wikipage\" href=\"/Wiki.jsp?page=HyperLink#section-HyperLink-Heading\">HyperLink#heading</a>",
                       translate(src) );
     }
 
+    public void testHyperlinksNamed3()
+    throws Exception
+    {
+        newPage("HyperLink");
+
+        String src = "!Heading Too\r\nThis should be a [HyperLink#heading too]";
+
+        assertEquals( "<h4 id=\"section-testpage-HeadingToo\">Heading Too</h4>\nThis should be a <a class=\"wikipage\" href=\"/Wiki.jsp?page=HyperLink#section-HyperLink-HeadingToo\">HyperLink#heading too</a>",
+                      translate(src) );
+    }
+    
     //
     //  Testing CamelCase hyperlinks
     //
@@ -229,7 +240,7 @@ public class JSPWikiMarkupParserTest extends TestCase
     {
         String src = "This should be a HyperLink.";
 
-        assertEquals( "This should be a <a class=\"createpage\" href=\"/Edit.jsp?page=HyperLink\" title=\"Create 'HyperLink'\">HyperLink</a>.",
+        assertEquals( "This should be a <a class=\"createpage\" href=\"/Edit.jsp?page=HyperLink\" title=\"Create &quot;HyperLink&quot;\">HyperLink</a>.",
                       translate(src) );
     }
 
@@ -397,6 +408,27 @@ public class JSPWikiMarkupParserTest extends TestCase
         assertEquals( "<a class=\"external\" href=\"http://www.foo.bar/ANewHope?q=foobar&amp;gobble=bobble+gnoo\">http://www.foo.bar/ANewHope?q=foobar&amp;gobble=bobble+gnoo</a>",
                       translate(src) );
     }
+
+    public void testHyperlinksCCURLs8()
+    throws Exception
+    {
+        String src = "http://www.foo.bar/~ANewHope/";
+
+        // System.out.println( "EX:"+translate(src) );
+        assertEquals( "<a class=\"external\" href=\"http://www.foo.bar/~ANewHope/\">http://www.foo.bar/~ANewHope/</a>",
+                      translate(src) );
+    }
+    
+    public void testHyperlinksCCURLs9()
+    throws Exception
+    {
+        String src = "http://www.foo.bar/%7EANewHope/";
+
+        // System.out.println( "EX:"+translate(src) );
+        assertEquals( "<a class=\"external\" href=\"http://www.foo.bar/%7EANewHope/\">http://www.foo.bar/%7EANewHope/</a>",
+                      translate(src) );
+    }
+
     public void testHyperlinksCCNegated()
     throws Exception
     {
@@ -559,7 +591,7 @@ public class JSPWikiMarkupParserTest extends TestCase
     {
         String src = "This should be a [HyperLinks]";
 
-        assertEquals( "This should be a <a class=\"createpage\" href=\"/Edit.jsp?page=HyperLinks\" title=\"Create 'HyperLinks'\">HyperLinks</a>",
+        assertEquals( "This should be a <a class=\"createpage\" href=\"/Edit.jsp?page=HyperLinks\" title=\"Create &quot;HyperLinks&quot;\">HyperLinks</a>",
                       translate(src) );
     }
 
@@ -604,6 +636,15 @@ public class JSPWikiMarkupParserTest extends TestCase
                       translate(src) );
     }
 
+    public void testHyperlinksInterWiki2()
+    throws Exception
+    {
+        String src = "This should be a [JSPWiki:HyperLink]";
+
+        assertEquals( "This should be a <a class=\"interwiki\" href=\"http://www.ecyrd.com/JSPWiki/Wiki.jsp?page=HyperLink\">JSPWiki:HyperLink</a>",
+                      translate(src) );
+    }
+
     public void testAttachmentLink()
     throws Exception
     {
@@ -616,7 +657,7 @@ public class JSPWikiMarkupParserTest extends TestCase
         String src = "This should be an [attachment link|Test/TestAtt.txt]";
 
         assertEquals( "This should be an <a class=\"attachment\" href=\"/attach/Test/TestAtt.txt\">attachment link</a>"+
-                      "<a href=\"/PageInfo.jsp?page=Test/TestAtt.txt\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
+                      "<a href=\"/PageInfo.jsp?page=Test/TestAtt.txt\" class=\"infolink\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
                       translate(src));
     }
 
@@ -639,7 +680,7 @@ public class JSPWikiMarkupParserTest extends TestCase
         String src = "This should be an [attachment link|Test/TestAtt.txt]";
 
         assertEquals( "This should be an <a class=\"attachment\" href=\"/attach/Test/TestAtt.txt\">attachment link</a>"+
-                      "<a href=\"/PageInfo.jsp?page=Test/TestAtt.txt\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
+                      "<a href=\"/PageInfo.jsp?page=Test/TestAtt.txt\" class=\"infolink\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
                       translate(testEngine2,src));
     }
 
@@ -662,7 +703,7 @@ public class JSPWikiMarkupParserTest extends TestCase
         String src = "[Test page/TestAtt.txt]";
 
         assertEquals( "<a class=\"attachment\" href=\"/attach/TestPage/TestAtt.txt\">Test page/TestAtt.txt</a>"+
-                      "<a href=\"/PageInfo.jsp?page=TestPage/TestAtt.txt\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
+                      "<a href=\"/PageInfo.jsp?page=TestPage/TestAtt.txt\" class=\"infolink\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
                       translate(testEngine2,src));
     }
 
@@ -682,7 +723,7 @@ public class JSPWikiMarkupParserTest extends TestCase
         String src = "["+testEngine2.beautifyTitle("TestPage/TestAtt.txt")+"]";
 
         assertEquals( "<a class=\"attachment\" href=\"/attach/TestPage/TestAtt.txt\">Test Page/TestAtt.txt</a>"+
-                      "<a href=\"/PageInfo.jsp?page=TestPage/TestAtt.txt\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
+                      "<a href=\"/PageInfo.jsp?page=TestPage/TestAtt.txt\" class=\"infolink\"><img src=\"/images/attachment_small.png\" border=\"0\" alt=\"(info)\" /></a>",
                       translate(testEngine2,src));
     }
 
@@ -1314,6 +1355,190 @@ public class JSPWikiMarkupParserTest extends TestCase
                       result );
     }
 
+    /**
+     * <pre>
+     *   * bullet A
+     *   ** bullet A_1
+     *   *# number A_1
+     *   * bullet B
+     * </pre>
+     * 
+     * would come out as:
+     * 
+     * <ul>
+     *   <li>bullet A
+     *     <ul>
+     *       <li>bullet A_1</li>
+     *     </ul>
+     *     <ol>
+     *       <li>number A_1</li>
+     *     </ol>
+     *   </li>
+     *   <li>bullet B</li>
+     * </ul>
+     *  
+     */
+
+    public void testMixedListOnSameLevel()
+    throws Exception
+    {
+        String src="* bullet A\n** bullet A_1\n*# number A_1\n* bullet B\n";
+
+        String result = translate(src);
+
+        // Remove newlines for easier parsing.
+        result = TextUtil.replaceString( result, "\n", "" );
+
+        assertEquals( "<ul>"+
+                      "<li>bullet A"+
+                      "<ul>"+
+                      "<li>bullet A_1</li>"+
+                      "</ul>"+
+                      "<ol>"+
+                      "<li>number A_1</li>"+
+                      "</ol>"+
+                      "</li>"+
+                      "<li>bullet B</li>"+
+                      "</ul>"
+                      ,
+                      result );
+    }
+    /**
+     * <pre>
+     *   * bullet A
+     *   ** bullet A_1
+     *   ## number A_1
+     *   * bullet B
+     * </pre>
+     * 
+     * would come out as:
+     * 
+     * <ul>
+     *   <li>bullet A
+     *     <ul>
+     *       <li>bullet A_1</li>
+     *     </ul>
+     *     <ol>
+     *       <li>number A_1</li>
+     *     </ol>
+     *   </li>
+     *   <li>bullet B</li>
+     * </ul>
+     *  
+     */
+
+    public void testMixedListOnSameLevel2()
+    throws Exception
+    {
+        String src="* bullet A\n** bullet A_1\n## number A_1\n* bullet B\n";
+
+        String result = translate(src);
+
+        // Remove newlines for easier parsing.
+        result = TextUtil.replaceString( result, "\n", "" );
+
+        assertEquals( "<ul>"+
+                      "<li>bullet A"+
+                      "<ul>"+
+                      "<li>bullet A_1</li>"+
+                      "</ul>"+
+                      "<ol>"+
+                      "<li>number A_1</li>"+
+                      "</ol>"+
+                      "</li>"+
+                      "<li>bullet B</li>"+
+                      "</ul>"
+                      ,
+                      result );
+    }
+
+    /**
+     * <pre>
+     *   * bullet 1
+     *   ## number 2
+     *   ** bullet 3
+     *   ## number 4
+     *   * bullet 5 
+     * </pre>
+     * 
+     * would come out as:
+     * 
+     *   <ul>
+     *       <li>bullet 1
+     *           <ol><li>number 2</li></ol>
+     *           <ul><li>bullet 3</li></ul>
+     *           <ol><li>number 4</li></ol>
+     *       </li>
+     *       <li>bullet 5</li>
+     *   </ul>
+     *  
+     */
+
+    public void testMixedListOnSameLevel3()
+    throws Exception
+    {
+        String src="* bullet 1\n## number 2\n** bullet 3\n## number 4\n* bullet 5\n";
+
+        String result = translate(src);
+
+        // Remove newlines for easier parsing.
+        result = TextUtil.replaceString( result, "\n", "" );
+
+        assertEquals( "<ul>"+
+                      "<li>bullet 1"+
+                      "<ol><li>number 2</li></ol>"+
+                      "<ul><li>bullet 3</li></ul>"+
+                      "<ol><li>number 4</li></ol>"+
+                      "</li>"+
+                      "<li>bullet 5</li>"+
+                      "</ul>"
+                      ,
+                      result );
+    }
+    /**
+     * <pre>
+     *   # number 1
+     *   ** bullet 2
+     *   ## number 3
+     *   ** bullet 4
+     *   # number 5 
+     * </pre>
+     * 
+     * would come out as:
+     * 
+     *   <ol>
+     *       <li>number 1
+     *           <ul><li>bullet 2</li></ul>
+     *           <ol><li>number 3</li></ol>
+     *           <ul><li>bullet 4</li></ul>
+     *       </li>
+     *       <li>number 5</li>
+     *   </ol>
+     *  
+     */
+
+    public void testMixedListOnSameLevel4()
+    throws Exception
+    {
+        String src="# number 1\n** bullet 2\n## number 3\n** bullet 4\n# number 5\n";
+
+        String result = translate(src);
+
+        // Remove newlines for easier parsing.
+        result = TextUtil.replaceString( result, "\n", "" );
+
+        assertEquals( "<ol>"+
+                      "<li>number 1"+
+                      "<ul><li>bullet 2</li></ul>"+
+                      "<ol><li>number 3</li></ol>"+
+                      "<ul><li>bullet 4</li></ul>"+
+                      "</li>"+
+                      "<li>number 5</li>"+
+                      "</ol>"
+                      ,
+                      result );
+    }
+
     public void testNestedList()
     throws Exception
     {
@@ -1819,7 +2044,7 @@ public class JSPWikiMarkupParserTest extends TestCase
     {
         String src="!!![Hello]";
 
-        assertEquals( "<h2 id=\"section-testpage-Hello\"><a class=\"createpage\" href=\"/Edit.jsp?page=Hello\" title=\"Create 'Hello'\">Hello</a></h2>",
+        assertEquals( "<h2 id=\"section-testpage-Hello\"><a class=\"createpage\" href=\"/Edit.jsp?page=Hello\" title=\"Create &quot;Hello&quot;\">Hello</a></h2>",
                       translate(src) );
     }
 
@@ -2122,6 +2347,7 @@ public class JSPWikiMarkupParserTest extends TestCase
 
         try
         {
+            testEngine.saveText( PAGE_NAME, "content" );
             Attachment att = new Attachment( testEngine, PAGE_NAME, "TestAtt.txt" );
             att.setAuthor( "FirstPost" );
             testEngine.getAttachmentManager().storeAttachment( att, testEngine.makeAttachmentFile() );
@@ -2311,7 +2537,7 @@ public class JSPWikiMarkupParserTest extends TestCase
     {
         String src = "[-phobous]";
 
-        assertEquals( "<a class=\"createpage\" href=\"/Edit.jsp?page=-phobous\" title=\"Create '-phobous'\">-phobous</a>", translate(src) );
+        assertEquals( "<a class=\"createpage\" href=\"/Edit.jsp?page=-phobous\" title=\"Create &quot;-phobous&quot;\">-phobous</a>", translate(src) );
     }
 
     public void testPunctuatedWikiNames2()
@@ -2319,7 +2545,7 @@ public class JSPWikiMarkupParserTest extends TestCase
     {
         String src = "[?phobous]";
 
-        assertEquals( "<a class=\"createpage\" href=\"/Edit.jsp?page=Phobous\" title=\"Create 'Phobous'\">?phobous</a>", translate(src) );
+        assertEquals( "<a class=\"createpage\" href=\"/Edit.jsp?page=Phobous\" title=\"Create &quot;Phobous&quot;\">?phobous</a>", translate(src) );
     }
 
     public void testPunctuatedWikiNames3()
@@ -2327,7 +2553,7 @@ public class JSPWikiMarkupParserTest extends TestCase
     {
         String src = "[Brightness (apical)]";
 
-        assertEquals( "<a class=\"createpage\" href=\"/Edit.jsp?page=Brightness%20%28apical%29\" title=\"Create 'Brightness (apical)'\">Brightness (apical)</a>", translate(src) );
+        assertEquals( "<a class=\"createpage\" href=\"/Edit.jsp?page=Brightness%20%28apical%29\" title=\"Create &quot;Brightness (apical)&quot;\">Brightness (apical)</a>", translate(src) );
     }
 
     public void testDeadlySpammer()
@@ -2367,6 +2593,15 @@ public class JSPWikiMarkupParserTest extends TestCase
 
         assertEquals( "<a class=\"wikipage\" href=\"/Wiki.jsp?page=Foo%20bar\">Foo        bar</a>", translate(src) );
     }
+    
+    public void testIllegalXML() throws Exception
+    {
+        String src = "Test \u001d foo";
+        
+        String dst = translate(src);
+        
+        assertTrue( "No error", dst.indexOf("JDOM") != -1 );
+    }
 
     public void testXSS1() throws Exception
     {
@@ -2376,7 +2611,27 @@ public class JSPWikiMarkupParserTest extends TestCase
 
         assertEquals( "<a class=\"external\" href=\"http://www.host.com/du=&quot;&gt; &lt;img src=&quot;foobar&quot; onerror=&quot;alert(document.cookie)&quot;/&gt;\">http://www.host.com/du=&quot;&gt; &lt;img src=&quot;foobar&quot; onerror=&quot;alert(document.cookie)&quot;/&gt;</a>", dst );
     }
+    
+    public void testAmpersand1() throws Exception
+    {
+        newPage( "Foo&Bar" );
+        String src = "[Foo&Bar]";
 
+        String dst = translate(src);
+
+        assertEquals( "<a class=\"wikipage\" href=\"/Wiki.jsp?page=Foo%26Bar\">Foo&amp;Bar</a>", dst );
+    }
+
+    public void testAmpersand2() throws Exception
+    {
+        newPage( "Foo & Bar" );
+        String src = "[Foo & Bar]";
+
+        String dst = translate(src);
+
+        assertEquals( "<a class=\"wikipage\" href=\"/Wiki.jsp?page=Foo%20%26%20Bar\">Foo &amp; Bar</a>", dst );
+    }
+    
     // This is a random find: the following page text caused an eternal loop in V2.0.x.
     private static final String brokenPageText =
         "Please ''check [RecentChanges].\n" +
