@@ -1,7 +1,11 @@
 
 package com.ecyrd.jspwiki.plugin;
 
-import junit.framework.*;
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
+
+
 
 public class AllTests extends TestCase
 {
@@ -14,13 +18,14 @@ public class AllTests extends TestCase
     {
         TestSuite suite = new TestSuite("JSPWiki plugins");
 
+        suite.addTest( CounterPluginTest.suite() );
+        suite.addTest( GroupsTest.suite() );
+        suite.addTest( InsertPageTest.suite() );
         suite.addTest( PluginManagerTest.suite() );
         suite.addTest( ReferringPagesPluginTest.suite() );
-        suite.addTest( CounterPluginTest.suite() );
-        suite.addTest( UndefinedPagesPluginTest.suite() );
         suite.addTest( TableOfContentsTest.suite() );
-        suite.addTest( InsertPageTest.suite() );
-        
+        suite.addTest( UndefinedPagesPluginTest.suite() );
+
         return suite;
     }
 }
