@@ -12,12 +12,13 @@ public class AllTests extends TestCase
 
     public static Test suite()
     {
-        TestSuite suite = new TestSuite();
+        TestSuite suite = new TestSuite("Providers suite");
 
+        suite.addTest( BasicAttachmentProviderTest.suite() );
+        suite.addTest( CachingProviderTest.suite() );
         suite.addTest( FileSystemProviderTest.suite() );
         suite.addTest( RCSFileProviderTest.suite() );
         suite.addTest( VersioningFileProviderTest.suite() );
-        suite.addTest( BasicAttachmentProviderTest.suite() );
 
         return suite;
     }
