@@ -17,9 +17,12 @@ public class AllTests extends TestCase
 
         suite.addTest( ClassUtilTest.suite() );
         suite.addTest( CommentedPropertiesTest.suite() );
+        suite.addTest( CryptoUtilTest.suite() );
         suite.addTest( MailUtilTest.suite() );
         suite.addTest( PriorityListTest.suite() );
+        suite.addTest( SerializerTest.suite() );
         suite.addTest( TextUtilTest.suite() );
+        suite.addTest( TimedCounterListTest.suite() );
         
         return suite;
     }
