@@ -12,7 +12,7 @@ public class AllTests extends TestCase
 
     public static Test suite()
     {
-        TestSuite suite = new TestSuite();
+        TestSuite suite = new TestSuite("XMLRPC tests");
 
         suite.addTest( RPCHandlerTest.suite() );
 
